@@ -3,6 +3,7 @@
 set -e
 
 bundle="$1"
+searchdir="$2"
 
 if [ ! -d "$bundle" ]; then
   echo "$bundle is not an application bundle"
@@ -14,16 +15,42 @@ frameworks="${bundle}/Contents/Frameworks"
 mkdir -p "$frameworks"
 rm -f ${frameworks}/*.processed
 
+absreadlink()
+{
+  if [ -L "$1" ]; then
+    local target=`readlink "$1"`
+    if [ "${target:0:1}" != "/" ]; then
+      local dir=`dirname "$1"`
+      target="$dir/$target"
+    fi
+    if [ -L "$target" ]; then
+      absreadlink "$target"
+    else
+      echo "$target"
+    fi
+  else
+    echo "$1"
+  fi
+}
 
 process_dylib()
 {
   local file="$1"
   local dylib="$2"
-  local name="${dylib##*/}"
+
+  local resolved=`absreadlink "$dylib"`
+  local name="${resolved##*/}"
 
   if [ ! -f "${frameworks}/$name" ] && [ ! -f "${frameworks}/$name.processed" ]; then
     touch "${frameworks}/$name.processed"
-    if [ -f "$dylib" ]; then
+
+    if [ -f "$searchdir/.libs/$name" ]; then
+      echo "Found dependency $name"
+      cp "$searchdir/.libs/$name" "${frameworks}/$name"
+    elif [ -f "$searchdir/$name" ]; then
+      echo "Found dependency $name"
+      cp "$searchdir/$name" "${frameworks}/$name"
+    elif [ -f "$dylib" ]; then
       echo "Found dependency $name"
       cp "$dylib" "${frameworks}/$name"
     else
@@ -52,7 +79,8 @@ process_dylibs()
 process_file()
 {
   local file="$1"
-  process_dylibs "$file" `otool -L "$file" | grep 'dylib' | sed 's/^[[:blank:]]*//' | sed 's/ .*//' | grep -v '^/usr/\|^/System/' | grep -v ':$'`
+  process_dylibs "$file" `otool -L "$file" | grep "dylib\\|\\.so" | sed 's/^[[:blank:]]*//' | sed 's/ .*//' | grep -v '^/usr/\|^/System/' | grep -v ':$'`
+  process_dylibs "$file" `otool -L "$file" | grep "dylib\\|\\.so" | sed 's/^[[:blank:]]*//' | sed 's/ .*//' | grep '^/usr/local/' | grep -v ':$'`
 }
 
 for file in "${bundle}/Contents/MacOS/"*; do
