@@ -1,13 +1,15 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "controlsocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
-#include "local_path.h"
 #include "lookup.h"
 #include "logging_private.h"
 #include "proxy.h"
 #include "servercapabilities.h"
-#include "sizeformatting_base.h"
+
+#include "../include/local_path.h"
+#include "../include/engine_options.h"
+#include "../include/sizeformatting_base.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/iputils.hpp>
@@ -59,7 +61,7 @@ Command CControlSocket::GetCurrentCommandId() const
 		return operations_.back()->opId;
 	}
 
-	return engine_.GetCurrentCommandId();
+	return Command::none;
 }
 
 void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData)
@@ -137,18 +139,19 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		nErrorCode = oldOperation->Reset(nErrorCode);
 	}
 	if (!operations_.empty()) {
-		int ret;
 		if (nErrorCode == FZ_REPLY_OK ||
 			nErrorCode == FZ_REPLY_ERROR ||
 			nErrorCode == FZ_REPLY_CRITICALERROR ||
 			nErrorCode == FZ_REPLY_ERROR_NOTFOUND)
 		{
-			ret = ParseSubcommandResult(nErrorCode, *oldOperation);
+			if (!oldOperation->topLevelOperation_) {
+				return ParseSubcommandResult(nErrorCode, std::move(oldOperation));
+			}
 		}
 		else {
-			ret = ResetOperation(nErrorCode);
+			oldOperation.reset();
+			return ResetOperation(nErrorCode);
 		}
-		return ret;
 	}
 
 	std::wstring prefix;
@@ -194,7 +197,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		case Command::transfer:
 			{
 				auto & data = static_cast<CFileTransferOpData &>(*oldOperation);
-				if (!data.download_ && data.transferInitiated_) {
+				if (!data.download() && data.transferInitiated_) {
 					if (!currentServer_) {
 						log(logmsg::debug_warning, L"currentServer_ is empty");
 					}
@@ -211,18 +214,24 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			}
 			break;
 		}
+
+		oldOperation.reset();
 	}
 
 	engine_.transfer_status_.Reset();
 
-	SetWait(false);
-
 	if (m_invalidateCurrentPath) {
 		currentPath_.clear();
 		m_invalidateCurrentPath = false;
 	}
 
-	return engine_.ResetOperation(nErrorCode);
+	if (operations_.empty()) {
+		SetWait(false);
+		return engine_.ResetOperation(nErrorCode);
+	}
+	else {
+		return SendNextCommand();
+	}
 }
 
 void CControlSocket::UpdateCache(COpData const &, CServerPath const& serverPath, std::wstring const& remoteFile, int64_t fileSize)
@@ -236,18 +245,8 @@ void CControlSocket::UpdateCache(COpData const &, CServerPath const& serverPath,
 int CControlSocket::DoClose(int nErrorCode)
 {
 	log(logmsg::debug_debug, L"CControlSocket::DoClose(%d)", nErrorCode);
-	if (m_closed) {
-		assert(operations_.empty());
-		return nErrorCode;
-	}
-
-	m_closed = true;
-
-	nErrorCode = ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | nErrorCode);
-
-	currentServer_.clear();
-
-	return nErrorCode;
+	currentPath_.clear();
+	return ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | nErrorCode);
 }
 
 std::wstring CControlSocket::ConvertDomainName(std::wstring const& domain)
@@ -304,39 +303,37 @@ CServer const& CControlSocket::GetCurrentServer() const
 	return currentServer_;
 }
 
-bool CControlSocket::ParsePwdReply(std::wstring reply, bool unquoted, CServerPath const& defaultPath)
+bool CControlSocket::ParsePwdReply(std::wstring reply, CServerPath const& defaultPath)
 {
-	if (!unquoted) {
-		size_t pos1 = reply.find('"');
-		size_t pos2 = reply.rfind('"');
-		// Due to searching the same character, pos1 is npos iff pos2 is npos
+	size_t pos1 = reply.find('"');
+	size_t pos2 = reply.rfind('"');
+	// Due to searching the same character, pos1 is npos iff pos2 is npos
 
-		if (pos1 == std::wstring::npos || pos1 >= pos2) {
-			pos1 = reply.find('\'');
-			pos2 = reply.rfind('\'');
+	if (pos1 == std::wstring::npos || pos1 >= pos2) {
+		pos1 = reply.find('\'');
+		pos2 = reply.rfind('\'');
 
-			if (pos1 != std::wstring::npos && pos1 < pos2) {
-				log(logmsg::debug_info, L"Broken server sending single-quoted path instead of double-quoted path.");
-			}
+		if (pos1 != std::wstring::npos && pos1 < pos2) {
+			log(logmsg::debug_info, L"Broken server sending single-quoted path instead of double-quoted path.");
 		}
-		if (pos1 == std::wstring::npos || pos1 >= pos2) {
-			log(logmsg::debug_info, L"Broken server, no quoted path found in pwd reply, trying first token as path");
-			pos1 = reply.find(' ');
-			if (pos1 != std::wstring::npos) {
-				reply = reply.substr(pos1 + 1);
-				pos2 = reply.find(' ');
-				if (pos2 != std::wstring::npos)
-					reply = reply.substr(0, pos2);
-			}
-			else {
-				reply.clear();
-			}
+	}
+	if (pos1 == std::wstring::npos || pos1 >= pos2) {
+		log(logmsg::debug_info, L"Broken server, no quoted path found in pwd reply, trying first token as path");
+		pos1 = reply.find(' ');
+		if (pos1 != std::wstring::npos) {
+			reply = reply.substr(pos1 + 1);
+			pos2 = reply.find(' ');
+			if (pos2 != std::wstring::npos)
+				reply = reply.substr(0, pos2);
 		}
 		else {
-			reply = reply.substr(pos1 + 1, pos2 - pos1 - 1);
-			fz::replace_substrings(reply, L"\"\"", L"\"");
+			reply.clear();
 		}
 	}
+	else {
+		reply = reply.substr(pos1 + 1, pos2 - pos1 - 1);
+		fz::replace_substrings(reply, L"\"\"", L"\"");
+	}
 
 	currentPath_.SetType(currentServer_.GetType());
 	if (reply.empty() || !currentPath_.SetPath(reply)) {
@@ -367,13 +364,15 @@ int CControlSocket::CheckOverwriteFile()
 	}
 
 	auto & data = static_cast<CFileTransferOpData &>(*operations_.back());
+	data.localFileSize_ = data.download() ? data.writer_factory_.size() : data.reader_factory_.size();
+	data.localFileTime_ = data.download() ? data.writer_factory_.mtime() : data.reader_factory_.mtime();
 
-	if (data.download_) {
-		if (fz::local_filesys::get_file_type(fz::to_native(data.localFile_), true) != fz::local_filesys::file) {
+	if (data.download()) {
+		if (data.localFileSize_ == aio_base::nosize && data.localFileTime_.empty()) {
 			return FZ_REPLY_OK;
 		}
 	}
-
+	
 	CDirentry entry;
 	bool dirDidExist;
 	bool matchedCase;
@@ -387,56 +386,75 @@ int CControlSocket::CheckOverwriteFile()
 	bool found = engine_.GetDirectoryCache().LookupFile(entry, currentServer_, remotePath, data.remoteFile_, dirDidExist, matchedCase);
 
 	// Ignore entries with wrong case
-	if (found && !matchedCase)
+	if (found && !matchedCase) {
 		found = false;
+	}
 
-	if (!data.download_) {
-		if (!found && data.remoteFileSize_ < 0 && data.fileTime_.empty()) {
+	if (found) {
+		if (data.remoteFileTime_.empty() && entry.has_date()) {
+			data.remoteFileTime_ = entry.time;
+		}
+	}
+
+	if (!data.download()) {
+		if (!found && data.remoteFileSize_ < 0 && data.remoteFileTime_.empty()) {
 			return FZ_REPLY_OK;
 		}
 	}
 
-	CFileExistsNotification *pNotification = new CFileExistsNotification;
+	
 
-	pNotification->download = data.download_;
-	pNotification->localFile = data.localFile_;
-	pNotification->remoteFile = data.remoteFile_;
-	pNotification->remotePath = data.remotePath_;
-	pNotification->localSize = data.localFileSize_;
-	pNotification->remoteSize = data.remoteFileSize_;
-	pNotification->remoteTime = data.fileTime_;
-	pNotification->ascii = !data.transferSettings_.binary;
+	auto notification = std::make_unique<CFileExistsNotification>();
 
-	if (data.download_ && pNotification->localSize >= 0) {
-		pNotification->canResume = true;
+	notification->download = data.download();
+	notification->localFile = data.localName_;
+	notification->remoteFile = data.remoteFile_;
+	notification->remotePath = data.remotePath_;
+	notification->localSize = static_cast<int64_t>(data.localFileSize_);
+	notification->localTime = data.localFileTime_;
+	notification->remoteSize = data.remoteFileSize_;
+	notification->remoteTime = data.remoteFileTime_;
+
+	if (currentServer_.HasFeature(ProtocolFeature::DataTypeConcept)) {
+		notification->ascii = data.flags_ & ftp_transfer_flags::ascii;
 	}
-	else if (!data.download_ && pNotification->remoteSize >= 0) {
-		pNotification->canResume = true;
+
+	if (data.download() && notification->localSize >= 0) {
+		notification->canResume = true;
+	}
+	else if (!data.download() && notification->remoteSize >= 0) {
+		notification->canResume = true;
 	}
 	else {
-		pNotification->canResume = false;
+		notification->canResume = false;
 	}
 
-	pNotification->localTime = fz::local_filesys::get_modification_time(fz::to_native(data.localFile_));
-
-	if (found) {
-		if (pNotification->remoteTime.empty() && entry.has_date()) {
-			pNotification->remoteTime = entry.time;
-			data.fileTime_ = entry.time;
-		}
-	}
-
-	SendAsyncRequest(pNotification);
+	SendAsyncRequest(std::move(notification));
 
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-CFileTransferOpData::CFileTransferOpData(wchar_t const* name, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
-	: COpData(Command::transfer, name)
-	, localFile_(local_file), remoteFile_(remote_file), remotePath_(remote_path)
-	, download_(is_download)
-	, transferSettings_(settings)
+SleepOpData::SleepOpData(CControlSocket & controlSocket, fz::duration const& delay)
+	: COpData(Command::sleep, L"SleepOpData")
+	, fz::event_handler(controlSocket.event_loop_)
+	, controlSocket_(controlSocket)
 {
+	add_timer(delay, true);
+	controlSocket_.SetWait(false);
+}
+
+void SleepOpData::operator()(fz::event_base const&)
+{
+	controlSocket_.ResetOperation(FZ_REPLY_OK);
+}
+
+CFileTransferOpData::CFileTransferOpData(wchar_t const* name, CFileTransferCommand const& cmd)
+	: COpData(Command::transfer, name)
+	, flags_(cmd.GetFlags())
+	, reader_factory_(cmd.GetReader()), writer_factory_(cmd.GetWriter()), localName_(reader_factory_ ? reader_factory_.name() : writer_factory_.name()), remoteFile_(cmd.GetRemoteFile()), remotePath_(cmd.GetRemotePath())
+{
+	localFileSize_ = download() ? writer_factory_.size() : reader_factory_.size();
+	localFileTime_ = download() ? writer_factory_.mtime() : reader_factory_.mtime();
 }
 
 std::wstring CControlSocket::ConvToLocal(char const* buffer, size_t len)
@@ -505,7 +523,7 @@ void CControlSocket::OnTimer(fz::timer_id)
 {
 	m_timer = 0; // It's a one-shot timer, no need to stop it
 
-	int const timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
+	int const timeout = engine_.GetOptions().get_int(OPTION_TIMEOUT);
 	if (timeout > 0) {
 		fz::duration elapsed = fz::monotonic_clock::now() - m_lastActivity;
 
@@ -538,7 +556,7 @@ void CControlSocket::SetWait(bool wait)
 
 		m_lastActivity = fz::monotonic_clock::now();
 
-		int timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
+		int timeout = engine_.GetOptions().get_int(OPTION_TIMEOUT);
 		if (!timeout) {
 			return;
 		}
@@ -597,17 +615,21 @@ int CControlSocket::SendNextCommand()
 	return FZ_REPLY_OK;
 }
 
-int CControlSocket::ParseSubcommandResult(int prevResult, COpData const& opData)
+int CControlSocket::ParseSubcommandResult(int prevResult, std::unique_ptr<COpData> && previousOperation)
 {
 	if (operations_.empty()) {
 		log(logmsg::debug_warning, L"CControlSocket::ParseSubcommandResult(%d) called without active operation", prevResult);
+		previousOperation.reset();
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	auto & data = *operations_.back();
 	log(logmsg::debug_verbose, L"%s::SubcommandResult(%d) in state %d", data.name_, prevResult, data.opState);
-	int res = data.SubcommandResult(prevResult, opData);
+	int res = data.SubcommandResult(prevResult, *previousOperation);
+
+	previousOperation.reset();
+
 	if (res == FZ_REPLY_WOULDBLOCK) {
 		return FZ_REPLY_WOULDBLOCK;
 	}
@@ -631,13 +653,12 @@ void CControlSocket::OnObtainLock()
 	}
 }
 
-void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
+void CControlSocket::InvalidateCurrentWorkingDir(CServerPath const& path)
 {
-	assert(!path.empty());
-	if (currentPath_.empty()) {
+	if (path.empty() || currentPath_.empty()) {
 		return;
 	}
-
+	
 	if (path.IsParentOf(currentPath_, false, true)) {
 		if (!operations_.empty()) {
 			m_invalidateCurrentPath = true;
@@ -648,29 +669,29 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 	}
 }
 
-fz::duration CControlSocket::GetTimezoneOffset() const
+fz::duration CControlSocket::GetInferredTimezoneOffset() const
 {
 	fz::duration ret;
 	if (currentServer_) {
 		int seconds = 0;
-		if (CServerCapabilities::GetCapability(currentServer_, timezone_offset, &seconds) == yes) {
+		if (CServerCapabilities::GetCapability(currentServer_, inferred_timezone_offset, &seconds) == yes) {
 			ret = fz::duration::from_seconds(seconds);
 		}
 	}
 	return ret;
 }
 
-void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
+void CControlSocket::SendAsyncRequest(std::unique_ptr<CAsyncRequestNotification> && notification)
 {
-	assert(pNotification);
-	assert(!operations_.empty());
-
-	pNotification->requestNumber = engine_.GetNextAsyncRequestNumber();
+	if (!notification || operations_.empty()) {
+		return;
+	}
+	notification->requestNumber = engine_.GetNextAsyncRequestNumber();
 
 	if (!operations_.empty()) {
 		operations_.back()->waitForAsyncRequest = true;
 	}
-	engine_.AddNotification(pNotification);
+	engine_.AddNotification(std::move(notification));
 }
 
 // ------------------
@@ -851,16 +872,16 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 	ratelimit_layer_ = std::make_unique<fz::rate_limited_layer>(this, *socket_, &engine_.GetRateLimiter());
 	active_layer_ = ratelimit_layer_.get();
 
-	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
+	const int proxy_type = engine_.GetOptions().get_int(OPTION_PROXY_TYPE);
 	if (proxy_type > static_cast<int>(ProxyType::NONE) && proxy_type < static_cast<int>(ProxyType::count) && !currentServer_.GetBypassProxy()) {
 		log(logmsg::status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port), CProxySocket::Name(static_cast<ProxyType>(proxy_type)));
 
-		fz::native_string proxy_host = fz::to_native(engine_.GetOptions().GetOption(OPTION_PROXY_HOST));
+		fz::native_string proxy_host = fz::to_native(engine_.GetOptions().get_string(OPTION_PROXY_HOST));
 
 		proxy_layer_ = std::make_unique<CProxySocket>(this, *active_layer_, this, static_cast<ProxyType>(proxy_type),
-			proxy_host, engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT),
-			engine_.GetOptions().GetOption(OPTION_PROXY_USER),
-			engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
+			proxy_host, engine_.GetOptions().get_int(OPTION_PROXY_PORT),
+			engine_.GetOptions().get_string(OPTION_PROXY_USER),
+			engine_.GetOptions().get_string(OPTION_PROXY_PASS));
 		active_layer_ = proxy_layer_.get();
 
 		if (fz::get_address_type(proxy_host) == fz::address_type::unknown) {
@@ -873,8 +894,6 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 		}
 	}
 
-	m_closed = false;
-
 	int res = active_layer_->connect(fz::to_native(ConvertDomainName(host)), port);
 
 	if (res) {
@@ -907,7 +926,9 @@ void CRealControlSocket::ResetSocket()
 
 bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNotification)
 {
-	assert(pFileExistsNotification);
+	if (!pFileExistsNotification) {
+		return false;
+	}
 
 	if (operations_.empty() || operations_.back()->opId != Command::transfer) {
 		log(logmsg::debug_info, L"SetFileExistsAction: No or invalid operation in progress, ignoring request reply %f", pFileExistsNotification->GetRequestID());
@@ -932,12 +953,12 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			SendNextCommand();
 		}
 		else {
-			if (data.download_) {
+			if (data.download()) {
 				std::wstring filename = data.remotePath_.FormatFilename(data.remoteFile_);
 				log(logmsg::status, _("Skipping download of %s"), filename);
 			}
 			else {
-				log(logmsg::status, _("Skipping upload of %s"), data.localFile_);
+				log(logmsg::status, _("Skipping upload of %s"), data.localName_);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
@@ -949,12 +970,12 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			SendNextCommand();
 		}
 		else {
-			if (data.download_) {
+			if (data.download()) {
 				std::wstring filename = data.remotePath_.FormatFilename(data.remoteFile_);
 				log(logmsg::status, _("Skipping download of %s"), filename);
 			}
 			else {
-				log(logmsg::status, _("Skipping upload of %s"), data.localFile_);
+				log(logmsg::status, _("Skipping upload of %s"), data.localName_);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
@@ -975,54 +996,39 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			SendNextCommand();
 		}
 		else {
-			if (data.download_) {
+			if (data.download()) {
 				auto const filename = data.remotePath_.FormatFilename(data.remoteFile_);
 				log(logmsg::status, _("Skipping download of %s"), filename);
 			}
 			else {
-				log(logmsg::status, _("Skipping upload of %s"), data.localFile_);
+				log(logmsg::status, _("Skipping upload of %s"), data.localName_);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
 		break;
 	case CFileExistsNotification::resume:
-		if (data.download_ && data.localFileSize_ >= 0) {
+		if (data.download() && data.localFileSize_ != aio_base::nosize) {
 			data.resume_ = true;
 		}
-		else if (!data.download_ && data.remoteFileSize_ >= 0) {
+		else if (!data.download() && data.remoteFileSize_ >= 0) {
 			data.resume_ = true;
 		}
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::rename:
-		if (data.download_) {
+		if (data.download()) {
 			{
-				std::wstring tmp;
-				CLocalPath l(data.localFile_, &tmp);
-				if (l.empty() || tmp.empty()) {
-					ResetOperation(FZ_REPLY_INTERNALERROR);
-					return false;
-				}
-				if (!l.ChangePath(pFileExistsNotification->newName)) {
-					ResetOperation(FZ_REPLY_INTERNALERROR);
-					return false;
-				}
-				if (!l.HasParent() || !l.MakeParent(&tmp)) {
+				if (!pFileExistsNotification->new_writer_factory_) {
 					ResetOperation(FZ_REPLY_INTERNALERROR);
 					return false;
 				}
 
-				data.localFile_ = l.GetPath() + tmp;
+				data.writer_factory_ = std::move(pFileExistsNotification->new_writer_factory_);
+				data.localName_ = data.writer_factory_.name();
 			}
 
-			int64_t size;
-			bool isLink;
-			if (fz::local_filesys::get_file_info(fz::to_native(data.localFile_), isLink, &size, nullptr, nullptr) == fz::local_filesys::file) {
-				data.localFileSize_ = size;
-			}
-			else {
-				data.localFileSize_ = -1;
-			}
+			data.localFileSize_ = data.writer_factory_.size();
+			data.localFileTime_ = data.writer_factory_.mtime();
 
 			if (CheckOverwriteFile() == FZ_REPLY_OK) {
 				SendNextCommand();
@@ -1030,8 +1036,8 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		else {
 			data.remoteFile_ = pFileExistsNotification->newName;
-			data.fileTime_ = fz::datetime();
 			data.remoteFileSize_ = -1;
+			data.remoteFileTime_ = fz::datetime();
 
 			CDirentry entry;
 			bool dir_did_exist;
@@ -1041,7 +1047,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			{
 				data.remoteFileSize_ = entry.size;
 				if (entry.has_date()) {
-					data.fileTime_ = entry.time;
+					data.remoteFileTime_ = entry.time;
 				}
 
 				if (CheckOverwriteFile() != FZ_REPLY_OK) {
@@ -1053,12 +1059,12 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::skip:
-		if (data.download_) {
+		if (data.download()) {
 			std::wstring filename = data.remotePath_.FormatFilename(data.remoteFile_);
 			log(logmsg::status, _("Skipping download of %s"), filename);
 		}
 		else {
-			log(logmsg::status, _("Skipping upload of %s"), data.localFile_);
+			log(logmsg::status, _("Skipping upload of %s"), data.localName_);
 		}
 		ResetOperation(FZ_REPLY_OK);
 		break;
@@ -1071,25 +1077,6 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 	return true;
 }
 
-void CControlSocket::CreateLocalDir(std::wstring const& local_file)
-{
-	std::wstring file;
-	CLocalPath local_path(local_file, &file);
-	if (local_path.empty() || !local_path.HasParent()) {
-		return;
-	}
-
-	CLocalPath last_successful;
-	local_path.Create(&last_successful);
-
-	if (!last_successful.empty()) {
-		// Send out notification
-		CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
-		n->dir = last_successful;
-		engine_.AddNotification(n);
-	}
-}
-
 void CControlSocket::List(CServerPath const&, std::wstring const&, int)
 {
 	Push(std::make_unique<CNotSupportedOpData>());
@@ -1125,6 +1112,16 @@ void CControlSocket::Chmod(CChmodCommand const&)
 	Push(std::make_unique<CNotSupportedOpData>());
 }
 
+void CControlSocket::Lookup(CServerPath const& path, std::wstring const& file, CDirentry * entry)
+{
+	Push(std::make_unique<LookupOpData>(*this, path, file, entry));
+}
+
+void CControlSocket::Lookup(CServerPath const& path, std::vector<std::wstring> const& files)
+{
+	Push(std::make_unique<LookupManyOpData>(*this, path, files));
+}
+
 void CControlSocket::operator()(fz::event_base const& ev)
 {
 	fz::dispatch<fz::timer_event, CObtainLockEvent>(ev, this,
@@ -1144,7 +1141,7 @@ void CControlSocket::SendDirectoryListingNotification(CServerPath const& path, b
 		return;
 	}
 
-	engine_.AddNotification(new CDirectoryListingNotification(path, operations_.size() == 1 && operations_.back()->opId == Command::list, failed));
+	engine_.AddNotification(std::make_unique<CDirectoryListingNotification>(path, operations_.size() == 1 && operations_.back()->opId == Command::list, failed));
 }
 
 void CControlSocket::CallSetAsyncRequestReply(CAsyncRequestNotification *pNotification)
@@ -1160,14 +1157,9 @@ void CControlSocket::CallSetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	SetAsyncRequestReply(pNotification);
 }
 
-void CControlSocket::Lookup(CServerPath const& path, std::wstring const& file, CDirentry * entry)
+void CControlSocket::Sleep(fz::duration const& delay)
 {
-	Push(std::make_unique<LookupOpData>(*this, path, file, entry));
-}
-
-void CControlSocket::Lookup(CServerPath const& path, std::vector<std::wstring> const& files)
-{
-	Push(std::make_unique<LookupManyOpData>(*this, path, files));
+	Push(std::make_unique<SleepOpData>(*this, delay));
 }
 
 int64_t CalculateNextChunkSize(int64_t remaining, int64_t lastChunkSize, fz::duration const& lastChunkDuration, int64_t minChunkSize, int64_t multiple, int64_t partCount, int64_t maxPartCount, int64_t maxChunkSize)
@@ -1198,7 +1190,7 @@ int64_t CalculateNextChunkSize(int64_t remaining, int64_t lastChunkSize, fz::dur
 
 	if (multiple) {
 		// Round up
-		int modulus = newChunkSize % multiple;
+		auto modulus = newChunkSize % multiple;
 		if (modulus) {
 			newChunkSize += multiple - modulus;
 		}
@@ -1214,3 +1206,8 @@ int64_t CalculateNextChunkSize(int64_t remaining, int64_t lastChunkSize, fz::dur
 
 	return newChunkSize;
 }
+
+int64_t CalculateNextChunkSize(int64_t remaining, int64_t lastChunkSize, fz::monotonic_clock const& lastChunkStart, int64_t minChunkSize, int64_t multiple, int64_t partCount, int64_t maxPartCount, int64_t maxChunkSize)
+{
+	return CalculateNextChunkSize(remaining, lastChunkSize, fz::monotonic_clock::now() - lastChunkStart, minChunkSize, multiple, partCount, maxPartCount, maxChunkSize);
+}
