@@ -12,11 +12,11 @@
 
 #include <assert.h>
 
-CFtpFileTransferOpData::CFtpFileTransferOpData(CFtpControlSocket& controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
-	: CFileTransferOpData(L"CFtpFileTransferOpData", is_download, local_file, remote_file, remote_path, settings)
+CFtpFileTransferOpData::CFtpFileTransferOpData(CFtpControlSocket& controlSocket, CFileTransferCommand const& cmd)
+	: CFileTransferOpData(L"CFtpFileTransferOpData", cmd)
 	, CFtpOpData(controlSocket)
 {
-	binary = settings.binary;
+	binary = !(cmd.GetFlags() & ftp_transfer_flags::ascii);
 }
 
 int CFtpFileTransferOpData::Send()
@@ -25,28 +25,15 @@ int CFtpFileTransferOpData::Send()
 	switch (opState)
 	{
 	case filetransfer_init:
-		if (localFile_.empty()) {
-			if (!download_) {
-				return FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED;
-			}
-			else {
-				return FZ_REPLY_SYNTAXERROR;
-			}
-		}
-
-		if (download_) {
+		if (download()) {
 			std::wstring filename = remotePath_.FormatFilename(remoteFile_);
 			log(logmsg::status, _("Starting download of %s"), filename);
 		}
 		else {
-			log(logmsg::status, _("Starting upload of %s"), localFile_);
+			log(logmsg::status, _("Starting upload of %s"), localName_);
 		}
 
-		int64_t size;
-		bool isLink;
-		if (fz::local_filesys::get_file_info(fz::to_native(localFile_), isLink, &size, nullptr, nullptr) == fz::local_filesys::file) {
-			localFileSize_ = size;
-		}
+		localFileSize_ = download() ? writer_factory_.size() : reader_factory_.size();
 
 		opState = filetransfer_waitcwd;
 
@@ -72,28 +59,14 @@ int CFtpFileTransferOpData::Send()
 		}
 
 		{
-			auto pFile = std::make_unique<fz::file>();
-			if (download_) {
-				int64_t startOffset = 0;
-
+			resumeOffset = 0;
+			if (download()) {
 				// Potentially racy
-				bool didExist = fz::local_filesys::get_file_type(fz::to_native(localFile_)) != fz::local_filesys::unknown;
+				localFileSize_ = writer_factory_.size(); 
+				fileDidExist_ = localFileSize_ != aio_base::nosize;
 
 				if (resume_) {
-					if (!pFile->open(fz::to_native(localFile_), fz::file::writing, fz::file::existing)) {
-						log(logmsg::error, _("Failed to open \"%s\" for appending/writing"), localFile_);
-						return FZ_REPLY_ERROR;
-					}
-
-					fileDidExist_ = didExist;
-
-					startOffset = pFile->seek(0, fz::file::end);
-
-					if (startOffset == -1) {
-						log(logmsg::error, _("Could not seek to the end of the file"));
-						return FZ_REPLY_ERROR;
-					}
-					localFileSize_ = startOffset;
+					resumeOffset = fileDidExist_ ? static_cast<int64_t>(localFileSize_) : 0;
 
 					// Check resume capabilities
 					if (opState == filetransfer_resumetest) {
@@ -104,122 +77,69 @@ int CFtpFileTransferOpData::Send()
 					}
 				}
 				else {
-					controlSocket_.CreateLocalDir(localFile_);
-
-					if (!pFile->open(fz::to_native(localFile_), fz::file::writing, fz::file::empty)) {
-						log(logmsg::error, _("Failed to open \"%s\" for writing"), localFile_);
-						return FZ_REPLY_ERROR;
-					}
-
-					fileDidExist_ = didExist;
 					localFileSize_ = 0;
 				}
 
-				resumeOffset = resume_ ? localFileSize_ : 0;
-
-				engine_.transfer_status_.Init(remoteFileSize_, startOffset, false);
-
-				if (engine_.GetOptions().get_int(OPTION_PREALLOCATE_SPACE)) {
-					// Try to preallocate the file in order to reduce fragmentation
-					int64_t sizeToPreallocate = remoteFileSize_ - startOffset;
-					if (sizeToPreallocate > 0) {
-						log(logmsg::debug_info, L"Preallocating %d bytes for the file \"%s\"", sizeToPreallocate, localFile_);
-						auto oldPos = pFile->seek(0, fz::file::current);
-						if (oldPos >= 0) {
-							if (pFile->seek(sizeToPreallocate, fz::file::end) == remoteFileSize_) {
-								if (!pFile->truncate()) {
-									log(logmsg::debug_warning, L"Could not preallocate the file");
-								}
-							}
-							if (pFile->seek(oldPos, fz::file::begin) != oldPos) {
-								log(logmsg::error, _("Could not seek to offset %d within file"), oldPos);
-								return FZ_REPLY_ERROR;
-							}
-						}
-					}
-				}
+				engine_.transfer_status_.Init(remoteFileSize_, resumeOffset, false);
 			}
 			else {
-				if (!pFile->open(fz::to_native(localFile_), fz::file::reading)) {
-					log(logmsg::error, _("Failed to open \"%s\" for reading"), localFile_);
-					return FZ_REPLY_ERROR;
-				}
-
-				int64_t startOffset;
 				if (resume_) {
 					if (remoteFileSize_ > 0) {
-						startOffset = remoteFileSize_;
+						resumeOffset = remoteFileSize_;
 
-						if (localFileSize_ < 0) {
-							auto s = pFile->size();
-							if (s >= 0) {
-								localFileSize_ = s;
-							}
-						}
-
-						if (startOffset == localFileSize_ && binary) {
+						if (localFileSize_ != aio_base::nosize && resumeOffset >= static_cast<int64_t>(localFileSize_) && binary) {
 							log(logmsg::debug_info, L"No need to resume, remote file size matches local file size.");
 
 							if (engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) &&
 								CServerCapabilities::GetCapability(currentServer_, mfmt_command) == yes)
 							{
-								fz::datetime mtime = fz::local_filesys::get_modification_time(fz::to_native(localFile_));
-								if (!mtime.empty()) {
-									fileTime_ = mtime;
+								localFileTime_ = reader_factory_.mtime();
+								if (!localFileTime_.empty()) {
 									opState = filetransfer_mfmt;
 									return FZ_REPLY_CONTINUE;
 								}
 							}
 							return FZ_REPLY_OK;
 						}
+					}
+				}
 
-						if (pFile->seek(startOffset, fz::file::begin) == -1) {
-							log(logmsg::error, _("Could not seek to offset %d within file"), startOffset);
+				engine_.transfer_status_.Init(reader_factory_.size(), resumeOffset, false);
+			}
+
+			controlSocket_.m_pTransferSocket = std::make_unique<CTransferSocket>(engine_, controlSocket_, download() ? TransferMode::download : TransferMode::upload);
+			controlSocket_.m_pTransferSocket->m_binaryMode = binary;
+			if (download()) {
+				auto writer = writer_factory_.open(resumeOffset, engine_, controlSocket_.m_pTransferSocket.get(), aio_base::shm_flag_none);
+				if (!writer) {
+					return FZ_REPLY_CRITICALERROR;
+				}
+				if (engine_.GetOptions().get_int(OPTION_PREALLOCATE_SPACE)) {
+					if (remoteFileSize_ != aio_base::nosize && remoteFileSize_ > resumeOffset) {
+						if (writer->preallocate(static_cast<uint64_t>(remoteFileSize_ - resumeOffset)) != aio_result::ok) {
 							return FZ_REPLY_ERROR;
 						}
 					}
-					else {
-						startOffset = 0;
-					}
 				}
-				else {
-					startOffset = 0;
-				}
-
-				if (CServerCapabilities::GetCapability(currentServer_, rest_stream) == yes) {
-					// Use REST + STOR if resuming
-					resumeOffset = startOffset;
-				}
-				else {
-					// Play it safe, use APPE if resuming
-					resumeOffset = 0;
-				}
-
-				auto len = pFile->size();
-				engine_.transfer_status_.Init(len, startOffset, false);
+				controlSocket_.m_pTransferSocket->set_writer(std::move(writer), flags_ & ftp_transfer_flags::ascii);
 			}
-			ioThread_ = std::make_unique<CIOThread>();
-			if (!ioThread_->Create(engine_.GetThreadPool(), std::move(pFile), !download_, binary)) {
-				// CIOThread will delete pFile
-				ioThread_.reset();
-				log(logmsg::error, _("Could not spawn IO thread"));
-				return FZ_REPLY_ERROR;
+			else {
+				auto reader = reader_factory_.open(resumeOffset, engine_, nullptr, aio_base::shm_flag_none);
+				if (!reader) {
+					return FZ_REPLY_CRITICALERROR;
+				}
+				controlSocket_.m_pTransferSocket->set_reader(std::move(reader), flags_ & ftp_transfer_flags::ascii);
 			}
 		}
 
-		controlSocket_.m_pTransferSocket = std::make_unique<CTransferSocket>(engine_, controlSocket_, download_ ? TransferMode::download : TransferMode::upload);
-		controlSocket_.m_pTransferSocket->m_binaryMode = transferSettings_.binary;
-		controlSocket_.m_pTransferSocket->SetIOThread(ioThread_.get());
-
-		if (download_) {
+		if (download()) {
 			cmd = L"RETR ";
 		}
-		else if (resume_) {
+		else if (resume_ && resumeOffset != 0) {
 			if (CServerCapabilities::GetCapability(currentServer_, rest_stream) == yes) {
 				cmd = L"STOR "; // In this case REST gets sent since resume offset was set earlier
 			}
 			else {
-				assert(resumeOffset == 0);
 				cmd = L"APPE ";
 			}
 		}
@@ -234,7 +154,7 @@ int CFtpFileTransferOpData::Send()
 	case filetransfer_mfmt:
 	{
 		cmd = L"MFMT ";
-		fz::datetime t = fileTime_;
+		fz::datetime t = localFileTime_;
 		t -= fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
 		cmd += t.format(L"%Y%m%d%H%M%S ", fz::datetime::utc);
 		cmd += remotePath_.FormatFilename(remoteFile_, !tryAbsolutePath_);
@@ -257,31 +177,31 @@ int CFtpFileTransferOpData::TestResumeCapability()
 {
 	log(logmsg::debug_verbose, L"CFtpFileTransferOpData::TestResumeCapability()");
 
-	if (!download_) {
+	if (!download()) {
 		return FZ_REPLY_CONTINUE;
 	}
 
 	for (int i = 0; i < 2; ++i) {
-		if (localFileSize_ >= (1ll << (i ? 31 : 32))) {
+		if (localFileSize_ >= (1ull << (i ? 31 : 32))) {
 			switch (CServerCapabilities::GetCapability(currentServer_, i ? resume2GBbug : resume4GBbug))
 			{
 			case yes:
-				if (remoteFileSize_ == localFileSize_) {
+				if (static_cast<uint64_t>(remoteFileSize_) == localFileSize_) {
 					log(logmsg::debug_info, _("Server does not support resume of files > %d GB. End transfer since file sizes match."), i ? 2 : 4);
 					return FZ_REPLY_OK;
 				}
 				log(logmsg::error, _("Server does not support resume of files > %d GB."), i ? 2 : 4);
 				return FZ_REPLY_CRITICALERROR;
 			case unknown:
-				if (remoteFileSize_ < localFileSize_) {
+				if (static_cast<uint64_t>(remoteFileSize_) < localFileSize_) {
 					// Don't perform test
 					break;
 				}
-				if (remoteFileSize_ == localFileSize_) {
+				if (static_cast<uint64_t>(remoteFileSize_) == localFileSize_) {
 					log(logmsg::debug_info, _("Server may not support resume of files > %d GB. End transfer since file sizes match."), i ? 2 : 4);
 					return FZ_REPLY_OK;
 				}
-				else if (remoteFileSize_ > localFileSize_) {
+				else if (static_cast<uint64_t>(remoteFileSize_) > localFileSize_) {
 					log(logmsg::status, _("Testing resume capabilities of server"));
 
 					opState = filetransfer_waitresumetest;
@@ -355,9 +275,9 @@ int CFtpFileTransferOpData::ParseResponse()
 	case filetransfer_mdtm:
 		opState = filetransfer_resumetest;
 		if (response.substr(0, 4) == L"213 " && response.size() > 16) {
-			fileTime_ = fz::datetime(response.substr(4), fz::datetime::utc);
-			if (!fileTime_.empty()) {
-				fileTime_ += fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
+			remoteFileTime_ = fz::datetime(response.substr(4), fz::datetime::utc);
+			if (!remoteFileTime_.empty()) {
+				remoteFileTime_ += fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
 			}
 		}
 
@@ -391,7 +311,7 @@ int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (!dirDidExist) {
 					opState = filetransfer_waitlist;
 				}
-				else if (download_ && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) && CServerCapabilities::GetCapability(currentServer_, mdtm_command) == yes) {
+				else if (download() && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) && CServerCapabilities::GetCapability(currentServer_, mdtm_command) == yes) {
 					opState = filetransfer_mdtm;
 				}
 				else {
@@ -406,10 +326,10 @@ int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 					if (matchedCase) {
 						remoteFileSize_ = entry.size;
 						if (entry.has_date()) {
-							fileTime_ = entry.time;
+							remoteFileTime_ = entry.time;
 						}
 
-						if (download_ &&
+						if (download() &&
 							!entry.has_time() &&
 							engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) &&
 							CServerCapabilities::GetCapability(currentServer_, mdtm_command) == yes)
@@ -451,7 +371,7 @@ int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (!dirDidExist) {
 					opState = filetransfer_size;
 				}
-				else if (download_ &&
+				else if (download() &&
 					engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) &&
 					CServerCapabilities::GetCapability(currentServer_, mdtm_command) == yes)
 				{
@@ -465,10 +385,10 @@ int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (matchedCase && !entry.is_unsure()) {
 					remoteFileSize_ = entry.size;
 					if (entry.has_date()) {
-						fileTime_ = entry.time;
+						remoteFileTime_ = entry.time;
 					}
 
-					if (download_ &&
+					if (download() &&
 						!entry.has_time() &&
 						engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS) &&
 						CServerCapabilities::GetCapability(currentServer_, mdtm_command) == yes)
@@ -497,19 +417,17 @@ int CFtpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 	}
 	else if (opState == filetransfer_waittransfer) {
 		if (prevResult == FZ_REPLY_OK && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
-			if (!download_ &&
+			if (!download() &&
 				CServerCapabilities::GetCapability(currentServer_, mfmt_command) == yes)
 			{
-				fz::datetime mtime = fz::local_filesys::get_modification_time(fz::to_native(localFile_));
-				if (!mtime.empty()) {
-					fileTime_ = mtime;
+				localFileTime_ = reader_factory_.mtime();
+				if (!localFileTime_.empty()) {
 					opState = filetransfer_mfmt;
 					return FZ_REPLY_CONTINUE;
 				}
 			}
-			else if (download_ && !fileTime_.empty()) {
-				ioThread_.reset();
-				if (!fz::local_filesys::set_modification_time(fz::to_native(localFile_), fileTime_)) {
+			else if (download() && !remoteFileTime_.empty()) {
+				if (!writer_factory_.set_mtime(remoteFileTime_)) {
 					log(logmsg::debug_warning, L"Could not set modification time");
 				}
 			}
