@@ -17,7 +17,6 @@
 #include "../directorycache.h"
 #include "../directorylistingparser.h"
 #include "../engineprivate.h"
-#include "../iothread.h"
 #include "../pathcache.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
@@ -346,18 +345,6 @@ int CFtpControlSocket::ResetOperation(int nErrorCode)
 				}
 			}
 		}
-		if (nErrorCode != FZ_REPLY_OK && data.download_ && !data.fileDidExist_) {
-			data.ioThread_.reset();
-			int64_t size;
-			bool isLink;
-			if (fz::local_filesys::get_file_info(fz::to_native(data.localFile_), isLink, &size, nullptr, nullptr) == fz::local_filesys::file && size == 0) {
-				// Download failed and a new local file was created before, but
-				// nothing has been written to it. Remove it again, so we don't
-				// leave a bunch of empty files all over the place.
-				log(logmsg::debug_verbose, L"Deleting empty file");
-				fz::remove_file(fz::to_native(data.localFile_));
-			}
-		}
 	}
 
 	if (!operations_.empty() && operations_.back()->opId == PrivCommand::rawtransfer &&
@@ -407,7 +394,7 @@ void CFtpControlSocket::ChangeDir(CServerPath const& path, std::wstring const& s
 	pData->link_discovery_ = link_discovery;
 
 	if (!operations_.empty() && operations_.back()->opId == Command::transfer &&
-		!static_cast<CFtpFileTransferOpData &>(*operations_.back()).download_)
+		!static_cast<CFtpFileTransferOpData &>(*operations_.back()).download())
 	{
 		pData->tryMkdOnFail_ = true;
 		assert(subDir.empty());
@@ -416,13 +403,11 @@ void CFtpControlSocket::ChangeDir(CServerPath const& path, std::wstring const& s
 	Push(std::move(pData));
 }
 
-void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-									std::wstring const& remoteFile, bool download,
-									CFileTransferCommand::t_transferSettings const& transferSettings)
+void CFtpControlSocket::FileTransfer(CFileTransferCommand const& cmd)
 {
 	log(logmsg::debug_verbose, L"CFtpControlSocket::FileTransfer()");
 
-	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
+	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, cmd);
 	Push(std::move(pData));
 }
 
