@@ -1,10 +1,9 @@
 #ifndef FILEZILLA_ENGINE_FTP_FTPCONTROLSOCKET_HEADER
 #define FILEZILLA_ENGINE_FTP_FTPCONTROLSOCKET_HEADER
 
-#include "logging_private.h"
-#include "controlsocket.h"
-#include "externalipresolver.h"
-#include "rtt.h"
+#include "../logging_private.h"
+#include "../controlsocket.h"
+#include "../rtt.h"
 
 #include <regex>
 
@@ -13,6 +12,7 @@ auto const cwd = Command::private1;
 auto const rawtransfer = Command::private2;
 }
 
+class CExternalIPResolver;
 class CTransferSocket;
 class CFtpTransferOpData;
 class CFtpRawTransferOpData;
@@ -30,20 +30,20 @@ class CFtpControlSocket final : public CRealControlSocket
 public:
 	CFtpControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CFtpControlSocket();
-	
+
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification) override;
 
 protected:
 
+	virtual void Push(std::unique_ptr<COpData> && pNewOpData) override;
+
 	virtual int ResetOperation(int nErrorCode) override;
 
 	// Implicit FZ_REPLY_CONTINUE
 	virtual void Connect(CServer const& server, Credentials const& credentials) override;
 	virtual void List(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring(), int flags = 0) override;
 	void ChangeDir(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring(), bool link_discovery = false);
-	virtual void FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-							 std::wstring const& remoteFile, bool download,
-							 CFileTransferCommand::t_transferSettings const& transferSettings) override;
+	virtual void FileTransfer(CFileTransferCommand const& cmd) override;
 	virtual void RawCommand(std::wstring const& command) override;
 	virtual void Delete(CServerPath const& path, std::vector<std::wstring>&& files) override;
 	virtual void RemoveDir(CServerPath const& path, std::wstring const& subDir) override;
@@ -56,7 +56,7 @@ protected:
 
 	virtual void OnConnect() override;
 	virtual void OnReceive() override;
-	
+
 	void OnVerifyCert(fz::tls_layer* source, fz::tls_session_info& info);
 
 	virtual void ResetSocket() override;
