@@ -1,15 +1,18 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "logon.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
 
+#include "../../include/engine_options.h"
+
 #include <libfilezilla/tls_layer.hpp>
 
-CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, Credentials const& credentials)
+#include <assert.h>
+
+CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket)
 	: COpData(Command::connect, L"CFtpLogonOpData")
 	, CFtpOpData(controlSocket)
-	, credentials_(credentials)
 {
 	for (int i = 0; i < LOGON_DONE; ++i) {
 		neededCommands[i] = 1;
@@ -44,9 +47,9 @@ int CFtpLogonOpData::Send()
 	case LOGON_CONNECT:
 		{
 			// Do not use FTP proxy if generic proxy is set
-			int const generic_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
+			int const generic_proxy_type = engine_.GetOptions().get_int(OPTION_PROXY_TYPE);
 			if ((generic_proxy_type <= static_cast<int>(ProxyType::NONE) || generic_proxy_type >= static_cast<int>(ProxyType::count)) && !currentServer_.GetBypassProxy()) {
-				ftp_proxy_type_ = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE);
+				ftp_proxy_type_ = engine_.GetOptions().get_int(OPTION_FTP_PROXY_TYPE);
 			}
 
 			if (!PrepareLoginSequence()) {
@@ -54,7 +57,7 @@ int CFtpLogonOpData::Send()
 			}
 
 			if (ftp_proxy_type_) {
-				host_ = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_HOST);
+				host_ = engine_.GetOptions().get_string(OPTION_FTP_PROXY_HOST);
 
 				size_t pos = -1;
 				if (!host_.empty() && host_[0] == '[') {
@@ -92,7 +95,7 @@ int CFtpLogonOpData::Send()
 					return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 				}
 
-				log(logmsg::status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
+				log(logmsg::status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
 			}
 			else {
 				host_ = currentServer_.GetHost();
@@ -107,7 +110,7 @@ int CFtpLogonOpData::Send()
 
 				// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
 				// firewalls which terminate the control connection on long transfers.
-				int v = engine_.GetOptions().GetOptionVal(OPTION_TCP_KEEPALIVE_INTERVAL);
+				int v = engine_.GetOptions().get_int(OPTION_TCP_KEEPALIVE_INTERVAL);
 				if (v >= 1 && v < 10000) {
 					controlSocket_.socket_->set_keepalive_interval(fz::duration::from_minutes(v));
 				}
@@ -130,8 +133,7 @@ int CFtpLogonOpData::Send()
 			return FZ_REPLY_CONTINUE;
 		}
 		else {
-			auto notification = new CInsecureConnectionNotification(currentServer_);
-			controlSocket_.SendAsyncRequest(notification);
+			controlSocket_.SendAsyncRequest(std::make_unique<CInsecureConnectionNotification>(currentServer_));
 			return FZ_REPLY_WOULDBLOCK;
 		}
 	case LOGON_SYST:
@@ -142,13 +144,13 @@ int CFtpLogonOpData::Send()
 			switch (cmd.type)
 			{
 			case loginCommandType::user:
-				if (credentials_.logonType_ == LogonType::interactive) {
+				if (controlSocket_.credentials_.logonType_ == LogonType::interactive) {
 					waitChallenge = true;
 					challenge.clear();
 				}
 
 				if (cmd.command.empty()) {
-					std::wstring const user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+					std::wstring const user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
 					return controlSocket_.SendCommand(L"USER " + user);
 				}
 				else {
@@ -156,18 +158,18 @@ int CFtpLogonOpData::Send()
 				}
 			case loginCommandType::pass:
 				if (!challenge.empty()) {
-					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(CInteractiveLoginNotification::interactive, challenge, false);
-					pNotification->server = currentServer_;
-					pNotification->handle_ = controlSocket_.handle_;
-					pNotification->credentials = credentials_;
+					auto notification = std::make_unique<CInteractiveLoginNotification>(CInteractiveLoginNotification::interactive, challenge, false);
+					notification->server = currentServer_;
+					notification->handle_ = controlSocket_.handle_;
+					notification->credentials = controlSocket_.credentials_;
 					challenge.clear();
 
-					controlSocket_.SendAsyncRequest(pNotification);
+					controlSocket_.SendAsyncRequest(std::move(notification));
 
 					return FZ_REPLY_WOULDBLOCK;
 				}
 				else {
-					std::wstring pass = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : credentials_.GetPass();
+					std::wstring pass = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : controlSocket_.credentials_.GetPass();
 					if (cmd.command.empty()) {
 						return controlSocket_.SendCommand(L"PASS " + pass, true);
 					}
@@ -182,7 +184,7 @@ int CFtpLogonOpData::Send()
 				break;
 			case loginCommandType::account:
 				if (cmd.command.empty()) {
-					return controlSocket_.SendCommand(L"ACCT " + credentials_.account_);
+					return controlSocket_.SendCommand(L"ACCT " + controlSocket_.credentials_.account_);
 				}
 				else {
 					return controlSocket_.SendCommand(cmd.command);
@@ -292,7 +294,7 @@ int CFtpLogonOpData::ParseResponse()
 				if (!user.empty() && (user.front() == ' ' || user.back() == ' ')) {
 					log(logmsg::status, _("Check your login credentials. The entered username starts or ends with a space character."));
 				}
-				auto const pw = credentials_.GetPass();
+				auto const pw = controlSocket_.credentials_.GetPass();
 				if (!pw.empty() && (pw.front() == ' ' || pw.back() == ' ')) {
 					log(logmsg::status, _("Check your login credentials. The entered password starts or ends with a space character."));
 				}
@@ -305,10 +307,10 @@ int CFtpLogonOpData::ParseResponse()
 				if (!fz::str_is_ascii(currentServer_.GetUser())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(credentials_.GetPass())) {
+				if (!fz::str_is_ascii(controlSocket_.credentials_.GetPass())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(credentials_.account_)) {
+				if (!fz::str_is_ascii(controlSocket_.credentials_.account_)) {
 					asciiOnly = false;
 				}
 				if (!asciiOnly) {
@@ -348,7 +350,7 @@ int CFtpLogonOpData::ParseResponse()
 		}
 		else if (code == 3 && loginSequence.empty()) {
 			log(logmsg::error, _("Login sequence fully executed yet not logged in, aborting."));
-			if (cmd.type == loginCommandType::pass && credentials_.account_.empty()) {
+			if (cmd.type == loginCommandType::pass && controlSocket_.credentials_.account_.empty()) {
 				log(logmsg::error, _("Server might require an account. Try specifying an account using the Site Manager"));
 			}
 			return FZ_REPLY_CRITICALERROR | FZ_REPLY_DISCONNECTED;
@@ -587,26 +589,26 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
 	else if (ftp_proxy_type_ == 1) {
-		std::wstring const proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
+		std::wstring const proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
 			t_loginCommand cmd = {false, false, loginCommandType::other, L"USER " + proxyUser};
 			loginSequence.push_back(cmd);
 			cmd.optional = true;
 			cmd.hide_arguments = true;
-			cmd.command = L"PASS " + engine_.GetOptions().GetOption(OPTION_FTP_PROXY_PASS);
+			cmd.command = L"PASS " + engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
 			loginSequence.push_back(cmd);
 		}
 		// User@host
-		std::wstring const user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
-		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", user, currentServer_.Format(ServerFormat::with_optional_port, credentials_))};
+		std::wstring const user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", user, currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_))};
 		loginSequence.push_back(cmd);
 
 		// Password
@@ -617,31 +619,31 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
 	else if (ftp_proxy_type_ == 2 || ftp_proxy_type_ == 3) {
-		std::wstring const proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
+		std::wstring const proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
 			t_loginCommand cmd = {false, false, loginCommandType::other, L"USER " + proxyUser};
 			loginSequence.push_back(cmd);
 			cmd.optional = true;
 			cmd.hide_arguments = true;
-			cmd.command = L"PASS " + engine_.GetOptions().GetOption(OPTION_FTP_PROXY_PASS);
+			cmd.command = L"PASS " + engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
 			loginSequence.push_back(cmd);
 		}
 
 		// Site or Open
 		t_loginCommand cmd = {false, false, loginCommandType::user, L""};
 		if (ftp_proxy_type_ == 2) {
-			cmd.command = L"SITE " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
+			cmd.command = L"SITE " + currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
 		}
 		else {
-			cmd.command = L"OPEN " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
+			cmd.command = L"OPEN " + currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
 		}
 		loginSequence.push_back(cmd);
 
@@ -657,25 +659,25 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
 	else if (ftp_proxy_type_ == 4) {
-		std::wstring proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
-		std::wstring proxyPass = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_PASS);
-		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, credentials_);
-		std::wstring user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
-		std::wstring account = credentials_.account_;
+		std::wstring proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
+		std::wstring proxyPass = engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
+		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
+		std::wstring user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+		std::wstring account = controlSocket_.credentials_.account_;
 		fz::replace_substrings(proxyUser, L"%", L"%%");
 		fz::replace_substrings(proxyPass, L"%", L"%%");
 		fz::replace_substrings(host, L"%", L"%%");
 		fz::replace_substrings(user, L"%", L"%%");
 		fz::replace_substrings(account, L"%", L"%%");
 
-		std::wstring const loginSequenceStr = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_CUSTOMLOGINSEQUENCE);
+		std::wstring const loginSequenceStr = engine_.GetOptions().get_string(OPTION_FTP_PROXY_CUSTOMLOGINSEQUENCE);
 		std::vector<std::wstring> const tokens = fz::strtok(loginSequenceStr, L"\r\n");
 
 		for (auto token : tokens) {
@@ -807,7 +809,7 @@ void CFtpLogonOpData::ParseFeat(std::wstring line)
 		CServerCapabilities::SetCapability(currentServer_, mlsd_command, yes, facts);
 
 		// MLST/MLSD specs require use of UTC
-		CServerCapabilities::SetCapability(currentServer_, timezone_offset, no);
+		CServerCapabilities::SetCapability(currentServer_, inferred_timezone_offset, no);
 	}
 	else if (HasFeature(up, L"MLST")) {
 		std::wstring facts;
@@ -823,7 +825,7 @@ void CFtpLogonOpData::ParseFeat(std::wstring line)
 		CServerCapabilities::SetCapability(currentServer_, mlsd_command, yes, facts);
 
 		// MLST/MLSD specs require use of UTC
-		CServerCapabilities::SetCapability(currentServer_, timezone_offset, no);
+		CServerCapabilities::SetCapability(currentServer_, inferred_timezone_offset, no);
 	}
 	else if (HasFeature(up, L"MODE Z")) {
 		CServerCapabilities::SetCapability(currentServer_, mode_z_support, yes);
