@@ -1,7 +1,6 @@
 #include "../filezilla.h"
 #include "../directorylistingparser.h"
 #include "../engineprivate.h"
-#include "../iothread.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
 
@@ -16,6 +15,178 @@
 
 #include <assert.h>
 
+#ifndef FZ_WINDOWS
+#define HAVE_ASCII_TRANSFORM 1
+#endif
+
+#if HAVE_ASCII_TRANSFORM
+namespace {
+class ascii_writer final : public writer_base, public fz::event_handler
+{
+public:
+	ascii_writer(CFileZillaEnginePrivate& engine, fz::event_handler * handler, std::unique_ptr<writer_base> && writer)
+		: writer_base(writer->name(), engine, handler, true)
+		, fz::event_handler(engine.event_loop_)
+		, writer_(std::move(writer))
+	{
+		writer_->set_handler(this);
+	}
+
+	~ascii_writer() {
+		writer_.reset();
+		remove_handler();
+	}
+
+	virtual uint64_t size() const override
+	{
+		return writer_->size();
+	}
+
+	virtual get_write_buffer_result get_write_buffer(fz::nonowning_buffer & last_written) override
+	{
+		transform(last_written);
+		get_write_buffer_result r = writer_->get_write_buffer(last_written);
+		if (r.type_ == aio_result::ok) {
+			if (was_cr_) {
+				r.buffer_.append('\r');
+				was_cr_ = false;
+			}
+		}
+		return r;
+	}
+
+	virtual aio_result finalize(fz::nonowning_buffer & last_written) override
+	{
+		transform(last_written);
+		if (was_cr_) {
+			last_written.append('\r');
+			was_cr_ = false;
+		}
+		return writer_->finalize(last_written);
+	}
+
+private:
+	void transform(fz::nonowning_buffer & b)
+	{
+		if (!b.empty()) {
+			auto * const start = b.get();
+			auto *p = start;
+			auto *q = start;
+			auto * const end = start + b.size();
+			while (p != end) {
+				auto c = *(p++);
+				if (c == '\r') {
+					was_cr_ = true;
+				}
+				else if (c == '\n') {
+					was_cr_ = false;
+					*(q++) = c;
+				}
+				else {
+					if (was_cr_) {
+						*(q++) = '\r';
+						was_cr_ = false;
+					}
+					*(q++) = c;
+				}
+			}
+			b.resize(q - start);
+		}
+	}
+
+	std::unique_ptr<writer_base> writer_;
+
+	virtual void operator()(fz::event_base const&) override
+	{
+		if (handler_) {
+			handler_->operator()(write_ready_event(this));
+		}
+	}
+
+	bool was_cr_{};
+};
+
+class ascii_reader final : public reader_base, public fz::event_handler
+{
+public:
+	ascii_reader(CFileZillaEnginePrivate& engine, fz::event_handler * handler, std::unique_ptr<reader_base> && reader)
+		: reader_base(reader->name(), engine, handler)
+		, fz::event_handler(engine.event_loop_)
+		, reader_(std::move(reader))
+	{
+		reader_->set_handler(this);
+		size_ = reader_->size();
+	}
+
+	~ascii_reader()
+	{
+		reader_.reset();
+		remove_handler();
+	}
+
+	virtual aio_result seek(uint64_t, uint64_t = aio_base::nosize) override
+	{
+		return aio_result::error;
+	}
+
+	virtual read_result read() override
+	{
+		read_result ret = reader_->read();
+		if (ret.type_ != aio_result::ok) {
+			return ret;
+		}
+
+
+		buffer_.clear();
+
+		// In the worst case, length will doubled: If reading
+		// only LFs from the file
+		auto * q = buffer_.get(ret.buffer_.size() * 2);
+
+		auto const * p = ret.buffer_.get();
+		auto const * end = ret.buffer_.get() + ret.buffer_.size();
+
+
+		// Convert all stand-alone LFs into CRLF pairs.
+		while (p != end) {
+			auto const& c = *(p++);
+			if (c == '\n') {
+				if (!was_cr_) {
+					*(q++) = '\r';
+				}
+				was_cr_ = false;
+			}
+			else if (c == '\r') {
+				was_cr_ = true;
+			}
+			else {
+				was_cr_ = false;
+			}
+
+			*(q++) = c;
+		}
+
+		buffer_.add(q - buffer_.get());
+		ret.buffer_ = fz::nonowning_buffer(buffer_.get(), buffer_.capacity(), buffer_.size());
+		return ret;
+	}
+
+	std::unique_ptr<reader_base> reader_;
+
+	virtual void operator()(fz::event_base const&) override
+	{
+		if (handler_) {
+			handler_->operator()(read_ready_event(this));
+		}
+	}
+
+private:
+	fz::buffer buffer_;
+	bool was_cr_{};
+};
+}
+#endif
+
 CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode)
 : fz::event_handler(controlSocket.event_loop_)
 , engine_(engine)
@@ -31,14 +202,38 @@ CTransferSocket::~CTransferSocket()
 		m_transferEndReason = TransferEndReason::successful;
 	}
 	ResetSocket();
+	
+	reader_.reset();
+	writer_.reset();
+}
 
-	if (m_transferMode == TransferMode::upload || m_transferMode == TransferMode::download) {
-		if (ioThread_) {
-			if (m_transferMode == TransferMode::download) {
-				FinalizeWrite();
-			}
-			ioThread_->SetEventHandler(nullptr);
-		}
+void CTransferSocket::set_reader(std::unique_ptr<reader_base> && reader, bool ascii)
+{
+#if HAVE_ASCII_TRANSFORM
+	if (ascii) {
+		reader_ = std::make_unique<ascii_reader>(engine_, this, std::move(reader));
+	}
+	else
+#endif
+	{
+		(void)ascii;
+		reader_ = std::move(reader);
+		reader_->set_handler(this);
+	}
+}
+
+void CTransferSocket::set_writer(std::unique_ptr<writer_base> && writer, bool ascii)
+{
+#if HAVE_ASCII_TRANSFORM
+	if (ascii) {
+		writer_ = std::make_unique<ascii_writer>(engine_, this, std::move(writer));
+	}
+	else
+#endif
+	{
+		(void)ascii;
+		writer_ = std::move(writer);
+		writer_->set_handler(this);
 	}
 }
 
@@ -52,6 +247,7 @@ void CTransferSocket::ResetSocket()
 	proxy_layer_.reset();
 	ratelimit_layer_.reset();
 	socket_.reset();
+	buffer_.reset();
 }
 
 std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
@@ -262,7 +458,8 @@ void CTransferSocket::OnReceive()
 					return;
 				}
 
-				numread = active_layer_->read(m_pTransferBuffer, m_transferBufferLen, error);
+				size_t to_read = buffer_.capacity() - buffer_.size();
+				numread = active_layer_->read(buffer_.get(to_read), static_cast<unsigned int>(to_read), error);
 				if (numread <= 0) {
 					break;
 				}
@@ -272,10 +469,8 @@ void CTransferSocket::OnReceive()
 					m_madeProgress = 2;
 					engine_.transfer_status_.SetMadeProgress();
 				}
-				engine_.transfer_status_.Update(numread);
 
-				m_pTransferBuffer += numread;
-				m_transferBufferLen -= numread;
+				buffer_.add(static_cast<size_t>(numread));
 			}
 
 			if (numread < 0) {
@@ -294,9 +489,9 @@ void CTransferSocket::OnReceive()
 		}
 		else if (m_transferMode == TransferMode::resumetest) {
 			for (;;) {
-				char buffer[2];
+				char tmp[2];
 				int error;
-				int numread = active_layer_->read(buffer, 2, error);
+				int numread = active_layer_->read(tmp, 2, error);
 				if (numread < 0) {
 					if (error != EAGAIN) {
 						controlSocket_.log(logmsg::error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
@@ -306,19 +501,19 @@ void CTransferSocket::OnReceive()
 				}
 
 				if (!numread) {
-					if (m_transferBufferLen == 1) {
+					if (resumetest_ == 1) {
 						TransferEnd(TransferEndReason::successful);
 					}
 					else {
-						controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
+						controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", resumetest_);
 						TransferEnd(TransferEndReason::failed_resumetest);
 					}
 					return;
 				}
-				m_transferBufferLen += numread;
+				resumetest_ += numread;
 
-				if (m_transferBufferLen > 1) {
-					controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
+				if (resumetest_ > 1) {
+					controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", resumetest_);
 					TransferEnd(TransferEndReason::failed_resumetest);
 					return;
 				}
@@ -378,7 +573,7 @@ void CTransferSocket::OnSend()
 			return;
 		}
 
-		written = active_layer_->write(m_pTransferBuffer, m_transferBufferLen, error);
+		written = active_layer_->write(buffer_.get(), static_cast<int>(buffer_.size()), error);
 		if (written <= 0) {
 			break;
 		}
@@ -391,8 +586,7 @@ void CTransferSocket::OnSend()
 		}
 		engine_.transfer_status_.Update(written);
 
-		m_pTransferBuffer += written;
-		m_transferBufferLen -= written;
+		buffer_.consume(written);
 	}
 
 	if (written < 0) {
@@ -515,11 +709,6 @@ void CTransferSocket::SetActive()
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
-	if (m_transferMode == TransferMode::download || m_transferMode == TransferMode::upload) {
-		if (ioThread_) {
-			ioThread_->SetEventHandler(this);
-		}
-	}
 
 	m_bActive = true;
 	if (!socket_) {
@@ -613,25 +802,17 @@ std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer()
 
 bool CTransferSocket::CheckGetNextWriteBuffer()
 {
-	if (!m_transferBufferLen) {
-		int res = ioThread_->GetNextWriteBuffer(&m_pTransferBuffer);
-
-		if (res == IO_Again) {
+	if (buffer_.size() >= buffer_.capacity()) {
+		auto res = writer_->get_write_buffer(buffer_);
+		if (res == aio_result::wait) {
 			return false;
 		}
-		else if (res == IO_Error) {
-			std::wstring error = ioThread_->GetError();
-			if (error.empty()) {
-				controlSocket_.log(logmsg::error, _("Can't write data to file."));
-			}
-			else {
-				controlSocket_.log(logmsg::error, _("Can't write data to file: %s"), error);
-			}
+		else if (res == aio_result::error) {
 			TransferEnd(TransferEndReason::transfer_failure_critical);
 			return false;
 		}
 
-		m_transferBufferLen = BUFFERSIZE;
+		buffer_ = res.buffer_;
 	}
 
 	return true;
@@ -639,32 +820,43 @@ bool CTransferSocket::CheckGetNextWriteBuffer()
 
 bool CTransferSocket::CheckGetNextReadBuffer()
 {
-	if (!m_transferBufferLen) {
-		int res = ioThread_->GetNextReadBuffer(&m_pTransferBuffer);
-		if (res == IO_Again) {
+	if (buffer_.empty()) {
+		read_result res = reader_->read();
+		if (res == aio_result::wait) {
 			return false;
 		}
-		else if (res == IO_Error) {
-			controlSocket_.log(logmsg::error, _("Can't read from file"));
-			TransferEnd(TransferEndReason::transfer_failure);
+		else if (res == aio_result::error) {
+			TransferEnd(TransferEndReason::transfer_failure_critical);
 			return false;
 		}
-		else if (res == IO_Success) {
-			int res = active_layer_->shutdown();
-			if (res && res != EAGAIN) {
+
+		buffer_ = res.buffer_;
+		if (buffer_.empty()) {
+			int r = active_layer_->shutdown();
+			if (r && r != EAGAIN) {
 				TransferEnd(TransferEndReason::transfer_failure);
 				return false;
 			}
 			TransferEnd(TransferEndReason::successful);
 			return false;
 		}
-		m_transferBufferLen = res;
 	}
 
 	return true;
 }
 
-void CTransferSocket::OnIOThreadEvent()
+void CTransferSocket::OnInput(reader_base*)
+{
+	if (!m_bActive || m_transferEndReason != TransferEndReason::none) {
+		return;
+	}
+
+	if (m_transferMode == TransferMode::upload) {
+		OnSend();
+	}
+}
+
+void CTransferSocket::OnWrite(writer_base*)
 {
 	if (!m_bActive || m_transferEndReason != TransferEndReason::none) {
 		return;
@@ -673,33 +865,26 @@ void CTransferSocket::OnIOThreadEvent()
 	if (m_transferMode == TransferMode::download) {
 		OnReceive();
 	}
-	else if (m_transferMode == TransferMode::upload) {
-		OnSend();
-	}
 }
 
 void CTransferSocket::FinalizeWrite()
 {
-	bool res = ioThread_->Finalize(BUFFERSIZE - m_transferBufferLen);
-	m_transferBufferLen = BUFFERSIZE;
-
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
-	if (res) {
+	auto res = writer_->finalize(buffer_);
+	if (res == aio_result::wait) {
+		return;
+	}
+
+	if (res == aio_result::ok) {
 		TransferEnd(TransferEndReason::successful);
 	}
 	else {
-		std::wstring error = ioThread_->GetError();
-		if (error.empty()) {
-			controlSocket_.log(logmsg::error, _("Can't write data to file."));
-		}
-		else {
-			controlSocket_.log(logmsg::error, _("Can't write data to file: %s"), error);
-		}
 		TransferEnd(TransferEndReason::transfer_failure_critical);
 	}
+
 }
 
 void CTransferSocket::TriggerPostponedEvents()
@@ -734,9 +919,10 @@ void CTransferSocket::SetSocketBufferSizes(fz::socket_base& socket)
 
 void CTransferSocket::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<fz::socket_event, CIOThreadEvent, fz::timer_event>(ev, this,
+	fz::dispatch<fz::socket_event, read_ready_event, write_ready_event, fz::timer_event>(ev, this,
 		&CTransferSocket::OnSocketEvent,
-		&CTransferSocket::OnIOThreadEvent,
+		&CTransferSocket::OnInput,
+		&CTransferSocket::OnWrite,
 		&CTransferSocket::OnTimer);
 }
 
