@@ -1,21 +1,21 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "filetransfer.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <assert.h>
 #include <string.h>
 
 enum filetransferStates
 {
 	filetransfer_init = 0,
-	filetransfer_waitfileexists,
 	filetransfer_transfer,
 	filetransfer_waittransfer
 };
 
-CHttpFileTransferOpData::CHttpFileTransferOpData(CHttpControlSocket & controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
-	: CFileTransferOpData(L"CHttpFileTransferOpData", is_download, local_file, remote_file, remote_path, settings)
+CHttpFileTransferOpData::CHttpFileTransferOpData(CHttpControlSocket & controlSocket, CFileTransferCommand const& cmd)
+	: CFileTransferOpData(L"CHttpFileTransferOpData", cmd)
 	, CHttpOpData(controlSocket)
 {
 	rr_.request_.uri_ = fz::uri(fz::to_utf8(currentServer_.Format(ServerFormat::url)) + fz::percent_encode(fz::to_utf8(remotePath_.FormatFilename(remoteFile_)), true));
@@ -23,12 +23,13 @@ CHttpFileTransferOpData::CHttpFileTransferOpData(CHttpControlSocket & controlSoc
 
 }
 
-CHttpFileTransferOpData::CHttpFileTransferOpData(CHttpControlSocket & controlSocket, fz::uri const& uri, std::string const& verb, std::string const& body)
-	: CFileTransferOpData(L"CHttpFileTransferOpData", true, std::wstring(), std::wstring(), CServerPath(), CFileTransferCommand::t_transferSettings())
+CHttpFileTransferOpData::CHttpFileTransferOpData(CHttpControlSocket & controlSocket, fz::uri const& uri, std::string const& verb, reader_factory_holder const& reader, writer_factory_holder const& writer)
+	: CFileTransferOpData(L"CHttpFileTransferOpData", CFileTransferCommand(writer_factory_holder(), CServerPath(), std::wstring(), transfer_flags::download))
 	, CHttpOpData(controlSocket)
 {
+	reader_factory_ = reader;
+	writer_factory_ = writer;
 	rr_.request_.uri_ = uri;
-	rr_.request_.body_ = std::make_unique<simple_body>(body);
 	rr_.request_.verb_ = verb;
 }
 
@@ -37,7 +38,7 @@ int CHttpFileTransferOpData::Send()
 {
 	switch (opState) {
 	case filetransfer_init:
-		if (!download_) {
+		if (!download()) {
 			return FZ_REPLY_NOTSUPPORTED;
 		}
 
@@ -46,9 +47,19 @@ int CHttpFileTransferOpData::Send()
 			return FZ_REPLY_ERROR;
 		}
 
-		opState = filetransfer_waitfileexists;
-		if (!localFile_.empty()) {
-			localFileSize_ = fz::local_filesys::get_size(fz::to_native(localFile_));
+		if (reader_factory_) {
+			rr_.request_.body_ = reader_factory_.open(0, engine_, nullptr, aio_base::shm_flag_none);
+			if (!rr_.request_.body_) {
+				return FZ_REPLY_CRITICALERROR;
+			}
+		}
+
+		opState = filetransfer_transfer;
+		if (writer_factory_) {
+			auto s = writer_factory_.size();
+			if (s != aio_base::nosize) {
+				localFileSize_ = static_cast<int64_t>(s);
+			}
 
 			int res = controlSocket_.CheckOverwriteFile();
 			if (res != FZ_REPLY_OK) {
@@ -56,23 +67,12 @@ int CHttpFileTransferOpData::Send()
 			}
 		}
 		return FZ_REPLY_CONTINUE;
-	case filetransfer_waitfileexists:
-		if (!localFile_.empty()) {
-			int res = OpenFile();
-			if (res != FZ_REPLY_OK) {
-				return res;
-			}
-		}
-		opState = filetransfer_transfer;
-		return FZ_REPLY_CONTINUE;
 	case filetransfer_transfer:
 		if (resume_) {
 			rr_.request_.headers_["Range"] = fz::sprintf("bytes=%d-", localFileSize_);
 		}
 
-		rr_.response_ = HttpResponse();
 		rr_.response_.on_header_ = [this](auto const&) { return this->OnHeader(); };
-		rr_.response_.on_data_ = [this](auto data, auto len) { return this->OnData(data, len); };
 
 		opState = filetransfer_waittransfer;
 		controlSocket_.Request(make_simple_rr(&rr_));
@@ -84,51 +84,12 @@ int CHttpFileTransferOpData::Send()
 	return FZ_REPLY_INTERNALERROR;
 }
 
-int CHttpFileTransferOpData::OpenFile()
-{
-	log(logmsg::debug_verbose, L"CHttpFileTransferOpData::OpenFile");
-	if (file_.opened()) {
-		if (transferSettings_.fsync) {
-			file_.fsync();
-		}
-		file_.close();
-	}
-
-	controlSocket_.CreateLocalDir(localFile_);
-
-	if (!file_.open(fz::to_native(localFile_),
-		download_ ? fz::file::writing : fz::file::reading,
-		fz::file::existing))
-	{
-		log(logmsg::error, _("Failed to open \"%s\" for writing"), localFile_);
-		return FZ_REPLY_ERROR;
-	}
-
-	assert(download_);
-	int64_t end = file_.seek(0, fz::file::end);
-	if (end < 0) {
-		log(logmsg::error, _("Could not seek to the end of the file"));
-		return FZ_REPLY_ERROR;
-	}
-	if (!end) {
-		resume_ = false;
-	}
-	localFileSize_ = fz::local_filesys::get_size(fz::to_native(localFile_));
-	return FZ_REPLY_OK;
-}
-
 int CHttpFileTransferOpData::OnHeader()
 {
 	log(logmsg::debug_verbose, L"CHttpFileTransferOpData::OnHeader");
 
 	if (rr_.response_.code_ == 416 && resume_) {
-		assert(file_.opened());
-		if (file_.seek(0, fz::file::begin) != 0) {
-			log(logmsg::error, _("Could not seek to the beginning of the file"));
-			return FZ_REPLY_ERROR;
-		}
 		resume_ = false;
-
 		opState = filetransfer_transfer;
 		return FZ_REPLY_ERROR;
 	}
@@ -181,14 +142,17 @@ int CHttpFileTransferOpData::OnHeader()
 
 	// Check if the server disallowed resume
 	if (resume_ && rr_.response_.code_ != 206) {
-		assert(file_.opened());
-		if (file_.seek(0, fz::file::begin) != 0) {
-			log(logmsg::error, _("Could not seek to the beginning of the file"));
-			return FZ_REPLY_ERROR;
-		}
 		resume_ = false;
 	}
 
+	if (writer_factory_) {
+		auto writer = writer_factory_.open(resume_ ? localFileSize_ : 0, engine_, &controlSocket_, aio_base::shm_flag_none);
+		if (!writer) {
+			return FZ_REPLY_CRITICALERROR;
+		}
+		rr_.response_.writer_ = std::move(writer);
+	}
+
 	int64_t totalSize = fz::to_integral<int64_t>(rr_.response_.get_header("Content-Length"), -1);
 	if (totalSize == -1) {
 		if (remoteFileSize_ != -1) {
@@ -204,45 +168,11 @@ int CHttpFileTransferOpData::OnHeader()
 	return FZ_REPLY_CONTINUE;
 }
 
-int CHttpFileTransferOpData::OnData(unsigned char const* data, unsigned int len)
-{
-	if (opState != filetransfer_waittransfer) {
-		return FZ_REPLY_INTERNALERROR;
-	}
-
-	if (localFile_.empty()) {
-		char* q = new char[len];
-		memcpy(q, data, len);
-		engine_.AddNotification(new CDataNotification(q, len));
-	}
-	else {
-		assert(file_.opened());
-
-		auto write = static_cast<int64_t>(len);
-		if (file_.write(data, write) != write) {
-			log(logmsg::error, _("Failed to write to file %s"), localFile_);
-			return FZ_REPLY_ERROR;
-		}
-	}
-
-	engine_.transfer_status_.Update(len);
-
-	return FZ_REPLY_CONTINUE;
-}
-
 int CHttpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 {
 	if (opState == filetransfer_transfer) {
 		return FZ_REPLY_CONTINUE;
 	}
 
-	if (opState == filetransfer_waittransfer) {
-		if (file_.opened()) {
-			if (transferSettings_.fsync) {
-				file_.fsync();
-			}
-		}
-	}
-
 	return prevResult;
 }
