@@ -10,21 +10,17 @@ class CServerPath;
 class CHttpFileTransferOpData final : public CFileTransferOpData, public CHttpOpData
 {
 public:
-	CHttpFileTransferOpData(CHttpControlSocket & controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings);
-	CHttpFileTransferOpData(CHttpControlSocket & controlSocket, fz::uri const& uri, std::string const& verb, std::string const& body);
+	CHttpFileTransferOpData(CHttpControlSocket & controlSocket, CFileTransferCommand const&);
+	CHttpFileTransferOpData(CHttpControlSocket & controlSocket, fz::uri const& uri, std::string const& verb, reader_factory_holder const& reader, writer_factory_holder const& writer);
 
 	virtual int Send() override;
 	virtual int ParseResponse() override { return FZ_REPLY_INTERNALERROR; }
 	virtual int SubcommandResult(int prevResult, COpData const& previousOperation) override;
 
 private:
-	int OpenFile();
-
 	int OnHeader();
-	int OnData(unsigned char const* data, unsigned int len);
 
 	HttpRequestResponse rr_;
-	fz::file file_;
 
 	int redirectCount_{};
 };
