@@ -13,74 +13,6 @@ auto const http_request = Command::private1;
 auto const http_connect = Command::private2;
 }
 
-class request_body
-{
-public:
-	virtual ~request_body() = default;
-
-	virtual uint64_t size() const = 0;
-
-	// data_request must write up to len bytes into the provided buffer,
-	// and update len with the amount written.
-	// Must return FZ_REPLY_CONTINUE or FZ_REPLY_ERROR
-	virtual int data_request(unsigned char* data, unsigned int & len) = 0;
-
-	// reset() must return FZ_REPLY_CONTINUE or FZ_REPLY_ERROR
-	virtual int rewind() = 0;
-};
-
-class simple_body final : public request_body
-{
-public:
-	simple_body() = default;
-	explicit simple_body(std::string const& body)
-		: body_(body)
-	{}
-
-	void set(std::string const& body)
-	{
-		body_ = body;
-		written_ = 0;
-	}
-
-	virtual int data_request(unsigned char* data, unsigned int & len) override;
-
-	virtual int rewind() override
-	{
-		written_ = 0;
-		return FZ_REPLY_CONTINUE;
-	}
-
-	virtual uint64_t size() const override { return body_.size(); }
-
-private:
-	size_t written_{};
-	std::string body_;
-};
-
-
-class file_body final : public request_body
-{
-public:
-	file_body(fz::file & file, uint64_t start, uint64_t size, fz::logger_interface & logger);
-
-	virtual uint64_t size() const override { return size_; }
-
-	virtual int data_request(unsigned char* data, unsigned int & len) override;
-
-	virtual int rewind() override;
-
-	std::function<void(int64_t)> progress_callback_;
-
-protected:
-	fz::file & file_;
-	uint64_t start_{};
-	uint64_t written_{};
-	uint64_t size_{};
-
-	fz::logger_interface & logger_;
-};
-
 #define HEADER_NAME_CONTENT_LENGTH "Content-Length"
 #define HEADER_NAME_CONTENT_TYPE "Content-Type"
 class WithHeaders
@@ -98,11 +30,6 @@ public:
 		return result;
 	}
 
-	void set_content_length(int64_t length)
-	{
-		headers_[HEADER_NAME_CONTENT_LENGTH] = fz::to_string(length);
-	}
-
 	void set_content_type(std::string const& content_type)
 	{
 		headers_[HEADER_NAME_CONTENT_TYPE] = content_type;
@@ -119,7 +46,14 @@ public:
 
 	bool keep_alive() const
 	{
-		return fz::str_tolower_ascii(get_header("Connection")) != "close";
+		auto h = fz::str_tolower_ascii(get_header("Connection"));
+		auto tokens = fz::strtok_view(h, ", ");
+		for (auto const& token : tokens) {
+			if (token == "close") {
+				return false;
+			}
+		}
+		return true;
 	}
 
 	HttpHeaders headers_;
@@ -133,11 +67,15 @@ public:
 
 	enum flags {
 		flag_sent_header = 0x01,
-		flag_sent_body = 0x02
+		flag_sent_body = 0x02,
+		flag_update_transferstatus = 0x04
 	};
 	int flags_{};
 
-	std::unique_ptr<request_body> body_;
+	std::unique_ptr<reader_base> body_;
+	fz::nonowning_buffer body_buffer_;
+
+	uint64_t update_content_length();
 
 	virtual int reset();
 };
@@ -155,7 +93,9 @@ public:
 class HttpResponse : public WithHeaders
 {
 public:
-	virtual ~HttpResponse() {};
+	// Use a writer if you need more.
+	static size_t constexpr max_simple_body_size{1024 * 1024 * 16};
+
 	unsigned int code_{};
 
 	enum flags {
@@ -169,7 +109,7 @@ public:
 
 	bool got_code() const { return flags_ & flag_got_code; }
 	bool got_header() const { return flags_ & flag_got_header; }
-	bool got_body() const { return flags_ & flag_got_body; }
+	bool got_body() const { return (flags_ & (flag_got_body | flag_no_body | flag_ignore_body)) == flag_got_body; }
 	bool no_body() const { return flags_ & flag_no_body; }
 
 	// Called once the complete header has been received.
@@ -179,12 +119,11 @@ public:
 	//   FZ_REPLY_ERROR: Abort connection
 	std::function<int(std::shared_ptr<HttpRequestResponseInterface> const&)> on_header_;
 
-	// Is only called after the on_header_ callback.
-	// Callback must return FZ_REPLY_CONTINUE or FZ_REPLY_ERROR
-	std::function<int(unsigned char const* data, unsigned int len)> on_data_;
+	// Writer isn't called if !success()
+	std::unique_ptr<writer_base> writer_;
 
-	// Called if !success && got_body
-	std::function<int(unsigned char const* data, unsigned int len)> on_error_data_;
+	// Holds error body and success body if there is no writer.
+	fz::buffer body_;
 
 	bool success() const {
 		return code_ >= 200 && code_ < 300;
@@ -250,9 +189,7 @@ public:
 
 protected:
 	virtual void Connect(CServer const& server, Credentials const& credentials) override;
-	virtual void FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-		std::wstring const& remoteFile, bool download,
-		CFileTransferCommand::t_transferSettings const& transferSettings) override;
+	virtual void FileTransfer(CFileTransferCommand const& cmd) override;
 
 	void Request(std::shared_ptr<HttpRequestResponseInterface> const& request);
 	void Request(std::deque<std::shared_ptr<HttpRequestResponseInterface>> && requests);
@@ -282,6 +219,9 @@ protected:
 	virtual void OnReceive() override;
 	virtual int OnSend() override;
 
+	virtual void operator()(fz::event_base const& ev) override;
+	void OnWriteReady(writer_base* writer);
+
 	virtual void ResetSocket() override;
 
 	friend class CProtocolOpData<CHttpControlSocket>;
