@@ -25,7 +25,7 @@ public:
 		remove_handler();
 	}
 
-	virtual void operator()(fz::event_base const& ev)
+	virtual void operator()(fz::event_base const& ev) override
 	{
 		if (fz::dispatch<fz::certificate_verification_event>(ev, this, &CHttpInternalConnectOpData::OnVerifyCert)) {
 			return;
@@ -38,7 +38,7 @@ public:
 			return;
 		}
 
-		controlSocket_.SendAsyncRequest(new CCertificateNotification(std::move(info)));
+		controlSocket_.SendAsyncRequest(std::make_unique<CCertificateNotification>(std::move(info)));
 	}
 
 	virtual int Send() override;
