@@ -1,19 +1,20 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "request.h"
 
-#include <string.h>
-
 #include <libfilezilla/encode.hpp>
 
+#include <assert.h>
+#include <string.h>
 
 CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::shared_ptr<HttpRequestResponseInterface> const& request)
 	: COpData(PrivCommand::http_request, L"CHttpRequestOpData")
 	, CHttpOpData(controlSocket)
+	, fz::event_handler(engine_.event_loop_)
 {
 	opState = request_init | request_reading;
 
-	request->request().flags_ = 0;
+	request->request().flags_ &= HttpRequest::flag_update_transferstatus;
 	request->response().flags_ = 0;
 
 	requests_.emplace_back(request);
@@ -22,15 +23,28 @@ CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::
 CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::deque<std::shared_ptr<HttpRequestResponseInterface>> && requests)
 	: COpData(PrivCommand::http_request, L"CHttpRequestOpData")
 	, CHttpOpData(controlSocket)
+	, fz::event_handler(engine_.event_loop_)
 	, requests_(requests)
 {
 	for (auto & rr : requests_) {
-		rr->request().flags_ = 0;
+		rr->request().flags_ &= HttpRequest::flag_update_transferstatus;
 		rr->response().flags_ = 0;
 	}
 	opState = request_init | request_reading;
 }
 
+CHttpRequestOpData::~CHttpRequestOpData()
+{
+	for (auto & rr : requests_) {
+		if (rr) {
+			if (rr->request().body_) {
+				rr->request().body_->set_handler(nullptr);
+			}
+		}
+	}
+	remove_handler();
+}
+
 void CHttpRequestOpData::AddRequest(std::shared_ptr<HttpRequestResponseInterface> const& rr)
 {
 	if (!(opState & request_send_mask)) {
@@ -53,7 +67,7 @@ void CHttpRequestOpData::AddRequest(std::shared_ptr<HttpRequestResponseInterface
 			}
 		}
 	}
-	rr->request().flags_ = 0;
+	rr->request().flags_ &= HttpRequest::flag_update_transferstatus;
 	rr->response().flags_ = 0;
 	requests_.push_back(rr);
 }
@@ -69,10 +83,23 @@ int CHttpRequestOpData::Send()
 		auto & rr = *requests_[send_pos_];
 		auto & req = rr.request();
 
+		// Check backoff
+		fz::duration backoff = controlSocket_.throttler_.get_throttle(req.uri_.host_);
+		if (backoff) {
+			if (backoff >= fz::duration::from_seconds(30)) {
+				log(logmsg::status, _("Server instructed us to wait %d seconds before sending next request"), backoff.get_seconds());
+			}
+			controlSocket_.Sleep(backoff);
+			return FZ_REPLY_CONTINUE;
+		}
+
 		int res = req.reset();
 		if (res != FZ_REPLY_CONTINUE) {
 			return res;
 		}
+		if (req.body_) {
+			req.body_->set_handler(this);
+		}
 
 		res = rr.response().reset();
 		if (res != FZ_REPLY_CONTINUE) {
@@ -135,29 +162,18 @@ int CHttpRequestOpData::Send()
 		if (send_pos_ >= requests_.size()) {
 			opState &= ~request_send;
 		}
+		else if (!requests_[send_pos_]) {
+			log(logmsg::debug_warning, L"Null request in request_send state.");
+			return FZ_REPLY_INTERNALERROR;
+		}
 		else {
 			auto & req = requests_[send_pos_]->request();
 			if (!(req.flags_ & HttpRequest::flag_sent_header)) {
-				if (req.body_) {
-					req.set_content_length(req.body_->size());
-				}
-				else {
-					if (req.verb_ == "GET" || req.verb_ == "HEAD" || req.verb_ == "OPTIONS") {
-						req.headers_.erase("Content-Length");
-					}
-					else {
-						req.set_content_length(0);
-					}
-				}
+				dataToSend_ = req.update_content_length();
 
-				auto const cl = req.get_header("Content-Length");
-				if (!cl.empty()) {
-					int64_t requestContentLength = fz::to_integral<int64_t>(cl, -1);
-					if (requestContentLength < 0) {
-						log(logmsg::error, _("Malformed request header: %s"), _("Invalid Content-Length"));
-						return FZ_REPLY_INTERNALERROR;
-					}
-					dataToSend_ = static_cast<uint64_t>(requestContentLength);
+				if (dataToSend_ == aio_base::nosize) {
+					log(logmsg::error, _("Malformed request header: %s"), _("Invalid Content-Length"));
+					return FZ_REPLY_INTERNALERROR;
 				}
 
 				// Assemble request and headers
@@ -211,27 +227,7 @@ int CHttpRequestOpData::Send()
 				return result;
 			}
 			else {
-				int const chunkSize = 65536;
-
-				while (dataToSend_ || controlSocket_.send_buffer_) {
-					if (!controlSocket_.send_buffer_) {
-						unsigned int len = chunkSize;
-						if (chunkSize > dataToSend_) {
-							len = static_cast<unsigned int>(dataToSend_);
-						}
-						int res = req.body_->data_request(controlSocket_.send_buffer_.get(len), len);
-						if (res != FZ_REPLY_CONTINUE) {
-							return res;
-						}
-						if (len > dataToSend_) {
-							log(logmsg::debug_warning, L"req.body_ returned too much data");
-							return FZ_REPLY_INTERNALERROR;
-						}
-
-						controlSocket_.send_buffer_.add(len);
-						dataToSend_ -= len;
-					}
-
+				while (controlSocket_.send_buffer_) {
 					int error;
 					int written = controlSocket_.active_layer_->write(controlSocket_.send_buffer_.get(), controlSocket_.send_buffer_.size(), error);
 					if (written < 0) {
@@ -242,14 +238,54 @@ int CHttpRequestOpData::Send()
 						}
 						return FZ_REPLY_WOULDBLOCK;
 					}
-
-					if (written) {
+					else if (written) {
 						controlSocket_.SetActive(CFileZillaEngine::send);
-
 						controlSocket_.send_buffer_.consume(static_cast<size_t>(written));
 					}
 				}
 
+				while (dataToSend_) {
+
+					if (req.body_buffer_.empty()) {
+						read_result r = req.body_->read();
+						if (r.type_ == aio_result::wait) {
+							return FZ_REPLY_WOULDBLOCK;
+						}
+						else if (r.type_ == aio_result::error) {
+							return FZ_REPLY_ERROR;
+						}
+						req.body_buffer_ = r.buffer_;
+
+						if (req.body_buffer_.empty() && dataToSend_) {
+							log(logmsg::error, _("Unexpected end-of-file on '%s'"), req.body_->name());
+							return FZ_REPLY_ERROR;
+						}
+						else if (req.body_buffer_.size() > dataToSend_) {
+							log(logmsg::error, _("Excess data read from '%s'"), req.body_->name());
+							return FZ_REPLY_ERROR;
+						}
+					}
+
+					int error;
+					int written = controlSocket_.active_layer_->write(req.body_buffer_.get(), req.body_buffer_.size(), error);
+					if (written < 0) {
+						if (error != EAGAIN) {
+							log(logmsg::error, _("Could not write to socket: %s"), fz::socket_error_description(error));
+							log(logmsg::error, _("Disconnected from server"));
+							return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+						}
+						return FZ_REPLY_WOULDBLOCK;
+					}
+					else if (written) {
+						controlSocket_.SetActive(CFileZillaEngine::send);
+						req.body_buffer_.consume(static_cast<size_t>(written));
+						dataToSend_ -= written;
+						if (req.flags_ & HttpRequest::flag_update_transferstatus) {
+							engine_.transfer_status_.Update(written);
+						}
+					}
+				}
+
 				log(logmsg::debug_info, "Finished sending request body");
 
 				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay, true);
@@ -292,13 +328,36 @@ int CHttpRequestOpData::SubcommandResult(int, COpData const&)
 	return FZ_REPLY_CONTINUE;
 }
 
-int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
+int CHttpRequestOpData::FinalizeResponseBody()
+{
+	auto & shared_response = requests_.front();
+	if (shared_response) {
+		auto & response = shared_response->response();
+		if (!(response.flags_ & (HttpResponse::flag_ignore_body | HttpResponse::flag_no_body))) {
+			response.flags_ |= HttpResponse::flag_got_body;
+			if (response.success() && response.writer_) {
+				auto r = response.writer_->finalize(read_state_.writer_buffer_);
+				switch (r) {
+				case aio_result::ok:
+					return FZ_REPLY_OK;
+				case aio_result::wait:
+					return FZ_REPLY_WOULDBLOCK;
+				default:
+					return FZ_REPLY_ERROR;
+				}
+			}
+		}
+	}
+	return FZ_REPLY_OK;
+}
+
+int CHttpRequestOpData::ParseReceiveBuffer()
 {
 	auto & shared_response = requests_.front();
 	if (shared_response) {
 		auto & request = shared_response->request();
 		if (!(request.flags_ & HttpRequest::flag_sent_header)) {
-			if (eof) {
+			if (read_state_.eof_) {
 				log(logmsg::debug_verbose, L"Socket closed before request got sent");
 				log(logmsg::error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
@@ -311,7 +370,7 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 		auto & response = shared_response->response();
 
 		if (!response.got_header()) {
-			if (eof) {
+			if (read_state_.eof_) {
 				log(logmsg::debug_verbose, L"Socket closed before headers got received");
 				log(logmsg::error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
@@ -319,10 +378,14 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 
 			return ParseHeader();
 		}
+
+		if (response.flags_ & HttpResponse::flag_got_body) {
+			return FinalizeResponseBody();
+		}
 	}
 
 	if (read_state_.transfer_encoding_ == chunked) {
-		if (eof) {
+		if (read_state_.eof_) {
 			log(logmsg::debug_verbose, L"Socket closed, chunk incomplete");
 			log(logmsg::error, _("Connection closed by server"));
 			return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
@@ -330,7 +393,7 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 		return ParseChunkedData();
 	}
 	else {
-		if (eof) {
+		if (read_state_.eof_) {
 			assert(recv_buffer_.empty());
 
 			if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ != read_state_.responseContentLength_) {
@@ -339,16 +402,13 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 			else {
-				if (shared_response) {
-					shared_response->response().flags_ |= HttpResponse::flag_got_body;
-				}
-				return FZ_REPLY_OK;
+				return FinalizeResponseBody();
 			}
 		}
 		else {
 			size_t size = recv_buffer_.size();
 			if (!size) {
-				return FZ_REPLY_WOULDBLOCK;
+				return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
 			}
 
 			if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ + static_cast<int64_t>(size) > read_state_.responseContentLength_) {
@@ -356,7 +416,7 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 			}
 
 			int res = ProcessData(recv_buffer_.get(), size);
-			recv_buffer_.consume(size);
+			recv_buffer_.consume(recv_buffer_.size() - size);
 			return res;
 		}
 	}
@@ -364,40 +424,60 @@ int CHttpRequestOpData::ParseReceiveBuffer(bool eof)
 	return FZ_REPLY_INTERNALERROR;
 }
 
-int CHttpRequestOpData::OnReceive()
+int CHttpRequestOpData::OnReceive(writer_base* writer)
+{
+	if (requests_.empty() || requests_.back()->response().writer_.get() != writer) {
+		log(logmsg::debug_warning, L"Stale writer event");
+		return FZ_REPLY_WOULDBLOCK;;
+	}
+	return OnReceive(true);
+}
+
+int CHttpRequestOpData::OnReceive(bool repeatedProcessing)
 {
 	while (controlSocket_.socket_) {
-		int error;
-		size_t const recv_size = 1024 * 64;
-		int read = controlSocket_.active_layer_->read(recv_buffer_.get(recv_size), recv_size, error);
-		if (read <= -1) {
-			if (error != EAGAIN) {
-				log(logmsg::error, _("Could not read from socket: %s"), fz::socket_error_description(error));
-				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
-			}
-			return FZ_REPLY_WOULDBLOCK;
+		if (repeatedProcessing) {
+			repeatedProcessing = false;
 		}
-		recv_buffer_.add(static_cast<size_t>(read));
+		else {
+			int error;
+			size_t const recv_size = 1024 * 64;
+			int read = controlSocket_.active_layer_->read(recv_buffer_.get(recv_size), recv_size, error);
+			if (read <= -1) {
+				if (error != EAGAIN) {
+					log(logmsg::error, _("Could not read from socket: %s"), fz::socket_error_description(error));
+					return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+				}
+				return FZ_REPLY_WOULDBLOCK;
+			}
+			recv_buffer_.add(static_cast<size_t>(read));
 
-		controlSocket_.SetActive(CFileZillaEngine::recv);
+			controlSocket_.SetActive(CFileZillaEngine::recv);
 
-		bool const eof = read == 0;
+			read_state_.eof_ = read == 0;
+		}
 
 		while (!requests_.empty()) {
 			assert(!requests_.empty());
 
-			int res = ParseReceiveBuffer(eof);
+			int res = ParseReceiveBuffer();
 			if (res == FZ_REPLY_WOULDBLOCK) {
+				return res;
+			}
+			else if (res == (FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE)) {
 				break;
 			}
 
 			if (res == FZ_REPLY_OK) {
 				log(logmsg::debug_info, L"Finished a response");
+				if (requests_.front() && requests_.front()->request().body_) {
+					requests_.front()->request().body_->set_handler(nullptr);
+				}
 				requests_.pop_front();
 				--send_pos_;
 
-				bool keep_alive = read_state_.keep_alive_;
-				if (!keep_alive || eof) {
+				bool const socket_done = read_state_.eof_ || !read_state_.keep_alive_;
+				if (socket_done) {
 					if (!recv_buffer_.empty()) {
 						log(logmsg::error, _("Malformed response: %s"), _("Server sent too much data."));
 						return FZ_REPLY_ERROR;
@@ -419,7 +499,7 @@ int CHttpRequestOpData::OnReceive()
 					return FZ_REPLY_OK;
 				}
 
-				if (!keep_alive || eof) {
+				if (socket_done) {
 					send_pos_ = 0;
 					opState = request_init | request_reading;
 					return FZ_REPLY_CONTINUE;
@@ -468,7 +548,7 @@ int CHttpRequestOpData::ParseHeader()
 				log(logmsg::error, _("Too long header line"));
 				return FZ_REPLY_ERROR;
 			}
-			return FZ_REPLY_WOULDBLOCK;
+			return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
 		}
 
 		std::wstring wline = fz::to_wstring_from_utf8(reinterpret_cast<char const*>(recv_buffer_.get()), i);
@@ -538,12 +618,12 @@ int CHttpRequestOpData::ParseHeader()
 		}
 	}
 
-	return FZ_REPLY_WOULDBLOCK;
+	return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
 }
 
 int CHttpRequestOpData::ProcessCompleteHeader()
 {
-	log(logmsg::debug_verbose, L"CHttpRequestOpData::ParseHeader()");
+	log(logmsg::debug_verbose, L"CHttpRequestOpData::ProcessCompleteHeader()");
 
 	auto & srr = requests_.front();
 	auto & request = srr->request();
@@ -571,6 +651,33 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 		return FZ_REPLY_ERROR;
 	}
 	
+	auto retry = response.get_header("Retry-After");
+	if (response.code_ >= 400 && !retry.empty()) {
+		// TODO: Retry-After for redirects
+		auto const now = fz::datetime::now();
+
+		fz::duration d;
+		int seconds = fz::to_integral<int>(retry, -1);
+		if (seconds > 0) {
+			d = fz::duration::from_seconds(seconds);
+		}
+		else {
+			fz::datetime t;
+			if (t.set_rfc822(retry)) {
+				if (t > now) {
+					d = t - now;
+				}
+			}
+		}
+
+		if (!d && response.code_ == 429) {
+			d = fz::duration::from_seconds(1);
+		}
+		if (d) {
+			log(logmsg::debug_verbose, "Got Retry-After with %d", d.get_seconds());
+			controlSocket_.throttler_.throttle(request.uri_.host_, now + d);
+		}
+	}
 
 	int64_t length{-1};
 	auto const cl = response.get_header("Content-Length");
@@ -595,21 +702,29 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 	if (response.on_header_) {
 		res = response.on_header_(srr);
 
-		if (res != FZ_REPLY_CONTINUE) {
-			if (res == FZ_REPLY_OK) {
-				// Clear the pointer, we no longer need the request to finish, all needed information is in read_state_
-				srr.reset();
-				res = FZ_REPLY_CONTINUE;
+		if (res == FZ_REPLY_CONTINUE) {
+			if (response.writer_) {
+				response.writer_->set_handler(&controlSocket_);
 			}
-			else {
-				response.flags_ |= HttpResponse::flag_ignore_body;
+
+		}
+		else {
+			if (res == FZ_REPLY_OK) {
+				if (!request.body_ || request.flags_ & HttpRequest::flag_sent_body) {
+					// Clear the pointer, we no longer need the request to finish, all needed information is in read_state_
+					srr.reset();
+				}
+				else {
+					response.flags_ |= HttpResponse::flag_ignore_body;
+				}
+				res = FZ_REPLY_CONTINUE;
 			}
 		}
 	}
 
 	if (res == FZ_REPLY_CONTINUE) {
 		if (!read_state_.responseContentLength_) {
-			res = FZ_REPLY_OK;
+			return FinalizeResponseBody();
 		}
 	}
 
@@ -624,12 +739,14 @@ int CHttpRequestOpData::ParseChunkedData()
 			if (read_state_.chunk_data_.size < recv_buffer_.size()) {
 				dataLen = static_cast<size_t>(read_state_.chunk_data_.size);
 			}
-			int res = ProcessData(recv_buffer_.get(), dataLen);
+			size_t remaining = dataLen;
+			int res = ProcessData(recv_buffer_.get(), remaining);
+			recv_buffer_.consume(dataLen - remaining);
+			read_state_.chunk_data_.size -= dataLen - remaining;
+
 			if (res != FZ_REPLY_CONTINUE) {
 				return res;
 			}
-			recv_buffer_.consume(dataLen);
-			read_state_.chunk_data_.size -= dataLen;
 
 			if (read_state_.chunk_data_.size == 0) {
 				read_state_.chunk_data_.terminateChunk = true;
@@ -674,12 +791,7 @@ int CHttpRequestOpData::ParseChunkedData()
 			if (!i) {
 				// We're done
 				recv_buffer_.consume(2);
-
-				auto & response = requests_.front();
-				if (response) {
-					response->response().flags_ |= HttpResponse::flag_got_body;
-				}
-				return FZ_REPLY_OK;
+				return FinalizeResponseBody();
 			}
 
 			// Ignore the trailer
@@ -712,39 +824,68 @@ int CHttpRequestOpData::ParseChunkedData()
 		recv_buffer_.consume(i + 2);
 	}
 
-	return FZ_REPLY_WOULDBLOCK;
+	return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
 }
 
-int CHttpRequestOpData::ProcessData(unsigned char* data, unsigned int len)
+int CHttpRequestOpData::ProcessData(unsigned char* data, size_t & remaining)
 {
-	read_state_.receivedData_ += len;
-
 	int res = FZ_REPLY_CONTINUE;
+	size_t initial = remaining;
 
 	auto & shared_response = requests_.front();
 	if (shared_response) {
 		auto & response = shared_response->response();
 
-		decltype(response.on_data_) *on_data = nullptr;
+		if (!(response.flags_ & HttpResponse::flag_ignore_body)) {
+			if (response.success()) {
+				if (response.writer_) {
+					while (remaining) {
+						if (read_state_.writer_buffer_.size() >= read_state_.writer_buffer_.capacity()) {
+							auto r = response.writer_->get_write_buffer(read_state_.writer_buffer_);
+							if (r == aio_result::wait) {
+								res = FZ_REPLY_WOULDBLOCK;
+								break;
+							}
+							else if (r == aio_result::error) {
+								res =  FZ_REPLY_CRITICALERROR;
+								break;
+							}
 
-		if (response.on_data_ && !(response.flags_ & HttpResponse::flag_ignore_body)) {
-			on_data = &response.on_data_;
+							read_state_.writer_buffer_ = r.buffer_;
+						}
+
+						size_t s = std::min(remaining, read_state_.writer_buffer_.capacity() - read_state_.writer_buffer_.size());
+						read_state_.writer_buffer_.append(data, s);
+						data += s;
+						remaining -= s;
+					}
+				}
+				else {
+					if (response.body_.size() < 1024*1024*16) {
+						response.body_.append(data, remaining);
+					}
+					remaining = 0;
+				}
+			}
+			else {
+				if (response.body_.size() < 1024*1024*16) {
+					response.body_.append(data, remaining);
+				}
+				remaining = 0;
+			}
 		}
-
-		if (response.on_error_data_ && !response.success() && !(response.flags_ & HttpResponse::flag_ignore_body)) {
-			on_data = &response.on_error_data_;
-		}
-
-		if (on_data) {
-			res = (*on_data)(data, len);
+		else {
+			remaining = 0;
 		}
 	}
+	else {
+		remaining = 0;
+	}
+
+	read_state_.receivedData_ += initial - remaining;
 
 	if (res == FZ_REPLY_CONTINUE && read_state_.receivedData_ == read_state_.responseContentLength_) {
-		if (shared_response) {
-			shared_response->response().flags_ |= HttpResponse::flag_got_body;
-		}
-		res = FZ_REPLY_OK;
+		res = FinalizeResponseBody();
 	}
 
 	return res;
@@ -770,3 +911,24 @@ int CHttpRequestOpData::Reset(int result)
 
 	return result;
 }
+
+void CHttpRequestOpData::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<read_ready_event>(ev, this, &CHttpRequestOpData::OnLocalData);
+}
+
+void CHttpRequestOpData::OnLocalData(reader_base * r)
+{
+	if (requests_.empty() || !requests_[send_pos_]) {
+		return;
+	}
+
+	auto & rr = *requests_[send_pos_];
+	auto & req = rr.request();
+	if (req.body_.get() != r) {
+		return;
+	}
+	if (req.flags_ & HttpRequest::flag_sent_header && !(req.flags_ & HttpRequest::flag_sent_body)) {
+		controlSocket_.SendNextCommand();
+	}
+}
