@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 CDirectoryListingNotification::CDirectoryListingNotification(CServerPath const& path, bool const primary, bool const failed)
 	: primary_(primary), m_failed(failed), m_path(path)
@@ -60,24 +60,6 @@ int CHostKeyNotification::GetPort() const
 	return m_port;
 }
 
-CDataNotification::CDataNotification(char* pData, size_t len)
-	: m_pData(pData), m_len(len)
-{
-}
-
-CDataNotification::~CDataNotification()
-{
-	delete [] m_pData;
-}
-
-char* CDataNotification::Detach(size_t& len)
-{
-	len = m_len;
-	char* pData = m_pData;
-	m_pData = nullptr;
-	return pData;
-}
-
 CCertificateNotification::CCertificateNotification(fz::tls_session_info&& info)
 	: info_(info)
 {
