@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "engineprivate.h"
 #include "proxy.h"
 #include "controlsocket.h"
@@ -7,6 +7,7 @@
 
 #include <algorithm>
 
+#include <assert.h>
 #include <string.h>
 
 enum handshake_state
