@@ -10,6 +10,25 @@
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
+#ifndef FZ_WINDOWS
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <libfilezilla/util.hpp>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
+#endif
+
+enum connectStates
+{
+	connect_init,
+	connect_proxy,
+	connect_keys,
+	connect_open
+};
+
 int CSftpConnectOpData::Send()
 {
 	switch (opState)
@@ -18,6 +37,7 @@ int CSftpConnectOpData::Send()
 		{
 			log(logmsg::status, _("Connecting to %s..."), currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_));
 
+			engine_.GetRateLimiter().add(&controlSocket_);
 			if (!controlSocket_.credentials_.keyFile_.empty()) {
 				keyfiles_ = fz::strtok(controlSocket_.credentials_.keyFile_, L"\r\n");
 			}
@@ -41,15 +61,38 @@ int CSftpConnectOpData::Send()
 			if (executable.empty()) {
 				executable = fzT("fzsftp");
 			}
+
 			log(logmsg::debug_verbose, L"Going to execute %s", executable);
 
 			std::vector<fz::native_string> args = { fzT("-v") };
 			if (engine_.GetOptions().get_int(OPTION_SFTP_COMPRESSION)) {
 				args.push_back(fzT("-C"));
 			}
-			engine_.GetRateLimiter().add(&controlSocket_);
+#ifndef FZ_WINDOWS
+			if (controlSocket_.shm_fd_ == -1) {
+#if HAVE_MEMFD_CREATE
+				controlSocket_.shm_fd_ = memfd_create("fzsftp", MFD_CLOEXEC);
+#else
+				std::string name = "/" + fz::base32_encode(fz::random_bytes(16), fz::base32_type::locale_safe, false);
+				controlSocket_.shm_fd_ = shm_open(name.c_str(), O_CREAT|O_EXCL|O_RDWR, S_IRUSR|S_IWUSR);
+				if (controlSocket_.shm_fd_ != -1) {
+					shm_unlink(name.c_str());
+				}
+#endif
+				if (controlSocket_.shm_fd_ == -1) {
+					log(logmsg::debug_warning, L"Could not create shm_fd_");
+					return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+				}
+			}
+#endif
 			controlSocket_.process_ = std::make_unique<fz::process>();
+#ifndef FZ_WINDOWS
+			std::vector<int> fds;
+			fds.push_back(controlSocket_.shm_fd_);
+			if (!controlSocket_.process_->spawn(executable, args, fds)) {
+#else
 			if (!controlSocket_.process_->spawn(executable, args)) {
+#endif
 				log(logmsg::debug_warning, L"Could not create process");
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
