@@ -3,21 +3,12 @@
 
 #include "sftpcontrolsocket.h"
 
-enum connectStates
-{
-	connect_init,
-	connect_proxy,
-	connect_keys,
-	connect_open
-};
-
 class CSftpConnectOpData final : public COpData, public CSftpOpData
 {
 public:
-	CSftpConnectOpData(CSftpControlSocket & controlSocket, Credentials const& credentials)
+	CSftpConnectOpData(CSftpControlSocket & controlSocket)
 		: COpData(Command::connect, L"CSftpConnectOpData")
 		, CSftpOpData(controlSocket)
-		, credentials_(credentials)
 		, keyfile_(keyfiles_.cend())
 	{}
 
@@ -29,8 +20,6 @@ public:
 	CInteractiveLoginNotification::type lastChallengeType{ CInteractiveLoginNotification::interactive };
 	bool criticalFailure{};
 
-	Credentials credentials_;
-
 	std::vector<std::wstring> keyfiles_;
 	std::vector<std::wstring>::const_iterator keyfile_;
 };
