@@ -1,7 +1,11 @@
 #ifndef FILEZILLA_ENGINE_SFTP_EVENT_HEADER
 #define FILEZILLA_ENGINE_SFTP_EVENT_HEADER
 
-#define FZSFTP_PROTOCOL_VERSION 9
+#include <libfilezilla/event.hpp>
+
+#include <string>
+
+#define FZSFTP_PROTOCOL_VERSION 10
 
 enum class sftpEvent {
 	Unknown = -1,
@@ -31,6 +35,10 @@ enum class sftpEvent {
 	MacClientToServer,
 	MacServerToClient,
 	Hostkey,
+	io_size,
+	io_open,
+	io_nextbuf,
+	io_finalize,
 
 	count
 };
