@@ -6,6 +6,7 @@
 #include "../../include/engine_options.h"
 
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/process.hpp>
 
 #include <assert.h>
 
@@ -19,32 +20,26 @@ enum filetransferStates
 	filetransfer_chmtime
 };
 
+CSftpFileTransferOpData::~CSftpFileTransferOpData()
+{
+	remove_handler();
+	reader_.reset();
+}
+
 int CSftpFileTransferOpData::Send()
 {
 	if (opState == filetransfer_init) {
-
-		if (localFile_.empty()) {
-			if (!download_) {
-				return FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED;
-			}
-			else {
-				return FZ_REPLY_SYNTAXERROR;
-			}
-		}
-
-		if (download_) {
+		if (download()) {
 			std::wstring filename = remotePath_.FormatFilename(remoteFile_);
 			log(logmsg::status, _("Starting download of %s"), filename);
 		}
 		else {
-			log(logmsg::status, _("Starting upload of %s"), localFile_);
+			log(logmsg::status, _("Starting upload of %s"), localName_);
 		}
 
-		int64_t size;
-		bool isLink;
-		if (fz::local_filesys::get_file_info(fz::to_native(localFile_), isLink, &size, nullptr, nullptr) == fz::local_filesys::file) {
-			localFileSize_ = size;
-		}
+		localFileSize_ = download() ? writer_factory_.size() : reader_factory_.size();
+		localFileTime_ = download() ? writer_factory_.mtime() : reader_factory_.mtime();
+
 
 		opState = filetransfer_waitcwd;
 
@@ -64,11 +59,7 @@ int CSftpFileTransferOpData::Send()
 			cmd = "re";
 			logstr = L"re";
 		}
-		if (download_) {
-			if (!resume_) {
-				controlSocket_.CreateLocalDir(localFile_);
-			}
-
+		if (download()) {
 			engine_.transfer_status_.Init(remoteFileSize_, resume_ ? localFileSize_ : 0, false);
 			cmd += "get ";
 			logstr += L"get ";
@@ -81,7 +72,7 @@ int CSftpFileTransferOpData::Send()
 			cmd += remoteFile + " ";
 			logstr += controlSocket_.QuoteFilename(remotePath_.FormatFilename(remoteFile_, !tryAbsolutePath_)) + L" "; 
 			
-			std::wstring localFile = controlSocket_.QuoteFilename(localFile_);
+			std::wstring localFile = controlSocket_.QuoteFilename(localName_);
 			cmd += fz::to_utf8(localFile);
 			logstr += localFile;
 		}
@@ -90,7 +81,7 @@ int CSftpFileTransferOpData::Send()
 			cmd += "put ";
 			logstr += L"put ";
 
-			std::wstring localFile = controlSocket_.QuoteFilename(localFile_);
+			std::wstring localFile = controlSocket_.QuoteFilename(localName_);
 			cmd += fz::to_utf8(localFile) + " ";
 			logstr += localFile + L" ";
 
@@ -114,15 +105,15 @@ int CSftpFileTransferOpData::Send()
 		return controlSocket_.SendCommand(L"mtime " + quotedFilename);
 	}
 	else if (opState == filetransfer_chmtime) {
-		assert(!fileTime_.empty());
-		if (download_) {
+		assert(!localFileTime_.empty());
+		if (download()) {
 			log(logmsg::debug_info, L"  filetransfer_chmtime during download");
 			return FZ_REPLY_INTERNALERROR;
 		}
 
 		std::wstring quotedFilename = controlSocket_.QuoteFilename(remotePath_.FormatFilename(remoteFile_, !tryAbsolutePath_));
 
-		fz::datetime t = fileTime_;
+		fz::datetime t = localFileTime_;
 		t -= fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
 
 		// Y2K38
@@ -137,16 +128,17 @@ int CSftpFileTransferOpData::Send()
 int CSftpFileTransferOpData::ParseResponse()
 {
 	if (opState == filetransfer_transfer) {
+		writer_.reset();
 		if (controlSocket_.result_ == FZ_REPLY_OK && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
-			if (download_) {
-				if (!fileTime_.empty()) {
-					if (!fz::local_filesys::set_modification_time(fz::to_native(localFile_), fileTime_))
+			if (download()) {
+				if (!remoteFileTime_.empty()) {
+					if (!writer_factory_.set_mtime(remoteFileTime_)) {
 						log(logmsg::debug_warning, L"Could not set modification time");
+					}
 				}
 			}
 			else {
-				fileTime_ = fz::local_filesys::get_modification_time(fz::to_native(localFile_));
-				if (!fileTime_.empty()) {
+				if (!localFileTime_.empty()) {
 					opState = filetransfer_chmtime;
 					return FZ_REPLY_CONTINUE;
 				}
@@ -169,8 +161,8 @@ int CSftpFileTransferOpData::ParseResponse()
 			if (parsed) {
 				fz::datetime fileTime = fz::datetime(seconds, fz::datetime::seconds);
 				if (!fileTime.empty()) {
-					fileTime_ = fileTime;
-					fileTime_ += fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
+					remoteFileTime_ = fileTime;
+					remoteFileTime_+= fz::duration::from_minutes(currentServer_.GetTimezoneOffset());
 				}
 			}
 		}
@@ -183,7 +175,7 @@ int CSftpFileTransferOpData::ParseResponse()
 		return FZ_REPLY_CONTINUE;
 	}
 	else if (opState == filetransfer_chmtime) {
-		if (download_) {
+		if (download()) {
 			log(logmsg::debug_info, L"  filetransfer_chmtime during download");
 			return FZ_REPLY_INTERNALERROR;
 		}
@@ -208,7 +200,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (!dirDidExist) {
 					opState = filetransfer_waitlist;
 				}
-				else if (download_ && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
+				else if (download() && engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS)) {
 					opState = filetransfer_mtime;
 				}
 				else {
@@ -223,10 +215,10 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 					if (matchedCase) {
 						remoteFileSize_ = entry.size;
 						if (entry.has_date()) {
-							fileTime_ = entry.time;
+							remoteFileTime_ = entry.time;
 						}
 
-						if (download_ && !entry.has_time() &&
+						if (download() && !entry.has_time() &&
 							engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 						{
 							opState = filetransfer_mtime;
@@ -266,7 +258,7 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (!dirDidExist) {
 					opState = filetransfer_mtime;
 				}
-				else if (download_ &&
+				else if (download() &&
 					engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 				{
 					opState = filetransfer_mtime;
@@ -279,10 +271,10 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 				if (matchedCase && !entry.is_unsure()) {
 					remoteFileSize_ = entry.size;
 					if (entry.has_date()) {
-						fileTime_ = entry.time;
+						remoteFileTime_ = entry.time;
 					}
 
-					if (download_ && !entry.has_time() &&
+					if (download() && !entry.has_time() &&
 						engine_.GetOptions().get_int(OPTION_PRESERVE_TIMESTAMPS))
 					{
 						opState = filetransfer_mtime;
@@ -313,3 +305,147 @@ int CSftpFileTransferOpData::SubcommandResult(int prevResult, COpData const&)
 
 	return FZ_REPLY_CONTINUE;
 }
+
+void CSftpFileTransferOpData::OnOpenRequested(uint64_t offset)
+{
+	if (reader_ || writer_) {
+		controlSocket_.AddToStream("-0\n");
+		return;
+	}
+
+#ifdef FZ_WINDOWS
+	aio_base::shm_flag shm = true;
+#else
+	aio_base::shm_flag shm = controlSocket_.shm_fd_;
+#endif
+	decltype(std::declval<aio_base>().shared_memory_info()) info;
+	if (download()) {
+		if (resume_) {
+			offset = writer_factory_.size();
+			if (offset == aio_base::nosize) {
+				controlSocket_.AddToStream("-1\n");
+				return;
+			}
+		}
+		else {
+			offset = 0;
+		}
+		writer_ = writer_factory_.open(offset, engine_, this, shm);
+		if (!writer_) {
+			controlSocket_.AddToStream("--\n");
+			return;
+		}
+		info = writer_->shared_memory_info();
+	}
+	else {
+		reader_ = reader_factory_.open(offset, engine_, this, shm);
+		if (!reader_) {
+			controlSocket_.AddToStream("--\n");
+			return;
+		}
+		else {
+			info = reader_->shared_memory_info();
+		}
+	}
+#ifdef FZ_WINDOWS
+	HANDLE target;
+	if (!DuplicateHandle(GetCurrentProcess(), std::get<0>(info), controlSocket_.process_->handle(), &target, 0, false, DUPLICATE_SAME_ACCESS)) {
+		log(logmsg::debug_warning, L"DuplicateHandle failed");
+		controlSocket_.ResetOperation(FZ_REPLY_ERROR);
+		return;
+	}
+	controlSocket_.AddToStream(fz::sprintf("-%u %u %u\n", reinterpret_cast<uintptr_t>(target), std::get<2>(info), offset));
+#else
+	controlSocket_.AddToStream(fz::sprintf("-%d %u %u\n", std::get<0>(info), std::get<2>(info), offset));
+#endif
+	base_address_ = std::get<1>(info);
+}
+
+
+void CSftpFileTransferOpData::OnNextBufferRequested(uint64_t processed)
+{
+	if (reader_) {
+		auto r = reader_->read();
+		if (r == aio_result::wait) {
+			return;
+		}
+		if (r.type_ == aio_result::error) {
+			controlSocket_.AddToStream("--1\n");
+			return;
+		}
+		controlSocket_.AddToStream(fz::sprintf("-%d %d\n", r.buffer_.get() - base_address_, r.buffer_.size()));
+	}
+	else if (writer_) {
+		buffer_.resize(processed);
+		auto r = writer_->get_write_buffer(buffer_);
+		if (r == aio_result::wait) {
+			return;
+		}
+		if (r.type_ == aio_result::error) {
+			controlSocket_.AddToStream("--1\n");
+			return;
+		}
+		buffer_ = r.buffer_;
+		controlSocket_.AddToStream(fz::sprintf("-%d %d\n", buffer_.get() - base_address_, buffer_.capacity()));
+	}
+	else {
+		controlSocket_.AddToStream("--1\n");
+		return;
+	}
+}
+
+void CSftpFileTransferOpData::OnFinalizeRequested(uint64_t lastWrite)
+{
+	finalizing_ = true;
+	buffer_.resize(lastWrite);
+	auto res = writer_->finalize(buffer_);
+	if (res == aio_result::wait) {
+		return;
+	}
+	else if (res == aio_result::ok) {
+		controlSocket_.AddToStream(fz::sprintf("-1\n"));
+	}
+	else {
+		controlSocket_.AddToStream(fz::sprintf("-0\n"));
+	}
+}
+
+void CSftpFileTransferOpData::OnSizeRequested()
+{
+	uint64_t size = aio_base::nosize;
+	if (reader_) {
+		size = reader_->size();
+	}
+	else if (writer_) {
+		size = writer_->size();
+	}
+	if (size == aio_base::nosize) {
+		controlSocket_.AddToStream("--1\n");
+	}
+	else {
+		controlSocket_.AddToStream(fz::sprintf("-%d\n", size));
+	}
+}
+
+void CSftpFileTransferOpData::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<read_ready_event, write_ready_event>(ev, this,
+		&CSftpFileTransferOpData::OnReaderEvent,
+		&CSftpFileTransferOpData::OnWriterEvent
+	);
+}
+
+void CSftpFileTransferOpData::OnReaderEvent(reader_base*)
+{
+	OnNextBufferRequested(0);
+}
+
+void CSftpFileTransferOpData::OnWriterEvent(writer_base*)
+{
+	if (finalizing_) {
+		OnFinalizeRequested(0);
+	}
+	else {
+		OnNextBufferRequested(0);
+	}
+}
