@@ -1,30 +1,37 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "chmod.h"
 #include "connect.h"
 #include "cwd.h"
 #include "delete.h"
-#include "../directorycache.h"
-#include "directorylistingparser.h"
-#include "engineprivate.h"
 #include "event.h"
 #include "filetransfer.h"
 #include "list.h"
 #include "input_thread.h"
 #include "mkd.h"
-#include "pathcache.h"
-#include "proxy.h"
 #include "rename.h"
 #include "rmd.h"
-#include "servercapabilities.h"
 #include "sftpcontrolsocket.h"
 
+#include "../directorycache.h"
+#include "../directorylistingparser.h"
+#include "../engineprivate.h"
+#include "../pathcache.h"
+#include "../proxy.h"
+#include "../servercapabilities.h"
+
+#include "../../include/engine_options.h"
+
 #include <libfilezilla/event_loop.hpp>
-#include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
 #include <algorithm>
-#include <cwchar>
+
+#include <assert.h>
+
+#ifndef FZ_WINDOWS
+#include <unistd.h>
+#endif
 
 struct SftpRateAvailableEventType;
 typedef fz::simple_event<SftpRateAvailableEventType, fz::direction::type> SftpRateAvailableEvent;
@@ -44,45 +51,15 @@ CSftpControlSocket::~CSftpControlSocket()
 
 void CSftpControlSocket::Connect(CServer const& server, Credentials const& credentials)
 {
-	log(logmsg::status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port, credentials));
-	SetWait(true);
-
-	m_sftpEncryptionDetails = CSftpEncryptionNotification();
-
 	if (server.GetEncodingType() == ENCODING_CUSTOM) {
 		log(logmsg::debug_info, L"Using custom encoding: %s", server.GetCustomEncoding());
 		m_useUTF8 = false;
 	}
-	else {
-		m_useUTF8 = true;
-	}
 
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	auto pData = std::make_unique<CSftpConnectOpData>(*this, credentials);
-	if (!credentials.keyFile_.empty()) {
-		pData->keyfiles_ = fz::strtok(credentials.keyFile_, L"\r\n");
-	}
-	else {
-		pData->keyfiles_ = fz::strtok(engine_.GetOptions().GetOption(OPTION_SFTP_KEYFILES), L"\r\n");
-	}
-
-	pData->keyfiles_.erase(
-		std::remove_if(pData->keyfiles_.begin(), pData->keyfiles_.end(),
-			[this](std::wstring const& keyfile) {
-				if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
-					log(logmsg::status, _("Skipping non-existing key file \"%s\""), keyfile);
-					return true;
-				}
-				return false;
-		}), pData->keyfiles_.end());
-
-	pData->keyfile_ = pData->keyfiles_.cbegin();
-
-	process_ = std::make_unique<fz::process>();
-
-	engine_.GetRateLimiter().add(this);
-	Push(std::move(pData));
+	Push(std::make_unique<CSftpConnectOpData>(*this));
 }
 
 void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
@@ -140,23 +117,26 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 
 			bool tmp;
 			CTransferStatus status = engine_.transfer_status_.Get(tmp);
-			if (!status.empty() && !status.madeProgress) {
+			if (!status.empty()) {
 				if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
 					auto & data = static_cast<CSftpFileTransferOpData &>(*operations_.back());
-					if (data.download_) {
-						if (value > 0) {
-							engine_.transfer_status_.SetMadeProgress();
-						}
+					if (!data.download()) {
+						engine_.transfer_status_.Update(value);	
 					}
-					else {
-						if (status.currentOffset > status.startOffset + 65565) {
-							engine_.transfer_status_.SetMadeProgress();
+					if (!status.madeProgress) {
+						if (data.download()) {
+							if (value > 0) {
+								engine_.transfer_status_.SetMadeProgress();
+							}
+						}
+						else {
+							if (status.currentOffset > status.startOffset + 65565) {
+								engine_.transfer_status_.SetMadeProgress();
+							}
 						}
 					}
 				}
 			}
-
-			engine_.transfer_status_.Update(value);
 		}
 		break;
 	case sftpEvent::AskHostkey:
@@ -167,7 +147,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				DoClose(FZ_REPLY_INTERNALERROR);
 				break;
 			}
-			SendAsyncRequest(new CHostKeyNotification(message.text[0], port, m_sftpEncryptionDetails, message.type == sftpEvent::AskHostkeyChanged));
+			SendAsyncRequest(std::make_unique<CHostKeyNotification>(message.text[0], port, m_sftpEncryptionDetails, message.type == sftpEvent::AskHostkeyChanged));
 		}
 		break;
 	case sftpEvent::AskHostkeyBetteralg:
@@ -185,7 +165,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 			std::wstring const challengeIdentifier = m_requestPreamble + L"\n" + m_requestInstruction + L"\n" + message.text[0];
 
 			CInteractiveLoginNotification::type t = CInteractiveLoginNotification::interactive;
-			if (data.credentials_.logonType_ == LogonType::interactive || m_requestPreamble == L"SSH key passphrase") {
+			if (credentials_.logonType_ == LogonType::interactive || m_requestPreamble == L"SSH key passphrase") {
 				if (m_requestPreamble == L"SSH key passphrase") {
 					t = CInteractiveLoginNotification::keyfile;
 				}
@@ -200,12 +180,12 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				if (message.text[0] != L"Password:") {
 					challenge += message.text[0];
 				}
-				CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(t, challenge, data.lastChallenge == challengeIdentifier);
-				pNotification->server = currentServer_;
-				pNotification->handle_ = handle_;
-				pNotification->credentials = data.credentials_;
+				auto notification = std::make_unique<CInteractiveLoginNotification>(t, challenge, data.lastChallenge == challengeIdentifier);
+				notification->server = currentServer_;
+				notification->handle_ = handle_;
+				notification->credentials = credentials_;
 
-				SendAsyncRequest(pNotification);
+				SendAsyncRequest(std::move(notification));
 			}
 			else {
 				if (!data.lastChallenge.empty() && data.lastChallengeType != CInteractiveLoginNotification::keyfile) {
@@ -220,7 +200,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 					return;
 				}
 
-				std::wstring const pass = (data.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : data.credentials_.GetPass();
+				std::wstring const pass = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : credentials_.GetPass();
 				std::wstring show = L"Pass: ";
 				show.append(pass.size(), '*');
 				SendCommand(pass, show);
@@ -281,6 +261,30 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 			}
 		}
 		break;
+	case sftpEvent::io_nextbuf:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnNextBufferRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
+	case sftpEvent::io_open:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnOpenRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
+	case sftpEvent::io_size:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnSizeRequested();
+		}
+		break;
+	case sftpEvent::io_finalize:
+		if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
+			auto & data = static_cast<CSftpFileTransferOpData&>(*operations_.back());
+			data.OnFinalizeRequested(fz::to_integral<uint64_t>(message.text[0]));
+		}
+		break;
 	default:
 		log(logmsg::debug_warning, L"Message type %d not handled", message.type);
 		break;
@@ -417,8 +421,6 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				return false;
 			}
 
-			auto & data = static_cast<CSftpConnectOpData&>(*operations_.back());
-
 			auto *pInteractiveLoginNotification = static_cast<CInteractiveLoginNotification *>(pNotification);
 
 			if (!pInteractiveLoginNotification->passwordSet) {
@@ -427,7 +429,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 			}
 			std::wstring const& pass = pInteractiveLoginNotification->credentials.GetPass();
 			if (pInteractiveLoginNotification->GetType() != CInteractiveLoginNotification::keyfile) {
-				data.credentials_.SetPass(pass);
+				credentials_.SetPass(pass);
 			}
 			std::wstring show = L"Pass: ";
 			show.append(pass.size(), '*');
@@ -444,21 +446,6 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 
 void CSftpControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		log(logmsg::status, _("Retrieving directory listing..."));
-	}
-	else {
-		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
 	Push(std::make_unique<CSftpListOpData>(*this, path, subDir, flags));
 }
 
@@ -470,7 +457,7 @@ void CSftpControlSocket::ChangeDir(CServerPath const& path, std::wstring const&
 	pData->link_discovery_ = link_discovery;
 
 	if (!operations_.empty() && operations_.back()->opId == Command::transfer &&
-		!static_cast<CSftpFileTransferOpData&>(*operations_.back()).download_)
+		!static_cast<CSftpFileTransferOpData&>(*operations_.back()).download())
 	{
 		pData->tryMkdOnFail_ = true;
 		assert(subDir.empty());
@@ -518,18 +505,14 @@ void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 		}
 	}
 }
-void CSftpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-									std::wstring const& remoteFile, bool download,
-									CFileTransferCommand::t_transferSettings const& transferSettings)
+void CSftpControlSocket::FileTransfer(CFileTransferCommand const& cmd)
 {
-	auto pData = std::make_unique<CSftpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
-	Push(std::move(pData));
+	Push(std::make_unique<CSftpFileTransferOpData>(*this, cmd));
 }
 
 int CSftpControlSocket::DoClose(int nErrorCode)
 {
 	remove_bucket();
-
 	if (process_) {
 		process_->kill();
 	}
@@ -550,6 +533,16 @@ int CSftpControlSocket::DoClose(int nErrorCode)
 		event_loop_.filter_events(threadEventsFilter);
 	}
 	process_.reset();
+
+#ifndef FZ_WINDOWS
+	if (shm_fd_ != -1) {
+		close(shm_fd_);
+		shm_fd_ = -1;
+	}
+#endif
+
+	m_sftpEncryptionDetails = CSftpEncryptionNotification();
+
 	return CControlSocket::DoClose(nErrorCode);
 }
 
@@ -562,15 +555,6 @@ void CSftpControlSocket::Cancel()
 
 void CSftpControlSocket::Mkdir(CServerPath const& path)
 {
-	/* Directory creation works like this: First find a parent directory into
-	 * which we can CWD, then create the subdirs one by one. If either part
-	 * fails, try MKD with the full path directly.
-	 */
-
-	if (operations_.empty()) {
-		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CSftpMkdirOpData>(*this);
 	pData->path_ = path;
 	Push(std::move(pData));
@@ -587,7 +571,7 @@ void CSftpControlSocket::Delete(const CServerPath& path, std::vector<std::wstrin
 	assert(!files.empty());
 
 	log(logmsg::debug_verbose, L"CSftpControlSocket::Delete");
-	
+
 	auto pData = std::make_unique<CSftpDeleteOpData>(*this);
 	pData->path_ = path;
 	pData->files_ = std::move(files);
@@ -606,13 +590,11 @@ void CSftpControlSocket::RemoveDir(CServerPath const& path, std::wstring const&
 
 void CSftpControlSocket::Chmod(CChmodCommand const& command)
 {
-	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
 	Push(std::make_unique<CSftpChmodOpData>(*this, command));
 }
 
 void CSftpControlSocket::Rename(CRenameCommand const& command)
 {
-	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
 	Push(std::make_unique<CSftpRenameOpData>(*this, command));
 }
 
@@ -627,20 +609,20 @@ void CSftpControlSocket::OnQuotaRequest(fz::direction::type const d)
 		return;
 	}
 
-	size_t bytes = available(d);
+	fz::rate::type bytes = available(d);
 	if (bytes == fz::rate::unlimited) {
 		AddToStream(fz::sprintf("-%d-\n", d));
 	}
 	else if (bytes > 0) {
 		int b;
-		if (bytes > std::numeric_limits<int>::max()) {
+		if (bytes > static_cast<size_t>(std::numeric_limits<int>::max())) {
 			b = std::numeric_limits<int>::max();
 		}
 		else {
 			b = static_cast<int>(bytes);
 		}
-		AddToStream(fz::sprintf("-%d%d,%d\n", d, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(d))));
-		consume(d, static_cast<size_t>(bytes));
+		AddToStream(fz::sprintf("-%d%d,%d\n", d, b, engine_.GetOptions().get_int(d ? OPTION_SPEEDLIMIT_OUTBOUND : OPTION_SPEEDLIMIT_INBOUND)));
+		consume(d, static_cast<fz::rate::type>(b));
 	}
 }
 
@@ -656,3 +638,15 @@ void CSftpControlSocket::operator()(fz::event_base const& ev)
 
 	CControlSocket::operator()(ev);
 }
+
+void CSftpControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!process_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CSftpConnectOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CControlSocket::Push(std::move(connOp));
+		}
+	}
+}
