@@ -1,7 +1,7 @@
 #ifndef FILEZILLA_ENGINE_SFTP_SFTPCONTROLSOCKET_HEADER
 #define FILEZILLA_ENGINE_SFTP_SFTPCONTROLSOCKET_HEADER
 
-#include "controlsocket.h"
+#include "../controlsocket.h"
 
 #include <libfilezilla/rate_limiter.hpp>
 
@@ -22,9 +22,7 @@ public:
 	virtual void Connect(CServer const& server, Credentials const& credentials) override;
 	virtual void List(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring(), int flags = 0) override;
 	void ChangeDir(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring(), bool link_discovery = false);
-	virtual void FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
-		std::wstring const& remoteFile, bool download,
-		CFileTransferCommand::t_transferSettings const& transferSettings) override;
+	virtual void FileTransfer(CFileTransferCommand const& cmd) override;
 	virtual void Delete(CServerPath const& path, std::vector<std::wstring>&& files) override;
 	virtual void RemoveDir(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring()) override;
 	virtual void Mkdir(CServerPath const& path) override;
@@ -37,6 +35,8 @@ public:
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification) override;
 
 protected:
+	virtual void Push(std::unique_ptr<COpData> && pNewOpData) override;
+
 	// Replaces filename"with"quotes with
 	// "filename""with""quotes"
 	std::wstring QuoteFilename(std::wstring const& filename);
@@ -52,6 +52,9 @@ protected:
 	virtual void wakeup(fz::direction::type const d) override;
 	void OnQuotaRequest(fz::direction::type const d);
 
+#ifndef FZ_WINDOWS
+	int shm_fd_{-1};
+#endif
 	std::unique_ptr<fz::process> process_;
 	std::unique_ptr<CSftpInputThread> input_thread_;
 
