@@ -6,28 +6,21 @@
 enum connectStates
 {
 	connect_init,
-	connect_timeout,
-	connect_proxy,
 	connect_host,
 	connect_user,
-	connect_pass,
-	connect_key
+	connect_pass
 };
 
 class CStorjConnectOpData final : public COpData, public CStorjOpData
 {
 public:
-	CStorjConnectOpData(CStorjControlSocket & controlSocket, Credentials const& credentials)
+	CStorjConnectOpData(CStorjControlSocket & controlSocket)
 		: COpData(Command::connect, L"CStorjConnectOpData")
 		, CStorjOpData(controlSocket)
-		, credentials_(credentials)
 	{}
 
 	virtual int Send() override;
 	virtual int ParseResponse() override;
-
-private:
-	Credentials const credentials_;
 };
 
 #endif
