@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "../directorycache.h"
 #include "delete.h"
@@ -18,11 +18,7 @@ int CStorjDeleteOpData::Send()
 			return FZ_REPLY_CRITICALERROR;
 		}
 
-		opState = delete_resolve;
-		return FZ_REPLY_CONTINUE;
-	case delete_resolve:
-		opState = delete_resolve;
-		controlSocket_.Resolve(path_, files_, bucket_, fileIds_);
+		opState = delete_delete;
 		return FZ_REPLY_CONTINUE;
 	case delete_delete:
 		if (files_.empty()) {
@@ -30,12 +26,6 @@ int CStorjDeleteOpData::Send()
 		}
 
 		std::wstring const& file = files_.back();
-		std::wstring const& id = fileIds_.back();
-		if (id.empty()) {
-			files_.pop_back();
-			fileIds_.pop_back();
-			return FZ_REPLY_CONTINUE;
-		}
 
 		if (time_.empty()) {
 			time_ = fz::datetime::now();
@@ -43,7 +33,7 @@ int CStorjDeleteOpData::Send()
 
 		engine_.GetDirectoryCache().InvalidateFile(currentServer_, path_, file);
 
-		return controlSocket_.SendCommand(L"rm " + bucket_ + L" " + id);
+		return controlSocket_.SendCommand(L"rm " + controlSocket_.QuoteFilename(path_.FormatFilename(file)));
 	}
 
 	log(logmsg::debug_warning, L"Unknown opState in CStorjDeleteOpData::Send()");
@@ -72,7 +62,6 @@ int CStorjDeleteOpData::ParseResponse()
 	}
 
 	files_.pop_back();
-	fileIds_.pop_back();
 
 	if (!files_.empty()) {
 		return FZ_REPLY_CONTINUE;
@@ -80,17 +69,3 @@ int CStorjDeleteOpData::ParseResponse()
 
 	return deleteFailed_ ? FZ_REPLY_ERROR : FZ_REPLY_OK;
 }
-
-int CStorjDeleteOpData::SubcommandResult(int prevResult, COpData const&)
-{
-	if (prevResult != FZ_REPLY_OK) {
-		return prevResult;
-	}
-
-	if (files_.size() != fileIds_.size()) {
-		return FZ_REPLY_INTERNALERROR;
-	}
-
-	opState = delete_delete;
-	return FZ_REPLY_CONTINUE;
-}
