@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "event.h"
 #include "input_thread.h"
@@ -114,10 +114,12 @@ void CStorjInputThread::processEvent(storjEvent eventType, std::wstring &error)
 	case storjEvent::Info:
 	case storjEvent::Status:
 	case storjEvent::Transfer:
+	case storjEvent::io_finalize:
+	case storjEvent::io_nextbuf:
 		lines = 1;
 		break;
 	case storjEvent::Listentry:
-		lines = 4;
+		lines = 3;
 		break;
 	};
 
