@@ -1,44 +1,37 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "mkd.h"
 
 enum mkdStates
 {
 	mkd_init = 0,
 	mkd_mkbucket,
-	mkd_resolve,
 	mkd_put
 };
 
-
 int CStorjMkdirOpData::Send()
 {
 	switch (opState) {
 	case mkd_init:
 		if (path_.SegmentCount() < 1) {
-			LogMessage(MessageType::Error, _("Invalid path"));
+			log(logmsg::error, _("Invalid path"));
 			return FZ_REPLY_CRITICALERROR;
 		}
+
+		if (controlSocket_.operations_.size() == 1) {
+			log(logmsg::status, _("Creating directory '%s'..."), path_.GetPath());
+		}
+
 		opState = mkd_mkbucket;
 		return FZ_REPLY_CONTINUE;
 	case mkd_mkbucket:
 		return controlSocket_.SendCommand(L"mkbucket " + controlSocket_.QuoteFilename(path_.GetFirstSegment()));
 	case mkd_put:
-		{
-			std::wstring path = path_.GetPath();
-			auto pos = path.find('/', 1);
-			if (pos == std::string::npos) {
-				return FZ_REPLY_INTERNALERROR;
-			}
-			else {
-				path = path.substr(pos + 1) + L"/";
-			}
-			return controlSocket_.SendCommand(L"put " + bucket_ + L" \"null\" " + controlSocket_.QuoteFilename(path));
-		}
+		return controlSocket_.SendCommand(L"put \"null\" " + controlSocket_.QuoteFilename(path_.FormatFilename(L".")));
 	}
 
-	LogMessage(MessageType::Debug_Warning, L"Unknown opState in CStorjMkdirOpData::Send()");
+	log(logmsg::debug_warning, L"Unknown opState in CStorjMkdirOpData::Send()");
 	return FZ_REPLY_INTERNALERROR;
 }
 
@@ -52,8 +45,7 @@ int CStorjMkdirOpData::ParseResponse()
 		}
 
 		if (path_.SegmentCount() > 1) {
-			opState = mkd_resolve;
-			controlSocket_.Resolve(path_, std::wstring(), bucket_, 0);
+			opState = mkd_put;
 			return FZ_REPLY_CONTINUE;
 		}
 		else {
@@ -72,22 +64,6 @@ int CStorjMkdirOpData::ParseResponse()
 		return controlSocket_.result_;
 	}
 
-	LogMessage(MessageType::Debug_Warning, L"Unknown opState in CStorjMkdirOpData::ParseResponse()");
-	return FZ_REPLY_INTERNALERROR;
-}
-
-int CStorjMkdirOpData::SubcommandResult(int prevResult, COpData const&)
-{
-	if (prevResult != FZ_REPLY_OK) {
-		return prevResult;
-	}
-
-	switch (opState) {
-	case mkd_resolve:
-		opState = mkd_put;
-		return FZ_REPLY_CONTINUE;
-	}
-
-	LogMessage(MessageType::Debug_Warning, L"Unknown opState in CStorjMkdirOpData::SubcommandResult()");
+	log(logmsg::debug_warning, L"Unknown opState in CStorjMkdirOpData::ParseResponse()");
 	return FZ_REPLY_INTERNALERROR;
 }
