@@ -5,7 +5,6 @@
 #define _UNICODE
 #endif
 
-
 //---------------------------------------------------------------------------
 #ifdef _MSC_VER
 	#include <objbase.h>
@@ -52,17 +51,21 @@
 #include <cstddef>
 #include <cstdint>
 
+#include <utility>
+
 //---------------------------------------------------------------------------
 #ifdef DEBUG
 #define DEBUG_MSG(MSG) Debug(MSG)
 #define DEBUG_MSG_W(MSG) DebugW(MSG)
 #define DEBUG_LOG_VERSION(MSG) LogVersion(MSG)
 #define DEBUG_INIT(KEY) DebugInit(KEY)
+#define DEBUG_MSG_PRINTF(FMT, ...) DebugPrintf(FMT, __VA_ARGS__)
 #else
 #define DEBUG_MSG(MSG)
 #define DEBUG_MSG_W(MSG)
 #define DEBUG_LOG_VERSION(MSG)
 #define DEBUG_INIT(KEY)
+#define DEBUG_MSG_PRINTF(FMT, ...)
 #endif
 
 //---------------------------------------------------------------------------
@@ -175,12 +178,14 @@ HANDLE GLogMutex = 0;
 
 void Debug(const char* Message)
 {
-	if (!GLogOn)
+	if (!GLogOn) {
 		return;
+	}
 
 	unsigned long WaitResult = WaitForSingleObject(GLogMutex, 1000);
-	if (WaitResult == WAIT_TIMEOUT)
+	if (WaitResult == WAIT_TIMEOUT) {
 		return;
+	}
 
 	if (GLogHandle == NULL) {
 		if (strlen(GLogFile) == 0) {
@@ -214,8 +219,9 @@ void Debug(const char* Message)
 
 void DebugW(const wchar_t* Message)
 {
-	if (!GLogOn)
+	if (!GLogOn) {
 		return;
+	}
 
 	int bytes = WideCharToMultiByte(CP_UTF8, 0, Message, -1, 0, 0, 0, 0);
 	if (bytes <= 0) {
@@ -225,21 +231,34 @@ void DebugW(const wchar_t* Message)
 	char *buffer = new char[bytes + 1];
 	if (buffer) {
 		int written = WideCharToMultiByte(CP_UTF8, 0, Message, -1, buffer, bytes, 0, 0);
-		if (!written)
+		if (!written) {
 			DEBUG_MSG("WideCharToMultiByte failed");
+		}
 		else {
 			buffer[written] = 0;
 			Debug(buffer);
 		}
-		delete[] buffer;
+		delete [] buffer;
 	}
 }
 
+template<typename ... Args>
+void DebugPrintf(char const* fmt, Args&&... args)
+{
+	char buffer[100];
+	for (unsigned int i = 0; i < 100; ++i) {
+		buffer[i] = 0;
+	}
+	snprintf(buffer, 99, fmt, std::forward<Args>(args)...);
+	Debug(buffer);
+}
+
 //---------------------------------------------------------------------------
 void LogVersion(HINSTANCE HInstance)
 {
-	if (!GLogOn)
+	if (!GLogOn) {
 		return;
+	}
 
 	char FileName[MAX_PATH];
 	if (!GetModuleFileNameA(HInstance, FileName, sizeof(FileName))) {
@@ -304,7 +323,7 @@ void DebugInit(HKEY Key)
 
 //---------------------------------------------------------------------------
 extern "C" int APIENTRY
-DllMain(HINSTANCE HInstance, DWORD Reason, LPVOID Reserved)
+DllMain(HINSTANCE HInstance, DWORD Reason, LPVOID)
 {
 	if (Reason == DLL_PROCESS_ATTACH) {
 		GInstance = HInstance;
@@ -414,20 +433,18 @@ bool RegisterServer(bool AllUsers)
 
 	if ((RegOpenKeyEx(RootKey, _T("Software\\Classes"), 0, KEY_WRITE, &HKey) ==
 		ERROR_SUCCESS) &&
-		(RegCreateKeyEx(HKey, _T("CLSID"), 0, NULL,
-		REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &HKey, &Unused) ==
+		(RegCreateKeyEx(HKey, _T("CLSID"), 0, nullptr,
+		REG_OPTION_NON_VOLATILE, KEY_WRITE, nullptr, &HKey, &Unused) ==
 		ERROR_SUCCESS))
 	{
-		if (RegCreateKey(HKey, ClassID, &HKey) == ERROR_SUCCESS)
-		{
-			RegSetValueEx(HKey, NULL, 0, REG_SZ,
+		if (RegCreateKey(HKey, ClassID, &HKey) == ERROR_SUCCESS) {
+			RegSetValueEx(HKey, nullptr, 0, REG_SZ,
 				reinterpret_cast<const unsigned char*>(DRAG_EXT_NAME), sizeof(DRAG_EXT_NAME));
 
-			if (RegCreateKey(HKey, _T("InProcServer32"), &HKey) == ERROR_SUCCESS)
-			{
+			if (RegCreateKey(HKey, _T("InProcServer32"), &HKey) == ERROR_SUCCESS) {
 				wchar_t Filename[MAX_PATH];
 				GetModuleFileName(GInstance, Filename, MAX_PATH);
-				RegSetValueEx(HKey, NULL, 0, REG_SZ,
+				RegSetValueEx(HKey, nullptr, 0, REG_SZ,
 					reinterpret_cast<LPBYTE>(Filename), (_tcslen(Filename) + 1) * sizeof(TCHAR));
 
 				RegSetValueEx(HKey, _T("ThreadingModel"), 0, REG_SZ,
@@ -441,15 +458,15 @@ bool RegisterServer(bool AllUsers)
 			0, KEY_WRITE, &HKey) == ERROR_SUCCESS) &&
 			(RegCreateKeyEx(HKey,
 			_T("directory\\shellex\\CopyHookHandlers\\FileZilla3CopyHook"),
-			0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &HKey,
+			0, nullptr, REG_OPTION_NON_VOLATILE, KEY_WRITE, nullptr, &HKey,
 			&Unused) == ERROR_SUCCESS))
 		{
-			RegSetValueEx(HKey, NULL, 0, REG_SZ,
+			RegSetValueEx(HKey, nullptr, 0, REG_SZ,
 				reinterpret_cast<LPBYTE>(ClassID), (_tcslen(ClassID) + 1) * 2);
 			RegCloseKey(HKey);
 
 			if ((RegCreateKeyEx(RootKey, DRAG_EXT_REG_KEY,
-				0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &HKey,
+				0, nullptr, REG_OPTION_NON_VOLATILE, KEY_WRITE, nullptr, &HKey,
 				&Unused) == ERROR_SUCCESS))
 			{
 				unsigned long Value = 1;
@@ -461,7 +478,7 @@ bool RegisterServer(bool AllUsers)
 				Result = true;
 			}
 
-			SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
+			SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nullptr, nullptr);
 		}
 	}
 
@@ -476,10 +493,12 @@ STDEXPORTAPI DllRegisterServer()
 	DEBUG_MSG("DllRegisterServer enter");
 
 	HRESULT Result;
-	if (RegisterServer(true) || RegisterServer(false))
+	if (RegisterServer(true) || RegisterServer(false)) {
 		Result = S_OK;
-	else
+	}
+	else {
 		Result = SELFREG_E_CLASS;
+	}
 
 	DEBUG_MSG("DllRegisterServer leave");
 
@@ -493,19 +512,22 @@ static bool RegDeleteEmptyKey(HKEY root, LPCTSTR name)
 	HKEY key;
 
 	// Can't use SHDeleteEmptyKey, it gives a linking error
-	if (RegOpenKeyEx(root, name, 0, KEY_READ, &key) != ERROR_SUCCESS)
+	if (RegOpenKeyEx(root, name, 0, KEY_READ, &key) != ERROR_SUCCESS) {
 		return false;
+	}
 
 	DWORD subKeys, values;
-	int ret = RegQueryInfoKey(key, 0, 0, 0, &subKeys, 0, 0, &values, 0, 0, 0,0);
+	int ret = RegQueryInfoKey(key, nullptr, nullptr, nullptr, &subKeys, nullptr, nullptr, &values, nullptr, nullptr, nullptr, nullptr);
 
 	RegCloseKey(key);
 
-	if (ret != ERROR_SUCCESS)
+	if (ret != ERROR_SUCCESS) {
 		return false;
+	}
 
-	if (subKeys || values)
+	if (subKeys || values) {
 		return false;
+	}
 
 	RegDeleteKey(root, name);
 
@@ -573,7 +595,7 @@ bool UnregisterServer(bool AllUsers)
 	RegDeleteEmptyKey(RootKey, DRAG_EXT_REG_KEY);
 	RegDeleteEmptyKey(RootKey, DRAG_EXT_REG_KEY_PARENT);
 
-	SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
+	SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nullptr, nullptr);
 
 	DEBUG_MSG("UnregisterServer leave");
 
@@ -677,8 +699,7 @@ STDMETHODIMP CShellExtClassFactory::CreateInstance(LPUNKNOWN UnkOuter,
 	// initialized.
 
 	CShellExt* ShellExt = new CShellExt();  //Create the CShellExt object
-
-	if (NULL == ShellExt) {
+	if (!ShellExt) {
 		return E_OUTOFMEMORY;
 	}
 
@@ -686,7 +707,7 @@ STDMETHODIMP CShellExtClassFactory::CreateInstance(LPUNKNOWN UnkOuter,
 }
 
 //---------------------------------------------------------------------------
-STDMETHODIMP CShellExtClassFactory::LockServer(BOOL Lock)
+STDMETHODIMP CShellExtClassFactory::LockServer(BOOL)
 {
 	DEBUG_MSG("LockServer");
 
@@ -701,7 +722,7 @@ CShellExt::CShellExt()
 
 	FDataObj = NULL;
 
-	FMutex = CreateMutex(NULL, false, DRAG_EXT_MUTEX);
+	FMutex = CreateMutex(nullptr, false, DRAG_EXT_MUTEX);
 
 	GRefThisDll++;
 
@@ -778,15 +799,13 @@ STDMETHODIMP_(ULONG) CShellExt::Release()
 	return 0;
 }
 
-//---------------------------------------------------------------------------
-STDMETHODIMP CShellExt::Initialize(LPCITEMIDLIST IDFolder,
-								   LPDATAOBJECT DataObj, HKEY RegKey)
+STDMETHODIMP CShellExt::Initialize(LPCITEMIDLIST, LPDATAOBJECT DataObj, HKEY)
 {
 	DEBUG_MSG("CShellExt::Initialize enter");
 
-	if (FDataObj != NULL) {
+	if (FDataObj != nullptr) {
 		FDataObj->Release();
-		FDataObj = NULL;
+		FDataObj = nullptr;
 	}
 
 	// duplicate the object pointer and registry handle
@@ -802,9 +821,11 @@ STDMETHODIMP CShellExt::Initialize(LPCITEMIDLIST IDFolder,
 }
 
 //---------------------------------------------------------------------------
-STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND Hwnd, UINT wFunc, UINT Flags,
-											LPCTSTR SrcFile, DWORD SrcAttribs, LPCTSTR DestFile, DWORD DestAttribs)
+STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND, UINT wFunc, UINT wFlags,
+											LPCTSTR SrcFile, DWORD, LPCTSTR DestFile, DWORD)
 {
+	(void)wFlags;
+
 	UINT Result = IDYES;
 
 	if (!GEnabled) {
@@ -813,9 +834,7 @@ STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND Hwnd, UINT wFunc, UINT Flags,
 	}
 
 	if (wFunc != FO_COPY && wFunc != FO_MOVE) {
-		char buffer[100];
-		sprintf(buffer, "CShellExt::CopyCallback return: wFunc is %u, NOT FO_COPY nor FO_MOVE", (unsigned int)wFunc);
-		DEBUG_MSG(buffer);
+		DEBUG_MSG_PRINTF("CShellExt::CopyCallback return: wFunc is %u, NOT FO_COPY nor FO_MOVE", static_cast<unsigned int>(wFunc));
 		return Result;
 	}
 	else if (wFunc == FO_COPY) {
@@ -825,6 +844,8 @@ STDMETHODIMP_(UINT) CShellExt::CopyCallback(HWND Hwnd, UINT wFunc, UINT Flags,
 		DEBUG_MSG("CShellExt::CopyCallback: wFunc is FO_MOVE");
 	}
 
+	DEBUG_MSG_PRINTF("CShellExt::CopyCallback: wFlags is %u", static_cast<unsigned int>(wFlags));
+
 	unsigned long Ticks = GetTickCount();
 	if ((Ticks - FLastTicks) < 100 && FLastTicks <= Ticks) {
 		DEBUG_MSG("CShellExt::CopyCallback return; interval NOT elapsed");
