@@ -2,19 +2,19 @@
 #define FILEZILLA_ENGINE_ENGINE_HEADER
 
 #include "commands.h"
+#include "notification.h"
 
-#include <notification.h>
+#include <functional>
 
 class CAsyncRequestNotification;
 class CFileZillaEngineContext;
 class CFileZillaEnginePrivate;
 class CNotification;
-class EngineNotificationHandler;
 
-class CFileZillaEngine final
+class FZC_PUBLIC_SYMBOL CFileZillaEngine final
 {
 public:
-	CFileZillaEngine(CFileZillaEngineContext& engine_context, EngineNotificationHandler& notificationHandler);
+	CFileZillaEngine(CFileZillaEngineContext& engine_context, std::function<void(CFileZillaEngine*)> const& notification_cb);
 	~CFileZillaEngine();
 
 	CFileZillaEngine(CFileZillaEngine const&) = delete;
@@ -41,7 +41,7 @@ public:
 	static bool IsActive(_direction direction);
 
 	// Returns the next pending notification.
-	// It is mandatory to call this function until it returns 0 each time you
+	// It is mandatory to call this function until it returns a nullptr each time you
 	// get the pending notifications event, or you'll either lose notifications
 	// or your memory will fill with pending notifications.
 	// See notification.h for details.
@@ -62,7 +62,7 @@ public:
 	int CacheLookup(CServerPath const& path, CDirectoryListing& listing);
 
 private:
-	CFileZillaEnginePrivate* const impl_;
+	std::unique_ptr<CFileZillaEnginePrivate> impl_;
 };
 
 #endif
