@@ -3,6 +3,8 @@
 
 #include "server.h"
 #include "serverpath.h"
+#include "reader.h"
+#include "writer.h"
 
 #include <libfilezilla/uri.hpp>
 
@@ -68,7 +70,7 @@ enum class Command
 // Actual commands //
 // --------------- //
 
-class CCommand
+class FZC_PUBLIC_SYMBOL CCommand
 {
 public:
 	CCommand() = default;
@@ -85,7 +87,7 @@ protected:
 };
 
 template<typename Derived, Command id>
-class CCommandHelper : public CCommand
+class FZC_PUBLIC_SYMBOL CCommandHelper : public CCommand
 {
 public:
 	virtual Command GetId() const final { return id; }
@@ -101,11 +103,11 @@ protected:
 };
 
 template<Command id>
-class CBasicCommand final : public CCommandHelper<CBasicCommand<id>, id>
+class FZC_PUBLIC_SYMBOL CBasicCommand final : public CCommandHelper<CBasicCommand<id>, id>
 {
 };
 
-class CConnectCommand final : public CCommandHelper<CConnectCommand, Command::connect>
+class FZC_PUBLIC_SYMBOL CConnectCommand final : public CCommandHelper<CConnectCommand, Command::connect>
 {
 public:
 	explicit CConnectCommand(CServer const& server, ServerHandle const& handle, Credentials const& credentials, bool retry_conncting = true);
@@ -130,7 +132,7 @@ typedef CBasicCommand<Command::disconnect> CDisconnectCommand;
 #define LIST_FLAG_FALLBACK_CURRENT 4
 #define LIST_FLAG_LINK 8
 #define LIST_FLAG_CLEARCACHE 16
-class CListCommand final : public CCommandHelper<CListCommand, Command::list>
+class FZC_PUBLIC_SYMBOL CListCommand final : public CCommandHelper<CListCommand, Command::list>
 {
 	// Without a given directory, the current directory will be listed.
 	// Directories can either be given as absolute path or as
@@ -166,52 +168,99 @@ protected:
 	int const m_flags;
 };
 
-class CFileTransferCommand final : public CCommandHelper<CFileTransferCommand, Command::transfer>
+enum class transfer_flags : unsigned short
 {
-public:
-	class t_transferSettings final
-	{
-	public:
-		bool binary{true};
-		bool fsync{};
-	};
+	none = 0,
 
-	// For uploads, set download to false.
-	// For downloads, localFile can be left empty if supported by protocol.
-	// Check for nId_data notification.
-	// FIXME: localFile empty iff protocol is HTTP.
-	CFileTransferCommand(std::wstring const& localFile, CServerPath const& remotePath, std::wstring const& remoteFile, bool download, t_transferSettings const& m_transferSettings);
+	interface_reserved_mask = 0x08u, // The engine will never touch these
+	
+	download = 0x10,
+	fsync = 0x20,
 
-	std::wstring GetLocalFile() const;
-	CServerPath GetRemotePath() const;
-	std::wstring GetRemoteFile() const;
-	bool Download() const;
-	const t_transferSettings& GetTransferSettings() const { return m_transferSettings; }
+	// Free bits in the middle
 
-protected:
-	std::wstring const m_localFile;
-	CServerPath const m_remotePath;
-	std::wstring const m_remoteFile;
-	bool const m_download;
-	t_transferSettings const m_transferSettings;
+	protocol_reserved_mask = 0xfe00,
+	protocol_reserved_max = 0x8000
 };
 
-class CHttpRequestCommand final : public CCommandHelper<CHttpRequestCommand, Command::httprequest>
+inline bool operator&(transfer_flags lhs, transfer_flags rhs)
+{
+	return (static_cast<std::underlying_type_t<transfer_flags>>(lhs) & static_cast<std::underlying_type_t<transfer_flags>>(rhs)) != 0;
+}
+
+inline transfer_flags operator|(transfer_flags lhs, transfer_flags rhs)
+{
+	return static_cast<transfer_flags>(static_cast<std::underlying_type_t<transfer_flags>>(lhs) | static_cast<std::underlying_type_t<transfer_flags>>(rhs));
+}
+
+inline transfer_flags& operator|=(transfer_flags& lhs, transfer_flags rhs)
+{
+	lhs = static_cast<transfer_flags>(static_cast<std::underlying_type_t<transfer_flags>>(lhs) | static_cast<std::underlying_type_t<transfer_flags>>(rhs));
+	return lhs;
+}
+
+inline transfer_flags operator-(transfer_flags lhs, transfer_flags rhs)
+{
+	return static_cast<transfer_flags>(static_cast<std::underlying_type_t<transfer_flags>>(lhs) & ~static_cast<std::underlying_type_t<transfer_flags>>(rhs));
+}
+
+inline transfer_flags& operator-=(transfer_flags& lhs, transfer_flags rhs)
+{
+	lhs = static_cast<transfer_flags>(static_cast<std::underlying_type_t<transfer_flags>>(lhs) & ~static_cast<std::underlying_type_t<transfer_flags>>(rhs));
+	return lhs;
+}
+
+inline bool operator!(transfer_flags flags)
+{
+	return static_cast<std::underlying_type_t<transfer_flags>>(flags) == 0;
+}
+
+namespace ftp_transfer_flags
+{
+	auto constexpr ascii = transfer_flags::protocol_reserved_max;
+}
+
+class FZC_PUBLIC_SYMBOL CFileTransferCommand final : public CCommandHelper<CFileTransferCommand, Command::transfer>
 {
 public:
-	// Reply body is delivered through nId_data notifications
-	CHttpRequestCommand(fz::uri const& uri, std::string const& verb = std::string("GET"), std::string const& body = std::string())
+	CFileTransferCommand(reader_factory_holder const& reader, CServerPath const& remotePath, std::wstring const& remoteFile, transfer_flags const& flags);
+	CFileTransferCommand(writer_factory_holder const& writer, CServerPath const& remotePath, std::wstring const& remoteFile, transfer_flags const& flags);
+
+	CServerPath GetRemotePath() const;
+	std::wstring GetRemoteFile() const;
+	bool Download() const { return flags_ & transfer_flags::download; }
+	transfer_flags const& GetFlags() const { return flags_; }
+
+	bool valid() const;
+
+	reader_factory_holder const& GetReader() const { return reader_; }
+	writer_factory_holder const& GetWriter() const { return writer_; }
+protected:
+	reader_factory_holder const reader_;
+	writer_factory_holder const writer_;
+	CServerPath const m_remotePath;
+	std::wstring const m_remoteFile;
+	transfer_flags const flags_;
+};
+
+class FZC_PUBLIC_SYMBOL CHttpRequestCommand final : public CCommandHelper<CHttpRequestCommand, Command::httprequest>
+{
+public:
+	CHttpRequestCommand(fz::uri const& uri, writer_factory_holder const& output, std::string const& verb = std::string("GET"), reader_factory_holder const& body = reader_factory_holder())
 		: uri_(uri)
 		, verb_(verb)
 		, body_(body)
+		, output_(output)
 	{}
 
 	fz::uri const uri_;
 	std::string const verb_;
-	std::string const body_;
+
+	reader_factory_holder body_;
+	writer_factory_holder output_;
 };
 
-class CRawCommand final : public CCommandHelper<CRawCommand, Command::raw>
+class FZC_PUBLIC_SYMBOL CRawCommand final : public CCommandHelper<CRawCommand, Command::raw>
 {
 public:
 	explicit CRawCommand(std::wstring const& command);
@@ -224,7 +273,7 @@ protected:
 	std::wstring m_command;
 };
 
-class CDeleteCommand final : public CCommandHelper<CDeleteCommand, Command::del>
+class FZC_PUBLIC_SYMBOL CDeleteCommand final : public CCommandHelper<CDeleteCommand, Command::del>
 {
 public:
 	CDeleteCommand(CServerPath const& path, std::vector<std::wstring> && files);
@@ -239,7 +288,7 @@ protected:
 	std::vector<std::wstring> files_;
 };
 
-class CRemoveDirCommand final : public CCommandHelper<CRemoveDirCommand, Command::removedir>
+class FZC_PUBLIC_SYMBOL CRemoveDirCommand final : public CCommandHelper<CRemoveDirCommand, Command::removedir>
 {
 public:
 	// Directories can either be given as absolute path or as
@@ -256,7 +305,7 @@ protected:
 	std::wstring const m_subDir;
 };
 
-class CMkdirCommand final : public CCommandHelper<CMkdirCommand, Command::mkdir>
+class FZC_PUBLIC_SYMBOL CMkdirCommand final : public CCommandHelper<CMkdirCommand, Command::mkdir>
 {
 public:
 	explicit CMkdirCommand(CServerPath const& path);
@@ -269,7 +318,7 @@ protected:
 	CServerPath const m_path;
 };
 
-class CRenameCommand final : public CCommandHelper<CRenameCommand, Command::rename>
+class FZC_PUBLIC_SYMBOL CRenameCommand final : public CCommandHelper<CRenameCommand, Command::rename>
 {
 public:
 	CRenameCommand(CServerPath const& fromPath, std::wstring const& fromFile,
@@ -289,11 +338,11 @@ protected:
 	std::wstring const m_toFile;
 };
 
-class CChmodCommand final : public CCommandHelper<CChmodCommand, Command::chmod>
+class FZC_PUBLIC_SYMBOL CChmodCommand final : public CCommandHelper<CChmodCommand, Command::chmod>
 {
 public:
 	// The permission string should be given in a format understandable by the server.
-	// Most likely it's the defaut octal representation used by the unix chmod command,
+	// Most likely it's the default octal representation used by the unix chmod command,
 	// i.e. chmod 755 foo.bar
 	CChmodCommand(CServerPath const& path, std::wstring const& file, std::wstring const& permission);
 
