@@ -1,6 +1,8 @@
 #ifndef FILEZILLA_ENGINE_LOCAL_PATH_HEADER
 #define FILEZILLA_ENGINE_LOCAL_PATH_HEADER
 
+#include "visibility.h"
+
 #include <libfilezilla/shared.hpp>
 
 // This class encapsulates local paths.
@@ -8,7 +10,7 @@
 // UNC paths.
 // On all other systems it uses /foo/bar/baz/
 
-class CLocalPath final
+class FZC_PUBLIC_SYMBOL CLocalPath final
 {
 public:
 	CLocalPath() = default;
@@ -27,6 +29,8 @@ public:
 	// Always terminated by a separator
 	std::wstring const& GetPath() const { return *m_path; }
 
+	explicit operator bool() const { return !empty(); }
+
 	bool empty() const;
 	void clear();
 
@@ -63,9 +67,6 @@ public:
 	// Checks if the directory exists.
 	bool Exists(std::wstring *error = nullptr) const;
 
-	// Craetes direcory if it doesn't yet exist
-	bool Create(CLocalPath *last_successful = nullptr);
-
 	static wchar_t const path_separator;
 
 	bool operator==(CLocalPath const& op) const;
