@@ -3,8 +3,11 @@
 
 #include <map>
 #include <memory>
+#include <string>
 #include <vector>
 
+#include "visibility.h"
+
 enum ServerProtocol
 {
 	// Never change any existing values or user's saved sites will become
@@ -42,7 +45,9 @@ enum ServerProtocol
 
 	INSECURE_WEBDAV,
 
-	MAX_VALUE = INSECURE_WEBDAV
+	RACKSPACE,
+
+	MAX_VALUE = RACKSPACE
 };
 
 enum ServerType
@@ -73,6 +78,7 @@ enum class ServerFormat
 {
 	host_only,
 	with_optional_port,
+	with_port,
 	with_user_and_optional_port,
 	url,
 	url_with_password
@@ -100,7 +106,8 @@ enum class ProtocolFeature
 	ServerAssignedHome,
 	TemporaryUrl,
 	S3Sse,
-	Security // Encryption, integrity protection and authentication
+	Security, // Encryption, integrity protection and authentication
+	UnixChmod
 };
 
 enum class CaseSensitivity
@@ -114,7 +121,7 @@ CaseSensitivity GetCaseSensitivity(ServerProtocol protocol);
 
 class Credentials;
 class CServerPath;
-class CServer final
+class FZC_PUBLIC_SYMBOL CServer final
 {
 public:
 
@@ -167,7 +174,6 @@ public:
 	std::wstring Format(ServerFormat formatType, Credentials const& credentials) const;
 
 	bool SetEncodingType(CharsetEncoding type, std::wstring const& encoding = std::wstring());
-	bool SetCustomEncoding(std::wstring const& encoding);
 	CharsetEncoding GetEncodingType() const;
 	std::wstring GetCustomEncoding() const;
 
@@ -209,16 +215,16 @@ protected:
 	ServerProtocol m_protocol{UNKNOWN};
 	ServerType m_type{DEFAULT};
 	std::wstring m_host;
-	unsigned int m_port{21};
 	std::wstring m_user;
+	unsigned int m_port{21};
 	int m_timezoneOffset{};
 	PasvMode m_pasvMode{MODE_DEFAULT};
 	int m_maximumMultipleConnections{};
+	bool m_bypassProxy{};
 	CharsetEncoding m_encodingType{ENCODING_AUTO};
 	std::wstring m_customEncoding;
 
 	std::vector<std::wstring> m_postLoginCommands;
-	bool m_bypassProxy{};
 
 	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
@@ -235,10 +241,10 @@ enum class LogonType
 
 	count
 };
-std::wstring GetNameFromLogonType(LogonType type);
-LogonType GetLogonTypeFromName(std::wstring const& name);
+std::wstring FZC_PUBLIC_SYMBOL GetNameFromLogonType(LogonType type);
+LogonType FZC_PUBLIC_SYMBOL GetLogonTypeFromName(std::wstring const& name);
 
-std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol);
+std::vector<LogonType> FZC_PUBLIC_SYMBOL GetSupportedLogonTypes(ServerProtocol protocol);
 
 
 namespace ParameterSection {
@@ -268,13 +274,13 @@ struct ParameterTraits
 	std::wstring hint_;
 };
 
-std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol protocol);
+std::vector<ParameterTraits> FZC_PUBLIC_SYMBOL const& ExtraServerParameterTraits(ServerProtocol protocol);
 
-std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol);
+std::tuple<std::wstring, std::wstring> FZC_PUBLIC_SYMBOL GetDefaultHost(ServerProtocol protocol);
 
-bool ProtocolHasUser(ServerProtocol protocol);
+bool FZC_PUBLIC_SYMBOL ProtocolHasUser(ServerProtocol protocol);
 
-class Credentials
+class FZC_PUBLIC_SYMBOL Credentials
 {
 public:
 	virtual ~Credentials() = default;
@@ -305,7 +311,7 @@ protected:
 	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
 
-struct ServerHandleData {
+struct FZC_PUBLIC_SYMBOL ServerHandleData {
 protected:
 	ServerHandleData() = default;
 	virtual ~ServerHandleData() = default;
