@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #ifdef _MSC_VER
 #pragma hdrstop
 #endif
@@ -12,13 +12,14 @@
 #include "msgbox.h"
 #include "themeprovider.h"
 #include "wxfilesystem_blob_handler.h"
+#include "../include/version.h"
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/translate.hpp>
 
 #include <wx/evtloop.h>
 
 #ifdef WITH_LIBDBUS
-#include <../dbus/session_manager.h>
+#include "../dbus/session_manager.h"
 #endif
 
 #if defined(__WXMAC__) || defined(__UNIX__)
@@ -33,7 +34,7 @@
 #include "osx_sandbox_userdirs.h"
 #endif
 
-#ifndef __WXGTK__
+#if !defined(__WXGTK__) && !defined(__MINGW32__)
 IMPLEMENT_APP(CFileZillaApp)
 #else
 IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
@@ -44,22 +45,12 @@ IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 #endif
 
 namespace {
-std::wstring GetEnv(std::string const& name)
-{
-	std::wstring ret;
-
-	char const* const p = getenv(name.c_str());
-	if (p && *p) {
-		ret = fz::to_wstring(std::string(p));
-	}
-
-	return ret;
-}
-
 #if FZ_WINDOWS
 std::wstring const PATH_SEP = L";";
+#define L_DIR_SEP L"\\"
 #else
 std::wstring const PATH_SEP = L":";
+#define L_DIR_SEP L"/"
 #endif
 }
 
@@ -126,12 +117,13 @@ CFileZillaApp::CFileZillaApp()
 
 CFileZillaApp::~CFileZillaApp()
 {
+	themeProvider_.reset();
 	COptions::Destroy();
 }
 
 void CFileZillaApp::InitLocale()
 {
-	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
+	wxString language = COptions::Get()->get_string(OPTION_LANGUAGE);
 	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
 	if (!language.empty()) {
 #ifdef __WXGTK__
@@ -169,18 +161,18 @@ void CFileZillaApp::InitLocale()
 			error += _("Please make sure the requested locale is installed on your system.");
 			wxMessageBoxEx(error, _("Failed to change language"), wxICON_EXCLAMATION);
 
-			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
+			COptions::Get()->set(OPTION_LANGUAGE, _T(""));
 		}
 #else
 		if (!pInfo || !SetLocale(pInfo->Language)) {
 			for (language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language)) {
 				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
 				if (fallbackInfo && SetLocale(fallbackInfo->Language)) {
-					COptions::Get()->SetOption(OPTION_LANGUAGE, language.ToStdWstring());
+					COptions::Get()->set(OPTION_LANGUAGE, language.ToStdWstring());
 					return;
 				}
 			}
-			COptions::Get()->SetOption(OPTION_LANGUAGE, std::wstring());
+			COptions::Get()->set(OPTION_LANGUAGE, std::wstring());
 			if (pInfo && !pInfo->Description.empty()) {
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
 			}
@@ -255,7 +247,7 @@ bool CFileZillaApp::OnInit()
 
 #if USE_MAC_SANDBOX
 	// Set PUTTYDIR so that fzsftp uses the sandboxed home to put settings.
-	std::wstring home = GetEnv("Home");
+	std::wstring home = GetEnv("HOME");
 	if (!home.empty()) {
 		if (home.back() != '/') {
 			home += '/';
@@ -283,8 +275,8 @@ USE AT OWN RISK"), _T("Important Information"));
 	else {
 		std::wstring v = GetEnv("FZDEBUG");
 		if (v != L"1") {
-			COptions::Get()->SetOption(OPTION_LOGGING_DEBUGLEVEL, 0);
-			COptions::Get()->SetOption(OPTION_LOGGING_RAWLISTING, 0);
+			COptions::Get()->set(OPTION_LOGGING_DEBUGLEVEL, 0);
+			COptions::Get()->set(OPTION_LOGGING_RAWLISTING, 0);
 		}
 	}
 #endif
@@ -345,7 +337,7 @@ int CFileZillaApp::OnExit()
 {
 	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUITNOW);
 
-	COptions::Get()->SaveIfNeeded();
+	COptions::Get()->Save();
 
 #ifdef WITH_LIBDBUS
 	CSessionManager::Uninit();
@@ -419,15 +411,15 @@ CLocalPath CFileZillaApp::GetDataDir(std::vector<std::wstring> const& fileToFind
 			return ret;
 		}
 
-		if (!prefixSub.empty() && selfDir.size() > 5 && fz::ends_with(selfDir, std::wstring(L"/bin/"))) {
-			std::wstring path = selfDir.substr(0, selfDir.size() - 4) + prefixSub + L"/";
+		if (!prefixSub.empty() && selfDir.size() > 5 && fz::ends_with(selfDir, std::wstring(L_DIR_SEP L"bin" L_DIR_SEP))) {
+			std::wstring path = selfDir.substr(0, selfDir.size() - 4) + prefixSub + L_DIR_SEP;
 			if (testPath(path)) {
 				return ret;
 			}
 		}
 
 		// Development paths
-		if (searchSelfDir && selfDir.size() > 7 && fz::ends_with(selfDir, std::wstring(L"/.libs/"))) {
+		if (searchSelfDir && selfDir.size() > 7 && fz::ends_with(selfDir, std::wstring(L_DIR_SEP L".libs" L_DIR_SEP))) {
 			std::wstring path = selfDir.substr(0, selfDir.size() - 6);
 			if (FileExists(path + L"Makefile")) {
 				if (testPath(path)) {
@@ -444,8 +436,8 @@ CLocalPath CFileZillaApp::GetDataDir(std::vector<std::wstring> const& fileToFind
 
 		for (auto const& segment : segments) {
 			auto const cur = CLocalPath(segment).GetPath();
-			if (cur.size() > 5 && fz::ends_with(cur, std::wstring(L"/bin/"))) {
-				std::wstring path = cur.substr(0, cur.size() - 4) + prefixSub + L"/";
+			if (cur.size() > 5 && fz::ends_with(cur, std::wstring(L_DIR_SEP L"bin" L_DIR_SEP))) {
+				std::wstring path = cur.substr(0, cur.size() - 4) + prefixSub + L_DIR_SEP;
 				if (testPath(path)) {
 					return ret;
 				}
@@ -616,7 +608,7 @@ void CFileZillaApp::CheckExistsFzstorj()
 }
 #endif
 
-void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, std::string const& env, int setting, std::wstring const& description)
+void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, char const* env, engineOptions setting, std::wstring const& description)
 {
 	// Get the correct path to the specified tool
 
@@ -701,7 +693,7 @@ void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::wstring const
 			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
-	COptions::Get()->SetOption(setting, executable);
+	COptions::Get()->set(setting, executable);
 }
 
 #ifdef __WXMSW__
@@ -731,7 +723,7 @@ int CFileZillaApp::ProcessCommandLine()
 		}
 
 		if (m_pCommandLine->HasSwitch(CCommandLine::version)) {
-			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
+			wxString out = wxString::Format(_T("FileZilla %s"), GetFileZillaVersion());
 			if (!CBuildInfo::GetBuildType().empty()) {
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
 			}
@@ -793,5 +785,17 @@ void CFileZillaApp::ShowStartupProfile()
 
 std::wstring CFileZillaApp::GetSettingsFile(std::wstring const& name) const
 {
-	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
+	return COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
 }
+
+#if defined(__MINGW32__)
+extern "C" {
+__declspec(dllexport) // This forces ld to not strip relocations so that ASLR can work on MSW.
+int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR, int nCmdShow)
+{
+	wxDISABLE_DEBUG_SUPPORT();
+	return wxEntry(hInstance, hPrevInstance, NULL, nCmdShow);
+}
+}
+#endif
+
