@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
@@ -28,8 +28,11 @@
 #include "timeformatting.h"
 
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/process.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
+#include <wx/menu.h>
+
 class CLocalListViewDropTarget final : public CFileDropTarget<wxListCtrlEx>
 {
 public:
@@ -306,7 +309,7 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 CLocalListView::~CLocalListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
+	COptions::Get()->set(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
 	volumeEnumeratorThread_.reset();
@@ -415,10 +418,12 @@ regular_dir:
 
 		int num = m_fileData.size();
 		CLocalFileData data;
-		bool wasLink;
+		bool wasLink{};
+		fz::local_filesys::type t{};
 		fz::native_string name;
-		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+		while (local_filesys.get_next_file(name, wasLink, t, &data.size, &data.time, &data.attributes)) {
 			data.name = fz::to_wstring(name);
+			data.dir = t == fz::local_filesys::dir;
 			if (name.empty() || data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
@@ -544,7 +549,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	}
 
 	if (data->dir) {
-		const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+		const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 		if (action == 3) {
 			// No action
 			wxBell();
@@ -576,7 +581,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_FILE);
+	const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_FILE);
 	if (action == 3) {
 		// No action
 		wxBell();
@@ -761,7 +766,7 @@ bool CLocalListView::IsItemValid(unsigned int item) const
 std::unique_ptr<CFileListCtrlSortBase> CLocalListView::GetSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
-	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
+	NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection) {
 		if (m_sortColumn == 1) {
@@ -825,7 +830,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		
 	const bool connected = m_state.IsRemoteConnected();
 	if (!connected) {
-		menu.Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
+		menu.Enable(XRCID("ID_EDIT"), COptions::Get()->get_int(OPTION_EDIT_TRACK_LOCAL) == 0);
 		menu.Enable(XRCID("ID_UPLOAD"), false);
 		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -1431,7 +1436,7 @@ void CLocalListView::SetInfoText(wxString const& text)
 
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -1710,7 +1715,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
+bool CLocalListView::get_next_file(std::wstring_view & name, std::wstring &, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -1742,11 +1747,9 @@ void CLocalListView::FinishComparison()
 	RefreshListOnly();
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther) {
-		return;
+	if (pOther) {
+		pOther->ScrollTopItem(GetTopItem());
 	}
-
-	pOther->ScrollTopItem(GetTopItem());
 }
 
 bool CLocalListView::CanStartComparison()
@@ -1803,7 +1806,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	CServerPath path;
 
 	if (!m_state.GetSite()) {
-		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
+		if (COptions::Get()->get_int(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
@@ -1910,33 +1913,30 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 			continue;
 		}
 
+
 		wxFileName fn(m_dir.GetPath(), data.name);
 		if (wxLaunchDefaultApplication(fn.GetFullPath(), 0)) {
 			continue;
 		}
-		bool program_exists = false;
-		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
-		if (cmd.empty()) {
-			auto pos = data.name.find('.');
-			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
-				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
-			}
+		auto cmd_with_args = GetSystemAssociation(fn.GetFullPath().ToStdWstring());
+		if (cmd_with_args.empty()) {
+			cmd_with_args = pEditHandler->GetAssociation(fn.GetFullPath().ToStdWstring());
 		}
-		if (cmd.empty()) {
+		if (cmd_with_args.empty()) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 			continue;
 		}
-		if (!program_exists) {
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
+		if (!ProgramExists(cmd_with_args.front())) {
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd_with_args.front());
 			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
 			continue;
 		}
 
-		if (wxExecute(cmd)) {
+		if (fz::spawn_detached_process(AssociationToCommand(cmd_with_args, fn.GetFullPath().ToStdWstring()))) {
 			continue;
 		}
-
 		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
+
 	}
 }
 
