@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #include "dndobjects.h"
 #include "dragdropmanager.h"
@@ -15,6 +15,8 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <wx/menu.h>
+
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
 #include <shlobj.h>
@@ -248,10 +250,11 @@ EVT_MENU(XRCID("ID_OPEN"), CLocalTreeView::OnMenuOpen)
 END_EVENT_TABLE()
 
 CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView *pQueueView)
-	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxNO_BORDER),
-	CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x),
-	CStateEventHandler(state),
-	m_pQueueView(pQueueView)
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxNO_BORDER)
+	, CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x)
+	, CStateEventHandler(state)
+	, COptionChangeEventHandler(this)
+	, m_pQueueView(pQueueView)
 {
 	wxGetApp().AddStartupProfileRecord("CLocalTreeView::CLocalTreeView");
 #ifdef __WXMAC__
@@ -265,7 +268,7 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 	SetImageList(GetSystemImageList());
 
 	UpdateSortMode();
-	RegisterOption(OPTION_FILELIST_NAMESORT);
+	COptions::Get()->watch(OPTION_FILELIST_NAMESORT, this);
 
 #ifdef __WXMSW__
 	m_pVolumeEnumeratorThread = 0;
@@ -295,6 +298,7 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 
 CLocalTreeView::~CLocalTreeView()
 {
+	COptions::Get()->unwatch_all(this);
 #ifdef __WXMSW__
 	delete m_pVolumeEnumeratorThread;
 #endif
@@ -492,15 +496,14 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, std::wstring const& dirname
 	bool matchedKnown = false;
 
 	fz::native_string file;
-	bool wasLink;
-	int attributes;
-	bool is_dir;
+	bool wasLink{};
+	int attributes{};
+	fz::local_filesys::type t{};
 	static int64_t const size(-1);
 	fz::datetime date;
 
 	wxTreeItemId last;
-	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
-		wxASSERT(is_dir);
+	while (local_filesys.get_next_file(file, wasLink, t, 0, &date, &attributes)) {
 		std::wstring wfile = fz::to_wstring(file);
 		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
@@ -561,13 +564,13 @@ std::wstring CLocalTreeView::HasSubdir(std::wstring const& dirname)
 	}
 
 	fz::native_string file;
-	bool wasLink;
-	int attributes;
-	bool is_dir;
+	bool wasLink{};
+	int attributes{};
+	fz::local_filesys::type t{};
 	static int64_t const size(-1);
 	fz::datetime date;
-	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
-		wxASSERT(is_dir);
+	while (local_filesys.get_next_file(file, wasLink, t, 0, &date, &attributes)) {
+
 		std::wstring wfile = fz::to_wstring(file);
 		if (file.empty() || wfile.empty()) {
 			wxGetApp().DisplayEncodingWarning();
@@ -694,18 +697,18 @@ std::wstring CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 
 void CLocalTreeView::UpdateSortMode()
 {
-	CFileListCtrlSortBase::NameSortMode sortMode;
-	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	NameSortMode sortMode;
+	switch (COptions::Get()->get_int(OPTION_FILELIST_NAMESORT))
 	{
 	case 0:
 	default:
-		sortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		sortMode = NameSortMode::case_insensitive;
 		break;
 	case 1:
-		sortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		sortMode = NameSortMode::case_sensitive;
 		break;
 	case 2:
-		sortMode = CFileListCtrlSortBase::namesort_natural;
+		sortMode = NameSortMode::natural;
 		break;
 	}
 	sortFunction_ = CFileListCtrlSortBase::GetCmpFunction(sortMode);
@@ -784,11 +787,11 @@ void CLocalTreeView::RefreshListing()
 
 		fz::native_string file;
 		static int64_t const size(-1);
-		bool was_link;
-		bool is_dir;
-		int attributes;
+		bool was_link{};
+		fz::local_filesys::type t{};
+		int attributes{};
 		fz::datetime date;
-		while (local_filesys.get_next_file(file, was_link, is_dir, 0, &date, &attributes)) {
+		while (local_filesys.get_next_file(file, was_link, t, 0, &date, &attributes)) {
 			std::wstring wfile = fz::to_wstring(file);
 			if (file.empty() || wfile.empty()) {
 				wxGetApp().DisplayEncodingWarning();
@@ -931,7 +934,7 @@ void CLocalTreeView::OnStateChange(t_statechange_notifications notification, std
 
 void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 {
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -1663,7 +1666,7 @@ void CLocalTreeView::OnMenuOpen(wxCommandEvent&)
 	OpenInFileManager(path);
 }
 
-void CLocalTreeView::OnOptionsChanged(changed_options_t const& options)
+void CLocalTreeView::OnOptionsChanged(watched_options const& options)
 {
 	if (options.test(OPTION_FILELIST_NAMESORT)) {
 		UpdateSortMode();
