@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "Mainfrm.h"
 
 #include "aboutdialog.h"
@@ -51,6 +51,13 @@
 #include "viewheader.h"
 #include "welcome_dialog.h"
 #include "window_state_manager.h"
+#include "../include/version.h"
+
+#if FZ_MANUALUPDATECHECK
+#include "overlay.h"
+#include <wx/hyperlink.h>
+#include <wx/mstream.h>
+#endif
 
 #ifdef __WXMSW__
 #include <wx/module.h>
@@ -66,8 +73,8 @@
 #include <map>
 
 #ifdef __WXGTK__
-DECLARE_EVENT_TYPE(fzEVT_TASKBAR_CLICK_DELAYED, -1)
-DEFINE_EVENT_TYPE(fzEVT_TASKBAR_CLICK_DELAYED)
+wxDECLARE_EVENT(fzEVT_TASKBAR_CLICK_DELAYED, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_TASKBAR_CLICK_DELAYED, wxCommandEvent);
 #endif
 
 static int tab_hotkey_ids[10];
@@ -87,7 +94,7 @@ wxTextEntry* GetSpecialTextEntry(wxWindow* w, wxChar cmd)
 {
 	if (cmd == 'A' || cmd == 'V') {
 		wxTextCtrl* text = dynamic_cast<wxTextCtrl*>(w);
-		if (text && text->GetWindowStyle() & wxTE_PASSWORD) {
+		if (text) {
 			return text;
 		}
 	}
@@ -250,51 +257,110 @@ protected:
 	CMainFrame* m_pMainFrame;
 };
 
-
-/*#include "overlay.h"
-#include <wx/hyperlink.h>
-
+#if FZ_MANUALUPDATECHECK
 namespace {
-void ShowStorjOverlay(wxWindow* parent, wxWindow* anchor, wxPoint const& offset)
+void ShowOverlay(std::wstring const& data, wxTopLevelWindow* parent, wxWindow* anchor, wxPoint const& offset)
 {
+	if (data.empty() || CBuildInfo::GetBuildType() != _T("official") || COptions::Get()->get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
+		return;
+	}
+
+	auto tokens = fz::strtok_view(data, ' ', false);
+	if (tokens.size() < 4) {
+		return;
+	}
+
+	if (tokens[0] != L"1") {
+		return;
+	}
+
+	int const id = fz::to_integral<int>(tokens[1]);
+	if (COptions::Get()->get_int(OPTION_SHOWN_OVERLAY) >= id) {
+		return;
+	}
+
+	if (fz::datetime(fz::to_integral<time_t>(tokens[2]), fz::datetime::seconds) < fz::datetime::now()) {
+		return;
+	}
+
+	auto linedata = fz::to_wstring_from_utf8(fz::base64_decode_s(tokens[3]));
+	auto lines = fz::strtok_view(linedata, '\n', false);
+
+	if (lines.empty()) {
+		return;
+	}
+
+	DialogLayout lay(parent);
+
 	auto p = new OverlayWindow(parent);
 
 	auto box = new wxBoxSizer(wxVERTICAL);
-	auto sizer = layout::createFlex(1);
-	box->Add(sizer, 0, wxALL, 7);
-
 	p->SetSizer(box);
+	
+	auto outer = lay.createFlex(1);
+	box->Add(outer, 0, wxALL, 7);
 
-	auto title = new wxStaticText(p, -1, _("Did you know about this new FileZilla feature?"));
+	auto sides = lay.createFlex(0, 1);
+	outer->Add(sides);
+	
+	auto right = lay.createFlex(1);
+	sides->Add(right, lay.valign);
+	if (tokens.size() > 4) {
+		auto raw = fz::base64_decode_s(tokens[4]);
+		wxMemoryInputStream s(raw.data(), raw.size());
+		wxImage img;
+		if (img.LoadFile(s, wxBITMAP_TYPE_ANY)) {
+			sides->Add(new wxStaticBitmap(p, nullID, wxBitmap(img)), lay.valign);
+		}
+	}
 
-	wxFont f = title->GetFont();
-	f.SetWeight(wxBOLD);
-	title->SetFont(f);
-	sizer->Add(title);
 
-	sizer->AddSpacer(0);
+	for (auto line : lines) {
+		if (line.empty()) {
+			right->AddSpacer(0);
+		}
+		else {
+			bool bold{};
+			if (line.front() == '+') {
+				line = line.substr(1);
+				bold = true;
+			}
+			wxWindow* ctrl{};
+			if (line.front() == '>') {
+				size_t pos = line.find(' ');
+				ctrl = new wxHyperlinkCtrl(p, nullID, std::wstring(line.substr((pos == std::wstring::npos) ? 1 : pos + 1)), std::wstring(line.substr(1, pos)));
+			}
+			else {
+				ctrl = new wxStaticText(p, nullID, std::wstring(line));
+			}
+			if (bold) {
+				wxFont f = ctrl->GetFont();
+				f.SetWeight(wxBOLD);
+				ctrl->SetFont(f);
+			}
+			right->Add(ctrl);
+		}
+	}
+	outer->AddSpacer(0);
 
-	sizer->Add(new wxStaticText(p, -1, _("You can use FileZilla to store your files securely in the Storj decentralized cloud.")));
-	sizer->Add(new wxHyperlinkCtrl(p, -1, _("Click to learn more"), L"https://wiki.filezilla-project.org/Storj"));
+	auto dismiss = new wxButton(p, nullID, L"Dismiss");
+	dismiss->Bind(wxEVT_BUTTON, [p, id](wxCommandEvent&) {
+		COptions::Get()->set(OPTION_SHOWN_OVERLAY, id);
+		p->Destroy();
+	});
 
-	sizer->AddSpacer(10);
-
-	//sizer->Add(new wxCheckBox(p, -1, L"Don't show this again"), 0, wxALIGN_RIGHT);
-
-	auto dismiss = new wxButton(p, -1, L"Dismiss");
-	dismiss->Bind(wxEVT_BUTTON, [p](wxCommandEvent&) { p->Destroy(); });
-	sizer->Add(dismiss, 0, wxALIGN_RIGHT);
-
-	wxGetApp().GetWrapEngine()->WrapRecursive(p, 3);
+	outer->Add(dismiss, 0, wxALIGN_RIGHT);
 
 	box->Fit(p);
 
 	p->SetAnchor(anchor, offset);
 }
-}*/
+}
+#endif
 
 CMainFrame::CMainFrame()
-	: m_engineContext(*COptions::Get(), CustomEncodingConverter::Get())
+	: COptionChangeEventHandler(this)
+	, m_engineContext(*COptions::Get(), CustomEncodingConverter::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
 {
 	m_pActivityLed[0] = m_pActivityLed[1] = 0;
@@ -345,14 +411,14 @@ CMainFrame::CMainFrame()
 	m_closeEventTimer.SetOwner(this);
 
 	if (CFilterManager::HasActiveFilters(true)) {
-		if (COptions::Get()->GetOptionVal(OPTION_FILTERTOGGLESTATE)) {
+		if (COptions::Get()->get_int(OPTION_FILTERTOGGLESTATE)) {
 			CFilterManager::ToggleFilters();
 		}
 	}
 
 	CreateMenus();
 	CreateMainToolBar();
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT)) {
+	if (COptions::Get()->get_int(OPTION_SHOW_QUICKCONNECT)) {
 		CreateQuickconnectBar();
 	}
 
@@ -375,7 +441,7 @@ CMainFrame::CMainFrame()
 	m_pBottomSplitter->SetMinimumPaneSize(20, 70);
 	m_pBottomSplitter->SetSashGravity(1.0);
 
-	const int message_log_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
+	const int message_log_position = COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION);
 	m_pQueueLogSplitter = new CSplitterWindowEx(m_pBottomSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER | wxSP_LIVE_UPDATE);
 	m_pQueueLogSplitter->SetMinimumPaneSize(50, 250);
 	m_pQueueLogSplitter->SetSashGravity(0.5);
@@ -399,8 +465,8 @@ CMainFrame::CMainFrame()
 	switch (message_log_position) {
 	case 1:
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
-		if (COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG)) {
-			if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE)) {
+		if (COptions::Get()->get_int(OPTION_SHOW_MESSAGELOG)) {
+			if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
 				m_pQueueLogSplitter->SplitVertically(m_pQueuePane, m_pStatusView);
 			}
 			else {
@@ -409,7 +475,7 @@ CMainFrame::CMainFrame()
 			}
 		}
 		else {
-			if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE)) {
+			if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
 				m_pStatusView->Hide();
 				m_pQueueLogSplitter->Initialize(m_pQueuePane);
 			}
@@ -422,7 +488,7 @@ CMainFrame::CMainFrame()
 		break;
 	case 2:
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
-		if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE)) {
+		if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
 		}
 		else {
@@ -432,14 +498,14 @@ CMainFrame::CMainFrame()
 		m_pQueuePane->AddPage(m_pStatusView, _("Message log"));
 		break;
 	default:
-		if (COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE)) {
+		if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
 		}
 		else {
 			m_pQueuePane->Hide();
 			m_pQueueLogSplitter->Hide();
 		}
-		if (COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG)) {
+		if (COptions::Get()->get_int(OPTION_SHOW_MESSAGELOG)) {
 			m_pTopSplitter->SplitHorizontally(m_pStatusView, m_pBottomSplitter);
 		}
 		else {
@@ -479,16 +545,16 @@ CMainFrame::CMainFrame()
 
 	FixTabOrder();
 
-	RegisterOption(OPTION_ICONS_THEME);
-	RegisterOption(OPTION_ICONS_SCALE);
-	RegisterOption(OPTION_MESSAGELOG_POSITION);
-	RegisterOption(OPTION_FILEPANE_LAYOUT);
-	RegisterOption(OPTION_FILEPANE_SWAP);
+	COptions::Get()->watch(OPTION_ICONS_THEME, this);
+	COptions::Get()->watch(OPTION_ICONS_SCALE, this);
+	COptions::Get()->watch(OPTION_MESSAGELOG_POSITION, this);
+	COptions::Get()->watch(OPTION_FILEPANE_LAYOUT, this);
+	COptions::Get()->watch(OPTION_FILEPANE_SWAP, this);
 }
 
 CMainFrame::~CMainFrame()
 {
-	UnregisterAllOptions();
+	COptions::Get()->unwatch_all(this);
 
 	CPowerManagement::Destroy();
 
@@ -574,7 +640,7 @@ bool CMainFrame::CreateMenus()
 	return true;
 }
 
-bool CMainFrame::CreateQuickconnectBar()
+void CMainFrame::CreateQuickconnectBar()
 {
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateQuickconnectBar");
 	delete m_pQuickconnectBar;
@@ -592,8 +658,6 @@ bool CMainFrame::CreateQuickconnectBar()
 		}
 		m_pQuickconnectBar->SetSize(0, 0, clientSize.GetWidth(), -1);
 	}
-
-	return true;
 }
 
 void CMainFrame::OnMenuHandler(wxCommandEvent &event)
@@ -679,19 +743,15 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 	else if (event.GetId() == XRCID("ID_CLEAR_UPDATER")) {
 #if FZ_MANUALUPDATECHECK
 		if (m_pUpdater) {
-			COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, std::wstring());
-			COptions::Get()->SetOption(OPTION_UPDATECHECK_NEWVERSION, std::wstring());
+			COptions::Get()->set(OPTION_UPDATECHECK_LASTDATE, std::wstring());
+			COptions::Get()->set(OPTION_UPDATECHECK_NEWVERSION, std::wstring());
 			m_pUpdater->Init();
 		}
 #endif
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_FILEEXISTS")) {
 		CDefaultFileExistsDlg dlg;
-		if (!dlg.Load(this, false)) {
-			return;
-		}
-
-		dlg.Run();
+		dlg.Run(this, false);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_EDIT_CLEARPRIVATEDATA")) {
 		CClearPrivateDataDialog* pDlg = CClearPrivateDataDialog::Create(this);
@@ -710,7 +770,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_VIEWHIDDEN")) {
-		bool showHidden = COptions::Get()->GetOptionVal(OPTION_VIEW_HIDDEN_FILES) ? 0 : 1;
+		bool showHidden = COptions::Get()->get_int(OPTION_VIEW_HIDDEN_FILES) ? 0 : 1;
 		if (showHidden) {
 			CConditionalDialog dlg(this, CConditionalDialog::viewhidden, CConditionalDialog::ok, false);
 			dlg.SetTitle(_("Force showing hidden files"));
@@ -722,7 +782,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			(void)dlg.Run();
 		}
 
-		COptions::Get()->SetOption(OPTION_VIEW_HIDDEN_FILES, showHidden ? 1 : 0);
+		COptions::Get()->set(OPTION_VIEW_HIDDEN_FILES, showHidden ? 1 : 0);
 		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 		for (auto & pState : *pStates) {
 			CServerPath path = pState->GetRemotePath();
@@ -744,13 +804,13 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		dlg.ShowModal();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_AUTO")) {
-		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 0);
+		COptions::Get()->set(OPTION_ASCIIBINARY, 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_ASCII")) {
-		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 1);
+		COptions::Get()->set(OPTION_ASCIIBINARY, 1);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_BINARY")) {
-		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 2);
+		COptions::Get()->set(OPTION_ASCIIBINARY, 2);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_PRESERVETIMES")) {
 		if (event.IsChecked()) {
@@ -759,7 +819,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			dlg.AddText(_("Please note that preserving timestamps on uploads on FTP, FTPS and FTPES servers only works if they support the MFMT command."));
 			dlg.Run();
 		}
-		COptions::Get()->SetOption(OPTION_PRESERVE_TIMESTAMPS, event.IsChecked() ? 1 : 0);
+		COptions::Get()->set(OPTION_PRESERVE_TIMESTAMPS, event.IsChecked() ? 1 : 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_PROCESSQUEUE")) {
 		if (m_pQueueView) {
@@ -776,7 +836,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		else {
 			url += _T("bugreport");
 		}
-		wxString version = CBuildInfo::GetVersion();
+		wxString version = GetFileZillaVersion();
 		if (version != _T("custom build")) {
 			url += _T("&version=");
 			// We need to urlencode version number
@@ -799,8 +859,8 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		wxLaunchDefaultBrowser(url);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_VIEW_FILELISTSTATUSBAR")) {
-		bool show = COptions::Get()->GetOptionVal(OPTION_FILELIST_STATUSBAR) == 0;
-		COptions::Get()->SetOption(OPTION_FILELIST_STATUSBAR, show ? 1 : 0);
+		bool show = COptions::Get()->get_int(OPTION_FILELIST_STATUSBAR) == 0;
+		COptions::Get()->set(OPTION_FILELIST_STATUSBAR, show ? 1 : 0);
 		CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 		if (controls && controls->pLocalListViewPanel) {
 			wxStatusBar* pStatusBar = controls->pLocalListViewPanel->GetStatusBar();
@@ -829,7 +889,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			wxSize clientSize = GetClientSize();
 			m_pTopSplitter->SetSize(0, 0, clientSize.GetWidth(), clientSize.GetHeight());
 		}
-		COptions::Get()->SetOption(OPTION_SHOW_QUICKCONNECT, m_pQuickconnectBar != 0);
+		COptions::Get()->set(OPTION_SHOW_QUICKCONNECT, m_pQuickconnectBar != 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_MANUAL")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
@@ -883,16 +943,16 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		dlg.Run(this, true);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_ENABLE")) {
-		bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) == 0;
+		bool enable = COptions::Get()->get_int(OPTION_SPEEDLIMIT_ENABLE) == 0;
 
-		const int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
-		const int uploadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
+		const int downloadLimit = COptions::Get()->get_int(OPTION_SPEEDLIMIT_INBOUND);
+		const int uploadLimit = COptions::Get()->get_int(OPTION_SPEEDLIMIT_OUTBOUND);
 		if (enable && !downloadLimit && !uploadLimit) {
 			CSpeedLimitsDialog dlg;
 			dlg.Run(this);
 		}
 		else {
-			COptions::Get()->SetOption(OPTION_SPEEDLIMIT_ENABLE, enable ? 1 : 0);
+			COptions::Get()->set(OPTION_SPEEDLIMIT_ENABLE, enable ? 1 : 0);
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_CONFIGURE")) {
@@ -905,8 +965,8 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		int old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
-		COptions::Get()->SetOption(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
+		int old_mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
+		COptions::Get()->set(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
 
 		CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 		if (pComparisonManager && pComparisonManager->IsComparing()) {
@@ -947,16 +1007,11 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 }
 
 void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
-{
-	CallAfter(&CMainFrame::DoOnEngineEvent, engine);
-}
-
-void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 {
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	CState* pState = 0;
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
-		if ((*iter)->m_pEngine != engine) {
+		if ((*iter)->engine_.get() != engine) {
 			continue;
 		}
 
@@ -967,15 +1022,15 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 		return;
 	}
 
-	std::unique_ptr<CNotification> pNotification = pState->m_pEngine->GetNextNotification();
+	std::unique_ptr<CNotification> pNotification = pState->engine_->GetNextNotification();
 	while (pNotification) {
 		switch (pNotification->GetID())
 		{
 		case nId_logmsg:
 			if (m_pStatusView) {
-				m_pStatusView->AddToLog(static_cast<CLogmsgNotification&>(*pNotification.get()));
+				m_pStatusView->AddToLog(std::move(static_cast<CLogmsgNotification&>(*pNotification.get())));
 			}
-			if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2 && m_pQueuePane) {
+			if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 2 && m_pQueuePane) {
 				m_pQueuePane->Highlight(3);
 			}
 			break;
@@ -1001,7 +1056,7 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 				auto pAsyncRequest = unique_static_cast<CAsyncRequestNotification>(std::move(pNotification));
 				if (pAsyncRequest->GetRequestID() == reqId_fileexists) {
 					if (m_pQueueView) {
-						m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pAsyncRequest));
+						m_pQueueView->ProcessNotification(pState->engine_.get(), std::move(pAsyncRequest));
 					}
 				}
 				else {
@@ -1009,7 +1064,7 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 						pState->SetSecurityInfo(static_cast<CCertificateNotification&>(*pAsyncRequest));
 					}
 					if (m_pAsyncRequestQueue) {
-						m_pAsyncRequestQueue->AddRequest(pState->m_pEngine, std::move(pAsyncRequest));
+						m_pAsyncRequestQueue->AddRequest(pState->engine_.get(), std::move(pAsyncRequest));
 					}
 				}
 			}
@@ -1022,7 +1077,7 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 			break;
 		case nId_transferstatus:
 			if (m_pQueueView) {
-				m_pQueueView->ProcessNotification(pState->m_pEngine, std::move(pNotification));
+				m_pQueueView->ProcessNotification(pState->engine_.get(), std::move(pNotification));
 			}
 			break;
 		case nId_sftp_encryption:
@@ -1046,7 +1101,7 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 			break;
 		}
 
-		pNotification = pState->m_pEngine->GetNextNotification();
+		pNotification = pState->engine_.get()->GetNextNotification();
 	}
 }
 
@@ -1057,17 +1112,17 @@ void CMainFrame::OnUpdateLedTooltip(wxCommandEvent&)
 	wxFileOffset downloadSpeed = m_pQueueView ? m_pQueueView->GetCurrentDownloadSpeed() : 0;
 	wxFileOffset uploadSpeed = m_pQueueView ? m_pQueueView->GetCurrentUploadSpeed() : 0;
 
-	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
+	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->get_int(OPTION_SIZE_FORMAT));
 	if (format == CSizeFormat::bytes) {
 		format = CSizeFormat::iec;
 	}
 
 	const wxString downloadSpeedStr = CSizeFormat::Format(downloadSpeed, true, format,
-														  COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
-														  COptions::Get()->GetOptionVal(OPTION_SIZE_DECIMALPLACES));
+														  COptions::Get()->get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
+														  COptions::Get()->get_int(OPTION_SIZE_DECIMALPLACES));
 	const wxString uploadSpeedStr = CSizeFormat::Format(uploadSpeed, true, format,
-														COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
-														COptions::Get()->GetOptionVal(OPTION_SIZE_DECIMALPLACES));
+														COptions::Get()->get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
+														COptions::Get()->get_int(OPTION_SIZE_DECIMALPLACES));
 	tooltipText.Printf(_("Download speed: %s/s\nUpload speed: %s/s"), downloadSpeedStr, uploadSpeedStr);
 
 	m_pActivityLed[0]->SetToolTip(tooltipText);
@@ -1080,7 +1135,7 @@ bool CMainFrame::CreateMainToolBar()
 	if (m_pToolBar) {
 #ifdef __WXMAC__
 		if (m_pToolBar) {
-			COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+			COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
 		}
 #endif
 		SetToolBar(0);
@@ -1089,7 +1144,7 @@ bool CMainFrame::CreateMainToolBar()
 	}
 
 #ifndef __WXMAC__
-	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0) {
+	if (COptions::Get()->get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
 		return true;
 	}
 #endif
@@ -1102,7 +1157,7 @@ bool CMainFrame::CreateMainToolBar()
 	SetToolBar(m_pToolBar);
 
 #ifdef __WXMAC__
-	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0) {
+	if (COptions::Get()->get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
 		m_pToolBar->Show(false);
 	}
 #endif
@@ -1303,7 +1358,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 
 #ifdef __WXMAC__
 		if (m_pToolBar) {
-			COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+			COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
 		}
 #endif
 		m_bQuit = true;
@@ -1385,7 +1440,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	CSiteManager::ClearIdMap();
 
 	bool filters_toggled = CFilterManager::HasActiveFilters(true) && !CFilterManager::HasActiveFilters(false);
-	COptions::Get()->SetOption(OPTION_FILTERTOGGLESTATE, filters_toggled ? 1 : 0);
+	COptions::Get()->set(OPTION_FILTERTOGGLESTATE, filters_toggled ? 1 : 0);
 
 	Destroy();
 }
@@ -1553,19 +1608,19 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 
 	COptions* pOptions = COptions::Get();
 
-	wxString oldLang = pOptions->GetOption(OPTION_LANGUAGE);
+	wxString oldLang = pOptions->get_string(OPTION_LANGUAGE);
 
-	int oldShowDebugMenu = pOptions->GetOptionVal(OPTION_DEBUG_MENU) != 0;
+	int oldShowDebugMenu = pOptions->get_int(OPTION_DEBUG_MENU) != 0;
 
 	int res = dlg.ShowModal();
 	if (res != wxID_OK) {
 		return;
 	}
 
-	wxString newLang = pOptions->GetOption(OPTION_LANGUAGE);
+	wxString newLang = pOptions->get_string(OPTION_LANGUAGE);
 
 	if (oldLang != newLang ||
-		oldShowDebugMenu != pOptions->GetOptionVal(OPTION_DEBUG_MENU))
+		oldShowDebugMenu != pOptions->get_int(OPTION_DEBUG_MENU))
 	{
 		CreateMenus();
 	}
@@ -1584,7 +1639,7 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 
 	bool shown;
 
-	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1) {
+	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter) {
 			return;
 		}
@@ -1618,8 +1673,8 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 		}
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) != 2) {
-		COptions::Get()->SetOption(OPTION_SHOW_MESSAGELOG, shown);
+	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) != 2) {
+		COptions::Get()->set(OPTION_SHOW_MESSAGELOG, shown);
 	}
 }
 
@@ -1646,8 +1701,8 @@ void CMainFrame::ShowDirectoryTree(bool local, bool show)
 		return;
 	}
 
-	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
-	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
+	const int layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
+	const int swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
 	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromTabIndex(i);
 		if (!controls) {
@@ -1677,7 +1732,7 @@ void CMainFrame::ShowDirectoryTree(bool local, bool show)
 		}
 	}
 
-	COptions::Get()->SetOption(local ? OPTION_SHOW_TREE_LOCAL : OPTION_SHOW_TREE_REMOTE, show);
+	COptions::Get()->set(local ? OPTION_SHOW_TREE_LOCAL : OPTION_SHOW_TREE_REMOTE, show);
 }
 
 void CMainFrame::OnToggleQueueView(wxCommandEvent&)
@@ -1687,7 +1742,7 @@ void CMainFrame::OnToggleQueueView(wxCommandEvent&)
 	}
 
 	bool shown;
-	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1) {
+	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter) {
 			return;
 		}
@@ -1721,7 +1776,7 @@ void CMainFrame::OnToggleQueueView(wxCommandEvent&)
 		shown = m_pBottomSplitter->IsSplit();
 	}
 
-	COptions::Get()->SetOption(OPTION_SHOW_QUEUE, shown);
+	COptions::Get()->set(OPTION_SHOW_QUEUE, shown);
 }
 
 void CMainFrame::OnMenuHelpAbout(wxCommandEvent&)
@@ -1760,7 +1815,7 @@ void CMainFrame::OnCheckForUpdates(wxCommandEvent& event)
 		return;
 	}
 
-	if (event.GetId() == XRCID("ID_CHECKFORUPDATES") || (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0)) {
+	if (event.GetId() == XRCID("ID_CHECKFORUPDATES") || (!COptions::Get()->get_int(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->get_int(OPTION_UPDATECHECK) != 0)) {
 		m_pUpdater->RunIfNeeded();
 	}
 
@@ -1838,10 +1893,10 @@ void CMainFrame::TriggerUpdateDialog()
 
 void CMainFrame::UpdateLayout()
 {
-	int const layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
-	int const swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
+	int const layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
+	int const swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
 
-	int const messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
+	int const messagelog_position = COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION);
 
 	// First handle changes in message log position as it can make size of the other panes change
 	{
@@ -2046,7 +2101,7 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 	}
 
 	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
-		int action = COptions::Get()->GetOptionVal(OPTION_ALREADYCONNECTED_CHOICE);
+		int action = COptions::Get()->get_int(OPTION_ALREADYCONNECTED_CHOICE);
 		if (action < 2) {
 			wxDialogEx dlg;
 			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED"))) {
@@ -2074,7 +2129,7 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 			if (XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked()) {
 				action |= 2;
 			}
-			COptions::Get()->SetOption(OPTION_ALREADYCONNECTED_CHOICE, action);
+			COptions::Get()->set(OPTION_ALREADYCONNECTED_CHOICE, action);
 		}
 
 		if (!(action & 1)) {
@@ -2144,7 +2199,14 @@ void CMainFrame::OnNavigationKeyEvent(wxNavigationKeyEvent& event)
 
 void CMainFrame::OnChar(wxKeyEvent& event)
 {
-	if (event.GetKeyCode() != WXK_F6) {
+	if (event.GetKeyCode() == WXK_F7) {
+		auto * controls = m_pContextControl->GetCurrentControls();
+		if (controls) {
+			controls->SwitchFocusedSide();
+		}
+		return;
+	}
+	else if (event.GetKeyCode() != WXK_F6) {
 		event.Skip();
 		return;
 	}
@@ -2221,6 +2283,7 @@ void CMainFrame::RememberSplitterPositions()
 		return;
 	}
 
+	auto const positions = controls->GetSplitterPositions();
 	std::wstring const posString = fz::sprintf(
 		L"%d %d %d %d %d %d",
 		// top_pos
@@ -2231,19 +2294,19 @@ void CMainFrame::RememberSplitterPositions()
 
 		// view_pos
 		// Note that we cannot use %f, it is locale-dependent
-		static_cast<int>(controls->pViewSplitter->GetRelativeSashPosition() * 1000000000),
+		static_cast<int>(std::get<0>(positions) * 1000000000),
 
 		// local_pos
-		controls->pLocalSplitter->GetSashPosition(),
+		std::get<1>(positions),
 
 		// remote_pos
-		controls->pRemoteSplitter->GetSashPosition(),
+		std::get<2>(positions),
 
 		// queuelog splitter
 		static_cast<int>(m_pQueueLogSplitter->GetRelativeSashPosition() * 1000000000)
 	);
 
-	COptions::Get()->SetOption(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
+	COptions::Get()->set(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
 }
 
 bool CMainFrame::RestoreSplitterPositions()
@@ -2253,7 +2316,8 @@ bool CMainFrame::RestoreSplitterPositions()
 	}
 
 	// top_pos bottom_height view_pos view_height_width local_pos remote_pos
-	auto tokens = fz::strtok(COptions::Get()->GetOption(OPTION_MAINWINDOW_SPLITTER_POSITION), L" ");
+	std::wstring const positions = COptions::Get()->get_string(OPTION_MAINWINDOW_SPLITTER_POSITION);
+	auto tokens = fz::strtok_view(positions, L" ");
 	if (tokens.size() < 6) {
 		return false;
 	}
@@ -2266,25 +2330,26 @@ bool CMainFrame::RestoreSplitterPositions()
 		}
 	}
 
-	m_pTopSplitter->SetSashPosition(values[0]);
-
-	m_pBottomSplitter->SetSashPosition(values[1]);
-
-	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
-	if (!controls) {
-		return false;
+	if (m_pTopSplitter) {
+		m_pTopSplitter->SetSashPosition(values[0]);
+	}
+	if (m_pBottomSplitter) {
+		m_pBottomSplitter->SetSashPosition(values[1]);
 	}
 
-	double pos = static_cast<double>(values[2]) / 1000000000;
-	if (pos >= 0 && pos <= 1) {
-		controls->pViewSplitter->SetRelativeSashPosition(pos);
+	if (m_pContextControl) {
+		double pos = static_cast<double>(values[2]) / 1000000000;
+		for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+			auto controls = m_pContextControl->GetControlsFromTabIndex(i);
+			if (!controls) {
+				continue;
+			}
+			controls->SetSplitterPositions(std::make_tuple(pos, values[3], values[4]));
+		}
 	}
 
-	controls->pLocalSplitter->SetSashPosition(values[3]);
-	controls->pRemoteSplitter->SetSashPosition(values[4]);
-
-	pos = static_cast<double>(values[5]) / 1000000000;
-	if (pos >= 0 && pos <= 1) {
+	double pos = static_cast<double>(values[5]) / 1000000000;
+	if (pos >= 0 && pos <= 1 && m_pQueueLogSplitter) {
 		m_pQueueLogSplitter->SetRelativeSashPosition(pos);
 	}
 
@@ -2293,22 +2358,39 @@ bool CMainFrame::RestoreSplitterPositions()
 
 void CMainFrame::SetDefaultSplitterPositions()
 {
-	m_pTopSplitter->SetSashPosition(97);
+	if (m_pTopSplitter) {
+		m_pTopSplitter->SetSashPosition(97);
+	}
 
 	wxSize size = m_pBottomSplitter->GetClientSize();
 	int h = size.GetHeight() - 135;
 	if (h < 50) {
 		h = 50;
 	}
-	m_pBottomSplitter->SetSashPosition(h);
+	if (m_pBottomSplitter) {
+		m_pBottomSplitter->SetSashPosition(h);
+	}
 
-	m_pQueueLogSplitter->SetSashPosition(0);
+	if (m_pQueueLogSplitter) {
+		m_pQueueLogSplitter->SetSashPosition(0);
+	}
 
-	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
-	if (controls) {
-		controls->pViewSplitter->SetSashPosition(0);
-		controls->pLocalSplitter->SetRelativeSashPosition(0.4);
-		controls->pRemoteSplitter->SetRelativeSashPosition(0.4);
+	if (m_pContextControl) {
+		for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+			CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromTabIndex(i);
+			if (!controls) {
+				continue;
+			}
+			if (controls->pViewSplitter) {
+				controls->pViewSplitter->SetRelativeSashPosition(0.5);
+			}
+			if (controls->pLocalSplitter) {
+				controls->pLocalSplitter->SetRelativeSashPosition(0.4);
+			}
+			if (controls->pRemoteSplitter) {
+				controls->pRemoteSplitter->SetRelativeSashPosition(0.4);
+			}
+		}
 	}
 }
 
@@ -2354,7 +2436,7 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 		return;
 	}
 
-	if (!COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT)) {
+	if (!COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT)) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (!controls) {
 			return;
@@ -2428,7 +2510,7 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 	menu->FindItem(XRCID("ID_TOOLBAR_COMPARISON"))->Check(pComparisonManager->IsComparing());
 
-	const int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
+	const int mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
 	if (mode == 0) {
 		menu->FindItem(XRCID("ID_COMPARE_SIZE"))->Check();
 	}
@@ -2436,7 +2518,7 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 		menu->FindItem(XRCID("ID_COMPARE_DATE"))->Check();
 	}
 
-	menu->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
+	menu->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0);
 
 	ShowDropdownMenu(menu, m_pToolBar, event);
 }
@@ -2471,13 +2553,16 @@ void CMainFrame::OnDropdownComparisonMode(wxCommandEvent& event)
 		return;
 	}
 
-	int old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
+	int old_mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
 	int new_mode = (event.GetId() == XRCID("ID_COMPARE_SIZE")) ? 0 : 1;
-	COptions::Get()->SetOption(OPTION_COMPARISONMODE, new_mode);
+	COptions::Get()->set(OPTION_COMPARISONMODE, new_mode);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-	if (old_mode != new_mode && pComparisonManager && pComparisonManager->IsComparing()) {
-		pComparisonManager->CompareListings();
+	if (old_mode != new_mode && pComparisonManager) {
+		pComparisonManager->SetComparisonMode(new_mode);
+		if (pComparisonManager->IsComparing()) {
+			pComparisonManager->CompareListings();
+		}
 	}
 }
 
@@ -2488,12 +2573,15 @@ void CMainFrame::OnDropdownComparisonHide(wxCommandEvent&)
 		return;
 	}
 
-	bool old_mode = COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0;
-	COptions::Get()->SetOption(OPTION_COMPARE_HIDEIDENTICAL, old_mode ? 0 : 1);
+	bool old_mode = COptions::Get()->get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0;
+	COptions::Get()->set(OPTION_COMPARE_HIDEIDENTICAL, old_mode ? 0 : 1);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
-	if (pComparisonManager && pComparisonManager->IsComparing()) {
-		pComparisonManager->CompareListings();
+	if (pComparisonManager) {
+		pComparisonManager->SetHideIdentical(old_mode ? 0 : 1);
+		if (pComparisonManager->IsComparing()) {
+			pComparisonManager->CompareListings();
+		}
 	}
 }
 
@@ -2522,7 +2610,7 @@ void CMainFrame::ProcessCommandLine()
 
 	std::wstring site;
 	if (pCommandLine->HasSwitch(CCommandLine::sitemanager)) {
-		if (COptions::Get()->GetOptionVal(OPTION_STARTUP_ACTION) != 1) {
+		if (COptions::Get()->get_int(OPTION_STARTUP_ACTION) != 1) {
 			OpenSiteManager();
 		}
 	}
@@ -2555,7 +2643,7 @@ void CMainFrame::ProcessCommandLine()
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
 			site.SetLogonType(LogonType::ask);
 			CLoginManager::Get().RememberPassword(site);
 		}
@@ -2657,7 +2745,7 @@ void CMainFrame::OnIconize(wxIconizeEvent& event)
 		return;
 	}
 
-	if (!COptions::Get()->GetOptionVal(OPTION_MINIMIZE_TRAY)) {
+	if (!COptions::Get()->get_int(OPTION_MINIMIZE_TRAY)) {
 		return;
 	}
 
@@ -2744,10 +2832,11 @@ void CMainFrame::PostInitialize()
 		}
 		);
 		m_pUpdater->Init();
+		ShowOverlay(m_pUpdater->GetResources(resource_type::overlay), this, m_pTopSplitter, wxPoint(-40, 30));
 	}
 #endif
 
-	int const startupAction = COptions::Get()->GetOptionVal(OPTION_STARTUP_ACTION);
+	int const startupAction = COptions::Get()->get_int(OPTION_STARTUP_ACTION);
 	bool startupReconnect = startupAction == 2;
 
 	if (startupAction == 1) {
@@ -2761,8 +2850,8 @@ void CMainFrame::PostInitialize()
 	}
 
 	if (m_pContextControl && startupReconnect) {
-		auto xml = COptions::Get()->GetOptionXml(OPTION_TAB_DATA);
-		pugi::xml_node tabs = xml ? xml->child("Tabs") : pugi::xml_node();
+		auto xml = COptions::Get()->get_xml(OPTION_TAB_DATA);
+		pugi::xml_node tabs = xml.child("Tabs");
 		int i = 0;
 		for (auto tab = tabs.child("Tab"); tab; tab = tab.next_sibling("Tab")) {
 			if (tab.attribute("connected").as_int()) {
@@ -2807,7 +2896,7 @@ void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 
 void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 {
-	COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
+	COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
 #ifdef __WXMAC__
 	if (m_pToolBar) {
 		m_pToolBar->Show(event.IsChecked());
@@ -2870,7 +2959,7 @@ void CMainFrame::SetupKeyboardAccelerators()
 	SetAcceleratorTable(accel);
 }
 
-void CMainFrame::OnOptionsChanged(changed_options_t const& options)
+void CMainFrame::OnOptionsChanged(watched_options const& options)
 {
 	if (options.test(OPTION_FILEPANE_LAYOUT) || options.test(OPTION_FILEPANE_SWAP) || options.test(OPTION_MESSAGELOG_POSITION)) {
 		UpdateLayout();
