@@ -2,8 +2,8 @@
 #define FILEZILLA_INTERFACE_MAINFRM_HEADER
 
 #include "statusbar.h"
-#include "engine_context.h"
-#include "notification.h"
+#include "../include/engine_context.h"
+#include "../include/notification.h"
 #include "serverdata.h"
 
 #include <wx/timer.h>
@@ -16,6 +16,8 @@
 #include "updater.h"
 #endif
 
+#include <list>
+
 class CAsyncRequestQueue;
 class CContextControl;
 class CLed;
@@ -31,7 +33,7 @@ class CState;
 class CToolBar;
 class CWindowStateManager;
 
-class CMainFrame final : public wxNavigationEnabled<wxFrame>, private COptionChangeEventHandler, public EngineNotificationHandler
+class CMainFrame final : public wxNavigationEnabled<wxFrame>, public COptionChangeEventHandler
 #if FZ_MANUALUPDATECHECK
 	, protected CUpdateHandler
 #endif
@@ -67,13 +69,15 @@ public:
 	bool ConnectToSite(Site & data, Bookmark const& bookmark, CState* pState = 0);
 
 	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
+	void OnEngineEvent(CFileZillaEngine* engine);
+
 private:
 	void UpdateLayout();
 	void FixTabOrder();
 
 	bool CloseDialogsAndQuit(wxCloseEvent &event);
 	bool CreateMenus();
-	bool CreateQuickconnectBar();
+	void CreateQuickconnectBar();
 	bool CreateMainToolBar();
 	void OpenSiteManager(Site const* site = 0);
 
@@ -113,10 +117,7 @@ private:
 
 	void SetupKeyboardAccelerators();
 
-	void OnOptionsChanged(changed_options_t const& options);
-
-	virtual void OnEngineEvent(CFileZillaEngine* engine); // Before thread jump
-	void DoOnEngineEvent(CFileZillaEngine* engine); // After thread jump
+	void OnOptionsChanged(watched_options const& options);
 
 	// Event handlers
 	DECLARE_EVENT_TABLE()
