@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "queue.h"
 #include "Mainfrm.h"
 #include "Options.h"
@@ -6,32 +6,38 @@
 #include "statuslinectrl.h"
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
+#include "file_utils.h"
 #include "ipcmutex.h"
 #include "local_recursive_operation.h"
 #include "state.h"
 #include "asyncrequestqueue.h"
 #include "defaultfileexistsdlg.h"
-#include <wx/dnd.h>
 #include "dndobjects.h"
 #include "loginmanager.h"
 #include "aui_notebook_ex.h"
 #include "queueview_failed.h"
 #include "queueview_successful.h"
 #include "commandqueue.h"
-#include <wx/utils.h>
-#include <wx/progdlg.h>
-#include <wx/sound.h>
 #include "statusbar.h"
 #include "remote_recursive_operation.h"
 #include "auto_ascii_files.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
+
+#include <libfilezilla/glue/wxinvoker.hpp>
+
 #if WITH_LIBDBUS
 #include "../dbus/desktop_notification.h"
 #elif defined(__WXGTK__) || defined(__WXMSW__)
 #include <wx/notifmsg.h>
 #endif
 
+#include <wx/dnd.h>
+#include <wx/menu.h>
+#include <wx/progdlg.h>
+#include <wx/sound.h>
+#include <wx/utils.h>
+
 #ifdef __WXMSW__
 #include <powrprof.h>
 #endif
@@ -204,9 +210,10 @@ EVT_LIST_COL_CLICK(wxID_ANY, CQueueView::OnColumnClicked)
 END_EVENT_TABLE()
 
 CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
-	: CQueueViewBase(parent, index, _("Queued files")),
-	m_pMainFrame(pMainFrame),
-	m_pAsyncRequestQueue(pAsyncRequestQueue)
+	: CQueueViewBase(parent, index, _("Queued files"))
+	, COptionChangeEventHandler(this)
+	, m_pMainFrame(pMainFrame)
+	, m_pAsyncRequestQueue(pAsyncRequestQueue)
 {
 	wxGetApp().AddStartupProfileRecord("CQueueView::CQueueView");
 
@@ -214,7 +221,7 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 		m_pAsyncRequestQueue->SetQueue(this);
 	}
 
-	int action = COptions::Get()->GetOptionVal(OPTION_QUEUE_COMPLETION_ACTION);
+	int action = COptions::Get()->get_int(OPTION_QUEUE_COMPLETION_ACTION);
 	if (action < 0 || action >= ActionAfterState::Count) {
 		action = 1;
 	}
@@ -226,9 +233,9 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	std::vector<ColumnId> extraCols({colTransferStatus});
 	CreateColumns(extraCols);
 
-	RegisterOption(OPTION_NUMTRANSFERS);
-	RegisterOption(OPTION_CONCURRENTDOWNLOADLIMIT);
-	RegisterOption(OPTION_CONCURRENTUPLOADLIMIT);
+	COptions::Get()->watch(OPTION_NUMTRANSFERS, this);
+	COptions::Get()->watch(OPTION_CONCURRENTDOWNLOADLIMIT, this);
+	COptions::Get()->watch(OPTION_CONCURRENTUPLOADLIMIT, this);
 
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_REWRITE_CREDENTIALS, false);
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_QUITNOW, false);
@@ -245,14 +252,15 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 
 CQueueView::~CQueueView()
 {
+	COptions::Get()->unwatch_all(this);
 	DeleteEngines();
 
 	m_resize_timer.Stop();
 }
 
-bool CQueueView::QueueFile(const bool queueOnly, const bool download,
+bool CQueueView::QueueFile(bool const queueOnly, bool const download,
 						   std::wstring const& sourceFile, std::wstring const& targetFile,
-						   const CLocalPath& localPath, const CServerPath& remotePath,
+						   CLocalPath const& localPath, CServerPath const& remotePath,
 						   Site const& site, int64_t size, CEditHandler::fileType edit,
 						   QueuePriority priority)
 {
@@ -271,15 +279,26 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download,
 		wxASSERT(edit == CEditHandler::none);
 	}
 	else {
-		fileItem = new CFileItem(pServerItem, queueOnly, download, sourceFile, targetFile, localPath, remotePath, size);
+		transfer_flags flags{};
+		if (download) {
+			flags |= transfer_flags::download;
+		}
+		if (queueOnly) {
+			flags |= queue_flags::queued;
+		}
 		if (site.server.HasFeature(ProtocolFeature::DataTypeConcept)) {
 			if (download) {
-				fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(sourceFile, remotePath.GetType()));
+				if (CAutoAsciiFiles::TransferRemoteAsAscii(sourceFile, remotePath.GetType())) {
+					flags |= ftp_transfer_flags::ascii;
+				}
 			}
 			else {
-				fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(sourceFile, remotePath.GetType()));
+				if (CAutoAsciiFiles::TransferLocalAsAscii(sourceFile, remotePath.GetType())) {
+					flags |= ftp_transfer_flags::ascii;
+				}
 			}
 		}
+		fileItem = new CFileItem(pServerItem, flags, sourceFile, targetFile, localPath, remotePath, size);
 		fileItem->m_edit = edit;
 		if (edit != CEditHandler::none) {
 			fileItem->m_onetime_action = CFileExistsNotification::overwrite;
@@ -335,16 +354,20 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 		}
 
 		std::wstring localFile = ReplaceInvalidCharacters(fileInfo.name);
-		if (dataObject.GetServerPath().GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
+		if (dataObject.GetServerPath().GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
 			localFile = StripVMSRevision(localFile);
 		}
 
-		CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, true,
+		transfer_flags flags = transfer_flags::download;
+		if (queueOnly) {
+			flags |= queue_flags::queued;
+		}
+		if (hasDataTypeConcept && CAutoAsciiFiles::TransferRemoteAsAscii(fileInfo.name, dataObject.GetServerPath().GetType())) {
+			flags |= ftp_transfer_flags::ascii;
+		}
+		CFileItem* fileItem = new CFileItem(pServerItem, flags,
 			fileInfo.name, (fileInfo.name != localFile) ? localFile : std::wstring(),
 			localPath, dataObject.GetServerPath(), fileInfo.size);
-		if (hasDataTypeConcept) {
-			fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(fileInfo.name, dataObject.GetServerPath().GetType()));
-		}
 
 		InsertItem(pServerItem, fileItem);
 	}
@@ -368,12 +391,16 @@ bool CQueueView::QueueFiles(const bool queueOnly, Site const& site, CLocalRecurs
 		bool const hasDataTypeConcept = site.server.HasFeature(ProtocolFeature::DataTypeConcept);
 
 		for (auto const& file : files) {
-			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, false,
+			transfer_flags flags{};
+			if (queueOnly) {
+				flags |= queue_flags::queued;
+			}
+			if (hasDataTypeConcept && CAutoAsciiFiles::TransferLocalAsAscii(file.name, listing.remotePath.GetType())) {
+				flags |= ftp_transfer_flags::ascii;
+			}
+			CFileItem* fileItem = new CFileItem(pServerItem, flags,
 				file.name, std::wstring(),
 				listing.localPath, listing.remotePath, file.size);
-			if (hasDataTypeConcept) {
-				fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(file.name, listing.remotePath.GetType()));
-			}
 
 			InsertItem(pServerItem, fileItem);
 		}
@@ -385,11 +412,6 @@ bool CQueueView::QueueFiles(const bool queueOnly, Site const& site, CLocalRecurs
 }
 
 void CQueueView::OnEngineEvent(CFileZillaEngine* engine)
-{
-	CallAfter(&CQueueView::DoOnEngineEvent, engine);
-}
-
-void CQueueView::DoOnEngineEvent(CFileZillaEngine* engine)
 {
 	t_EngineData* const pEngineData = GetEngineData(engine);
 	if (!pEngineData) {
@@ -421,8 +443,8 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<
 	switch (pNotification->GetID())
 	{
 	case nId_logmsg:
-		m_pMainFrame->GetStatusView()->AddToLog(static_cast<CLogmsgNotification&>(*pNotification.get()));
-		if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2) {
+		m_pMainFrame->GetStatusView()->AddToLog(std::move(static_cast<CLogmsgNotification&>(*pNotification.get())));
+		if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 2) {
 			m_pQueue->Highlight(3);
 		}
 		break;
@@ -446,7 +468,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<
 								{
 								case CFileExistsNotification::resume:
 									if (fileExistsNotification.canResume &&
-										!pFileItem->Ascii())
+										!fileExistsNotification.ascii)
 									{
 										fileExistsNotification.overwriteAction = CFileExistsNotification::resume;
 									}
@@ -567,7 +589,7 @@ bool CQueueView::CanStartTransfer(CServerItem const & server_item, t_EngineData
 	// At this point the following holds:
 	// max_count is limited to 1, only connection to server is held by the browsing connection
 
-	pEngineData = GetEngineData(browsingStateOnSameServer->m_pEngine);
+	pEngineData = GetEngineData(browsingStateOnSameServer->engine_.get());
 	if (pEngineData) {
 		wxASSERT(pEngineData->transient);
 		return pEngineData->transient && !pEngineData->active;
@@ -576,7 +598,7 @@ bool CQueueView::CanStartTransfer(CServerItem const & server_item, t_EngineData
 	pEngineData = new t_EngineData;
 	pEngineData->transient = true;
 	pEngineData->state = t_EngineData::waitprimary;
-	pEngineData->pEngine = browsingStateOnSameServer->m_pEngine;
+	pEngineData->pEngine = browsingStateOnSameServer->engine_.get();
 	m_engineData.push_back(pEngineData);
 	return true;
 }
@@ -588,13 +610,13 @@ bool CQueueView::TryStartNextTransfer()
 	}
 
 	// Check transfer limit
-	if (m_activeCount >= COptions::Get()->GetOptionVal(OPTION_NUMTRANSFERS)) {
+	if (m_activeCount >= COptions::Get()->get_int(OPTION_NUMTRANSFERS)) {
 		return false;
 	}
 
 	// Check limits for concurrent up/downloads
-	const int maxDownloads = COptions::Get()->GetOptionVal(OPTION_CONCURRENTDOWNLOADLIMIT);
-	const int maxUploads = COptions::Get()->GetOptionVal(OPTION_CONCURRENTUPLOADLIMIT);
+	const int maxDownloads = COptions::Get()->get_int(OPTION_CONCURRENTDOWNLOADLIMIT);
+	const int maxUploads = COptions::Get()->get_int(OPTION_CONCURRENTUPLOADLIMIT);
 	TransferDirection wantedDirection;
 	if (maxDownloads && m_activeCountDown >= maxDownloads) {
 		if (maxUploads && m_activeCountUp >= maxUploads) {
@@ -758,19 +780,13 @@ bool CQueueView::TryStartNextTransfer()
 
 void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification const& notification)
 {
-	if (notification.nReplyCode & FZ_REPLY_DISCONNECTED &&
-		notification.commandId == ::Command::none)
-	{
-		// Queue is not interested in disconnect notifications
-		return;
-	}
-	wxASSERT(notification.commandId != ::Command::none);
+	wxASSERT(notification.commandId_ != ::Command::none);
 
 	// Cancel pending requests
 	m_pAsyncRequestQueue->ClearPending(pEngineData->pEngine);
 
 	// Process reply from the engine
-	int replyCode = notification.nReplyCode;
+	int replyCode = notification.replyCode_;
 
 	if ((replyCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 		ResetReason reason;
@@ -856,10 +872,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 					return;
 				}
 			}
-
-			if (!pEngineData->transient) {
-				SwitchEngine(&pEngineData);
-			}
 		}
 		break;
 	case t_EngineData::transfer:
@@ -914,11 +926,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 				return;
 			}
 		}
-		if (replyCode & FZ_REPLY_DISCONNECTED) {
-			if (!SwitchEngine(&pEngineData)) {
-				pEngineData->state = t_EngineData::connect;
-			}
-		}
 		break;
 	case t_EngineData::mkdir:
 		if (replyCode == FZ_REPLY_OK) {
@@ -929,15 +936,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 			if (!IncreaseErrorCount(*pEngineData)) {
 				return;
 			}
-
-			if (pEngineData->transient) {
-				ResetEngine(*pEngineData, ResetReason::retry);
-				return;
-			}
-
-			if (!SwitchEngine(&pEngineData)) {
-				pEngineData->state = t_EngineData::connect;
-			}
 		}
 		else {
 			// Cannot retry
@@ -1113,7 +1111,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const ResetReason reason)
 		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
 			CState* pState = *iter;
-			if (pState->m_pEngine != data.pEngine) {
+			if (pState->engine_.get() != data.pEngine) {
 				continue;
 			}
 			CCommandQueue* pCommandQueue = pState->m_pCommandQueue;
@@ -1177,7 +1175,7 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 			CState* pState = 0;
 			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
-				if ((*iter)->m_pEngine != engineData.pEngine) {
+				if ((*iter)->engine_.get() != engineData.pEngine) {
 					continue;
 				}
 				pState = *iter;
@@ -1263,25 +1261,23 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 			fileItem->SetStatusMessage(CFileItem::Status::transferring);
 			RefreshItem(engineData.pItem);
 
-			CFileTransferCommand::t_transferSettings transferSettings;
-			transferSettings.binary = !fileItem->Ascii();
-			int res = engineData.pEngine->Execute(CFileTransferCommand(fileItem->GetLocalPath().GetPath() + fileItem->GetLocalFile(), fileItem->GetRemotePath(),
-												fileItem->GetRemoteFile(), fileItem->Download(), transferSettings));
+			int res;
+			if (!fileItem->Download()) {
+				auto cmd = CFileTransferCommand(file_reader_factory(fileItem->GetLocalPath().GetPath() + fileItem->GetLocalFile()),
+					fileItem->GetRemotePath(), fileItem->GetRemoteFile(), fileItem->flags());
+				res = engineData.pEngine->Execute(cmd);
+			}
+			else {
+				auto cmd = CFileTransferCommand(file_writer_factory(fileItem->GetLocalPath().GetPath() + fileItem->GetLocalFile()),
+					fileItem->GetRemotePath(), fileItem->GetRemoteFile(), fileItem->flags());
+				res = engineData.pEngine->Execute(cmd);
+			}
+
 			wxASSERT((res & FZ_REPLY_BUSY) != FZ_REPLY_BUSY);
 			if (res == FZ_REPLY_WOULDBLOCK) {
 				return;
 			}
 
-			if (res == FZ_REPLY_NOTCONNECTED) {
-				if (engineData.transient) {
-					ResetEngine(engineData, ResetReason::retry);
-					return;
-				}
-
-				engineData.state = t_EngineData::connect;
-				continue;
-			}
-
 			if (res == FZ_REPLY_OK) {
 				ResetEngine(engineData, ResetReason::success);
 				return;
@@ -1306,23 +1302,20 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 				return;
 			}
 
-			if (res == FZ_REPLY_NOTCONNECTED) {
-				if (engineData.transient) {
-					ResetEngine(engineData, ResetReason::retry);
-					return;
-				}
-
-				engineData.state = t_EngineData::connect;
-				continue;
-			}
-
 			if (res == FZ_REPLY_OK) {
 				ResetEngine(engineData, ResetReason::success);
 				return;
 			}
 
-			// Pointless to retry
-			ResetEngine(engineData, ResetReason::failure);
+			if (res & FZ_REPLY_DISCONNECTED) {
+				if (IncreaseErrorCount(engineData)) {
+					continue;
+				}
+			}
+			else {
+				// Pointless to retry
+				ResetEngine(engineData, ResetReason::failure);
+			}
 			return;
 		}
 	}
@@ -1430,7 +1423,7 @@ bool CQueueView::Quit()
 		m_quit = 2;
 	}
 
-	SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
+	SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, OPTIONS_NUM, OPTIONS_NUM);
 
 	m_resize_timer.Stop();
 
@@ -1485,7 +1478,7 @@ void CQueueView::CheckQueueState()
 bool CQueueView::IncreaseErrorCount(t_EngineData& engineData)
 {
 	++engineData.pItem->m_errorCount;
-	if (engineData.pItem->m_errorCount <= COptions::Get()->GetOptionVal(OPTION_RECONNECTCOUNT)) {
+	if (engineData.pItem->m_errorCount <= COptions::Get()->get_int(OPTION_RECONNECTCOUNT)) {
 		return true;
 	}
 
@@ -1560,7 +1553,7 @@ void CQueueView::DisplayQueueSize()
 void CQueueView::SaveQueue(bool silent)
 {
 	// Kiosk mode 2 doesn't save queue
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+	if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
 	}
 
@@ -1595,7 +1588,7 @@ void CQueueView::LoadQueueFromXML()
 
 	document.remove_child(queue);
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+	if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
 	}
 
@@ -1650,7 +1643,7 @@ void CQueueView::LoadQueue()
 		}
 
 		if (error || first_id > 0) {
-			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 2) {
+			if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 2) {
 				if (!m_queue_storage.Clear()) {
 					error = true;
 				}
@@ -1699,16 +1692,20 @@ void CQueueView::ImportQueue(pugi::xml_node element, bool updateSelections)
 				std::wstring localFile = GetTextElement(file, "LocalFile");
 				std::wstring remoteFile = GetTextElement(file, "RemoteFile");
 				std::wstring safeRemotePath = GetTextElement(file, "RemotePath");
-				bool download = GetTextElementInt(file, "Download") != 0;
+				
+				transfer_flags flags = queue_flags::queued | static_cast<transfer_flags>(GetTextElementInt(file, "Flags"));
+				bool const old_download = GetTextElementInt(file, "Download") != 0;
+				if (old_download) {
+					flags |= transfer_flags::download;
+				}
 				int64_t size = GetTextElementInt(file, "Size", -1);
 				unsigned char errorCount = static_cast<unsigned char>(GetTextElementInt(file, "ErrorCount"));
 				unsigned int priority = GetTextElementInt(file, "Priority", static_cast<unsigned int>(QueuePriority::normal));
 
-				int dataType = GetTextElementInt(file, "DataType", -1);
-				if (dataType == -1) {
-					dataType = GetTextElementInt(file, "TransferMode", 1);
+				int old_dataType = GetTextElementInt(file, "DataType", -1);
+				if (!old_dataType && site.server.HasFeature(ProtocolFeature::DataTypeConcept)) {
+					flags |= ftp_transfer_flags::ascii;
 				}
-				bool binary = dataType != 0;
 				int overwrite_action = GetTextElementInt(file, "OverwriteAction", CFileExistsNotification::unknown);
 
 				CServerPath remotePath;
@@ -1731,11 +1728,10 @@ void CQueueView::ImportQueue(pugi::xml_node element, bool updateSelections)
 						previousRemotePath = remotePath;
 					}
 
-					CFileItem* fileItem = new CFileItem(pServerItem, true, download,
-						download ? remoteFile : localFileName,
-						(remoteFile != localFileName) ? (download ? localFileName : remoteFile) : std::wstring(),
+					CFileItem* fileItem = new CFileItem(pServerItem, flags,
+						(flags & transfer_flags::download) ? remoteFile : localFileName,
+						(remoteFile != localFileName) ? ((flags & transfer_flags::download) ? localFileName : remoteFile) : std::wstring(),
 						previousLocalPath, previousRemotePath, size);
-					fileItem->SetAscii(!binary);
 					fileItem->SetPriorityRaw(QueuePriority(priority));
 					fileItem->m_errorCount = errorCount;
 					InsertItem(pServerItem, fileItem);
@@ -1748,8 +1744,12 @@ void CQueueView::ImportQueue(pugi::xml_node element, bool updateSelections)
 			for (auto folder = xServer.child("Folder"); folder; folder = folder.next_sibling("Folder")) {
 				CFolderItem* folderItem;
 
-				bool download = GetTextElementInt(folder, "Download") != 0;
-				if (download) {
+				transfer_flags flags = queue_flags::queued | static_cast<transfer_flags>(GetTextElementInt(folder, "Flags"));
+				bool const old_download = GetTextElementInt(folder, "Download") != 0;
+				if (old_download) {
+					flags |= transfer_flags::download;
+				}
+				if (flags & transfer_flags::download) {
 					std::wstring localFile = GetTextElement(folder, "LocalFile");
 					CLocalPath localPath(localFile);
 					if (localPath.empty()) {
@@ -1920,7 +1920,7 @@ void CQueueView::OnActionAfter(wxCommandEvent& event)
 	}
 	else if (event.GetId() == XRCID("ID_ACTIONAFTER_RUNCOMMAND")) {
 		wxTextEntryDialog dlg(m_pMainFrame, _("Please enter the complete path of a program and its arguments. This command will be executed when the queue has finished processing.\nE.g. c:\\somePath\\file.exe under MS Windows or /somePath/file under Unix.\nYou need to properly quote commands and their arguments if they contain spaces."), _("Enter command"));
-		dlg.SetValue(COptions::Get()->GetOption(OPTION_QUEUE_COMPLETION_COMMAND));
+		dlg.SetValue(COptions::Get()->get_string(OPTION_QUEUE_COMPLETION_COMMAND));
 
 		if (dlg.ShowModal() == wxID_OK) {
 			const wxString &command = dlg.GetValue();
@@ -1929,7 +1929,7 @@ void CQueueView::OnActionAfter(wxCommandEvent& event)
 			}
 			else {
 				m_actionAfterState = ActionAfterState::RunCommand;
-				COptions::Get()->SetOption(OPTION_QUEUE_COMPLETION_COMMAND, command.ToStdWstring());
+				COptions::Get()->set(OPTION_QUEUE_COMPLETION_COMMAND, command.ToStdWstring());
 			}
 		}
 	}
@@ -1943,7 +1943,7 @@ void CQueueView::OnActionAfter(wxCommandEvent& event)
 #endif
 
 	if (m_actionAfterState != ActionAfterState::Reboot && m_actionAfterState != ActionAfterState::Shutdown && m_actionAfterState != ActionAfterState::Sleep && m_actionAfterState != ActionAfterState::CloseOnce) {
-		COptions::Get()->SetOption(OPTION_QUEUE_COMPLETION_ACTION, m_actionAfterState);
+		COptions::Get()->set(OPTION_QUEUE_COMPLETION_ACTION, m_actionAfterState);
 	}
 }
 
@@ -2151,12 +2151,11 @@ void CQueueView::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAc
 
 void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 {
-	if (!HasSelection())
+	if (!HasSelection()) {
 		return;
+	}
 
 	CDefaultFileExistsDlg dlg;
-	if (!dlg.Load(this, true))
-		return;
 
 	// Get current default action for the item
 	CFileExistsNotification::OverwriteAction downloadAction = CFileExistsNotification::unknown;
@@ -2167,35 +2166,38 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 	bool upload_unknown = false;
 
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		CQueueItem* pItem = GetQueueItem(item);
-		if (!pItem)
+		if (!pItem) {
 			continue;
+		}
 
 		switch (pItem->GetType())
 		{
 		case QueueItemType::File:
 			{
 				CFileItem *pFileItem = (CFileItem*)pItem;
-				if (pFileItem->Download())
-				{
-					if (downloadAction == CFileExistsNotification::unknown)
+				if (pFileItem->Download()) {
+					if (downloadAction == CFileExistsNotification::unknown) {
 						downloadAction = pFileItem->m_defaultFileExistsAction;
-					else if (pFileItem->m_defaultFileExistsAction != downloadAction)
+					}
+					else if (pFileItem->m_defaultFileExistsAction != downloadAction) {
 						download_unknown = true;
+					}
 					has_download = true;
 				}
-				else
-				{
-					if (uploadAction == CFileExistsNotification::unknown)
+				else {
+					if (uploadAction == CFileExistsNotification::unknown) {
 						uploadAction = pFileItem->m_defaultFileExistsAction;
-					else if (pFileItem->m_defaultFileExistsAction != uploadAction)
+					}
+					else if (pFileItem->m_defaultFileExistsAction != uploadAction) {
 						upload_unknown = true;
+					}
 					has_upload = true;
 				}
 			}
@@ -2212,40 +2214,48 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 			break;
 		}
 	}
-	if (download_unknown)
+	if (download_unknown) {
 		downloadAction = CFileExistsNotification::unknown;
-	if (upload_unknown)
+	}
+	if (upload_unknown) {
 		uploadAction = CFileExistsNotification::unknown;
+	}
 
-	if (!dlg.Run(has_download ? &downloadAction : 0, has_upload ? &uploadAction : 0))
+	if (!has_upload && !has_download) {
 		return;
+	}
+
+	if (!dlg.Run(this, true, has_download ? &downloadAction : 0, has_upload ? &uploadAction : 0)) {
+		return;
+	}
 
 	item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		CQueueItem* pItem = GetQueueItem(item);
-		if (!pItem)
+		if (!pItem) {
 			continue;
+		}
 
 		switch (pItem->GetType())
 		{
 		case QueueItemType::File:
 			{
 				CFileItem *pFileItem = (CFileItem*)pItem;
-				if (pFileItem->Download())
-				{
-					if (!has_download)
+				if (pFileItem->Download()) {
+					if (!has_download) {
 						break;
+					}
 					pFileItem->m_defaultFileExistsAction = downloadAction;
 				}
-				else
-				{
-					if (!has_upload)
+				else {
+					if (!has_upload) {
 						break;
+					}
 					pFileItem->m_defaultFileExistsAction = uploadAction;
 				}
 			}
@@ -2253,10 +2263,12 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 		case QueueItemType::Server:
 			{
 				CServerItem *pServerItem = (CServerItem*)pItem;
-				if (has_download)
+				if (has_download) {
 					pServerItem->SetDefaultFileExistsAction(downloadAction, TransferDirection::download);
-				if (has_upload)
+				}
+				if (has_upload) {
 					pServerItem->SetDefaultFileExistsAction(uploadAction, TransferDirection::upload);
+				}
 			}
 			break;
 		default:
@@ -2299,10 +2311,10 @@ t_EngineData* CQueueView::GetIdleEngine(Site const& site, bool allowTransient)
 
 	if (!pFirstIdle) {
 		// Check whether we can create another engine
-		const int newEngineCount = COptions::Get()->GetOptionVal(OPTION_NUMTRANSFERS);
+		const int newEngineCount = COptions::Get()->get_int(OPTION_NUMTRANSFERS);
 		if (newEngineCount > static_cast<int>(m_engineData.size()) - transient) {
 			pFirstIdle = new t_EngineData;
-			pFirstIdle->pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext(), *this);
+			pFirstIdle->pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext(), fz::make_invoker(*this, [this](CFileZillaEngine* engine) { OnEngineEvent(engine); }));
 
 			m_engineData.push_back(pFirstIdle);
 		}
@@ -2741,7 +2753,7 @@ void CQueueView::ActionAfter(bool warned)
 		}
 		case ActionAfterState::RunCommand:
 		{
-			wxString cmd = COptions::Get()->GetOption(OPTION_QUEUE_COMPLETION_COMMAND);
+			wxString cmd = COptions::Get()->get_string(OPTION_QUEUE_COMPLETION_COMMAND);
 			if (!cmd.empty()) {
 				wxExecute(cmd);
 			}
@@ -2887,72 +2899,6 @@ void CQueueView::OnActionAfterTimerTick()
 }
 #endif
 
-bool CQueueView::SwitchEngine(t_EngineData** ppEngineData)
-{
-	if (m_engineData.size() < 2) {
-		return false;
-	}
-
-	t_EngineData* pEngineData = *ppEngineData;
-	for (auto & pNewEngineData : m_engineData) {
-		if (pNewEngineData == pEngineData) {
-			continue;
-		}
-
-		if (pNewEngineData->active || pNewEngineData->transient) {
-			continue;
-		}
-
-		if (pNewEngineData->lastSite != pEngineData->lastSite) {
-			continue;
-		}
-
-		if (!pNewEngineData->pEngine->IsConnected()) {
-			continue;
-		}
-
-		wxASSERT(!pNewEngineData->pItem);
-		pNewEngineData->pItem = pEngineData->pItem;
-		pNewEngineData->pItem->m_pEngineData = pNewEngineData;
-		pEngineData->pItem = 0;
-
-		pNewEngineData->active = true;
-		pEngineData->active = false;
-
-		delete pNewEngineData->m_idleDisconnectTimer;
-		pNewEngineData->m_idleDisconnectTimer = 0;
-
-		// Swap status line
-		CStatusLineCtrl* pOldStatusLineCtrl = pNewEngineData->pStatusLineCtrl;
-		pNewEngineData->pStatusLineCtrl = pEngineData->pStatusLineCtrl;
-		if (pNewEngineData->pStatusLineCtrl) {
-			pNewEngineData->pStatusLineCtrl->SetEngineData(pNewEngineData);
-		}
-		if (pOldStatusLineCtrl) {
-			pEngineData->pStatusLineCtrl = pOldStatusLineCtrl;
-			pEngineData->pStatusLineCtrl->SetEngineData(pEngineData);
-		}
-
-		// Set new state
-		if (pNewEngineData->pItem->GetType() == QueueItemType::File) {
-			pNewEngineData->state = t_EngineData::transfer;
-		}
-		else {
-			pNewEngineData->state = t_EngineData::mkdir;
-		}
-		if (pNewEngineData->pStatusLineCtrl) {
-			pNewEngineData->pStatusLineCtrl->ClearTransferStatus();
-		}
-
-		pEngineData->state = t_EngineData::none;
-
-		*ppEngineData = pNewEngineData;
-		return true;
-	}
-
-	return false;
-}
-
 bool CQueueView::IsOtherEngineConnected(t_EngineData* pEngineData)
 {
 	for (auto const* current : m_engineData) {
@@ -3020,7 +2966,7 @@ void CQueueView::OnSize(wxSizeEvent& event)
 	event.Skip();
 }
 
-void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString& newName, bool local)
+void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, std::wstring const& newName, bool local, writer_factory_holder & new_writer)
 {
 	t_EngineData* const pEngineData = GetEngineData(pEngine);
 	if (!pEngineData || !pEngineData->pItem) {
@@ -3036,62 +2982,30 @@ void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString&
 		wxFileName fn(pFile->GetLocalPath().GetPath(), pFile->GetLocalFile());
 		fn.SetFullName(newName);
 		pFile->SetTargetFile(fn.GetFullName().ToStdWstring());
+		new_writer = file_writer_factory(fn.GetFullPath().ToStdWstring());
 	}
 	else {
-		pFile->SetTargetFile(newName.ToStdWstring());
+		pFile->SetTargetFile(newName);
 	}
 
 	RefreshItem(pFile);
 }
 
-std::wstring CQueueView::ReplaceInvalidCharacters(std::wstring const& filename)
+std::wstring CQueueView::ReplaceInvalidCharacters(std::wstring const& filename, bool includeQuotesAndBreaks)
 {
-	if (!COptions::Get()->GetOptionVal(OPTION_INVALID_CHAR_REPLACE_ENABLE)) {
+	if (!COptions::Get()->get_int(OPTION_INVALID_CHAR_REPLACE_ENABLE)) {
 		return filename;
 	}
 
-	const wxChar replace = COptions::Get()->GetOption(OPTION_INVALID_CHAR_REPLACE)[0];
+	wchar_t const replace = COptions::Get()->get_string(OPTION_INVALID_CHAR_REPLACE)[0];
 
-	wxString result;
-	{
-		wxStringBuffer start(result, filename.size() + 1);
-		wxChar* buf = start;
-
-		const wxChar* p = filename.c_str();
-		while (*p) {
-			const wxChar c = *p;
-			switch (c)
-			{
-			case '/':
-	#ifdef __WXMSW__
-			case '\\':
-			case ':':
-			case '*':
-			case '?':
-			case '"':
-			case '<':
-			case '>':
-			case '|':
-	#endif
-				if (replace)
-					*buf++ = replace;
-				break;
-			default:
-	#ifdef __WXMSW__
-				if (c < 0x20)
-					*buf++ = replace;
-				else
-	#endif
-				{
-					*buf++ = c;
-				}
-			}
-			p++;
+	std::wstring ret = filename;
+	for (auto & c : ret) {
+		if (IsInvalidChar(c, includeQuotesAndBreaks)) {
+			c = replace;
 		}
-		*buf = 0;
 	}
-
-	return result.ToStdWstring();
+	return ret;
 }
 
 wxFileOffset CQueueView::GetCurrentDownloadSpeed()
@@ -3135,7 +3049,7 @@ void CQueueView::ReleaseExclusiveEngineLock(CFileZillaEngine* pEngine)
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
 		CState* pState = *iter;
-		if (pState->m_pEngine != pEngine) {
+		if (pState->engine_.get() != pEngine) {
 			continue;
 		}
 		CCommandQueue *pCommandQueue = pState->m_pCommandQueue;
@@ -3174,7 +3088,7 @@ WXLRESULT CQueueView::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPara
 }
 #endif
 
-void CQueueView::OnOptionsChanged(changed_options_t const&)
+void CQueueView::OnOptionsChanged(watched_options const&)
 {
 	if (m_activeMode) {
 		AdvanceQueue();
@@ -3208,7 +3122,7 @@ void CQueueView::OnStateChange(CState*, t_statechange_notifications notification
 			return;
 		}
 
-		bool const forget = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
+		bool const forget = COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 0;
 		for (auto it = m_serverList.begin(); it != m_serverList.end(); ) {
 			Site site = (*it)->GetSite();
 			if (!forget) {
@@ -3237,7 +3151,7 @@ void CQueueView::OnStateChange(CState*, t_statechange_notifications notification
 	else if (notification == STATECHANGE_QUITNOW) {
 		if (m_quit != 2) {
 			SaveQueue(false);
-			SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, -1, -1);
+			SaveColumnSettings(OPTION_QUEUE_COLUMN_WIDTHS, OPTIONS_NUM, OPTIONS_NUM);
 		}
 	}
 }
