@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
@@ -22,6 +22,7 @@
 #include "timeformatting.h"
 
 #include <wx/clipbrd.h>
+#include <wx/menu.h>
 
 #include <algorithm>
 
@@ -360,7 +361,7 @@ CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQue
 CRemoteListView::~CRemoteListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->SetOption(OPTION_REMOTEFILELIST_SORTORDER, str.ToStdWstring());
+	COptions::Get()->set(OPTION_REMOTEFILELIST_SORTORDER, str.ToStdWstring());
 }
 
 // See comment to OnGetItemText
@@ -875,7 +876,7 @@ std::wstring StripVMSRevision(std::wstring const& name)
 
 void CRemoteListView::OnItemActivated(wxListEvent &event)
 {
-	int const action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+	int const action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 	if (!m_state.IsRemoteIdle(action ? false : true)) {
 		wxBell();
 		return;
@@ -960,7 +961,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			}
 		}
 		else {
-			const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_FILE);
+			const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_FILE);
 			if (action == 3) {
 				// No action
 				wxBell();
@@ -983,7 +984,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			}
 
 			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
+			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
 			}
 			m_pQueue->QueueFile(queue_only, true, name,
@@ -1285,7 +1286,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		}
 		else {
 			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
+			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
 			}
 			m_pQueue->QueueFile(queue_only, true,
@@ -1301,7 +1302,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), m_pDirectoryListing->path, !queue_only);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 	}
 }
 
@@ -1532,9 +1533,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 
 		if (!root.empty()) {
 			pRecursiveOperation->AddRecursionRoot(std::move(root));
-
-			pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete,
-														 filter.GetActiveFilters(), m_pDirectoryListing->path);
+			pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters());
 		}
 	}
 }
@@ -1845,8 +1844,7 @@ void CRemoteListView::HandleGenericChmod(ChmodUICommand &command)
 		pRecursiveOperation->SetChmodData(std::move(chmodData));
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod,
-													 filter.GetActiveFilters(), m_pDirectoryListing->path);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters());
 
 		// Refresh listing. This gets done implicitely by the recursive operation, so
 		// only it if not recursing.
@@ -2165,7 +2163,7 @@ void CRemoteListView::SetInfoText()
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
 {
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -2431,7 +2429,7 @@ void CRemoteListView::StartComparison()
 	}
 }
 
-bool CRemoteListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
+bool CRemoteListView::get_next_file(std::wstring_view & name, std::wstring &, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -2565,7 +2563,7 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 std::unique_ptr<CFileListCtrlSortBase> CRemoteListView::GetSortComparisonObject()
 {
 	CFileListCtrlSort<CDirectoryListing>::DirSortMode dirSortMode = GetDirSortMode();
-	CFileListCtrlSort<CDirectoryListing>::NameSortMode nameSortMode = GetNameSortMode();
+	NameSortMode nameSortMode = GetNameSortMode();
 
 	CDirectoryListing const& directoryListing = *m_pDirectoryListing;
 	if (!m_sortDirection) {
@@ -2630,7 +2628,7 @@ void CRemoteListView::LinkIsNotDir(CServerPath const& path, std::wstring const&
 {
 	if (m_pLinkResolveState && m_pLinkResolveState->remote_path == path && m_pLinkResolveState->link == link) {
 		std::wstring localFile = CQueueView::ReplaceInvalidCharacters(link);
-		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
+		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
 			localFile = StripVMSRevision(localFile);
 		}
 		m_pQueue->QueueFile(false, true,
@@ -2808,6 +2806,6 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		return;
 	}
 
-	CFileTransferCommand *cmd = new CFileTransferCommand(emptyfile, m_pDirectoryListing->path, newFileName, false, CFileTransferCommand::t_transferSettings());
+	CFileTransferCommand *cmd = new CFileTransferCommand(memory_reader_factory(L"Empty file", std::string_view()), m_pDirectoryListing->path, newFileName, transfer_flags());
 	m_state.m_pCommandQueue->ProcessCommand(cmd);
 }
