@@ -420,7 +420,7 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		if (!child) {
 			CDirectoryListing listing;
 
-			if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
+			if (m_state.engine_->CacheLookup(path, listing) == FZ_REPLY_OK) {
 				child = AppendItem(parent, *iter, 0, 2, path.HasParent() ? 0 : new CItemData(path));
 				SetItemImages(child, false);
 			}
@@ -556,7 +556,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 		CDirectoryListing subListing;
 
-		if (m_state.m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
+		if (m_state.engine_->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
@@ -620,7 +620,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			if (m_state.engine_->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				if (!GetLastChild(child) && HasSubdirs(subListing, filter)) {
 					AppendItem(child, _T(""), -1, -1);
 				}
@@ -650,7 +650,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			if (m_state.engine_->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				last = InsertItem(parent, last, *iter, 0, 2, 0);
 				if (last) {
 					SetItemImages(last, false);
@@ -687,7 +687,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		childPath.AddSegment(*iter);
 
 		CDirectoryListing subListing;
-		if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+		if (m_state.engine_->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 			last = InsertItem(parent, last, *iter, 0, 2, 0);
 			if (last) {
 				SetItemImages(last, false);
@@ -997,7 +997,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		CDirectoryListing listing;
 
 		// ... and it needs to be cached
-		cached = m_state.m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
+		cached = m_state.engine_->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
 		if (cached) {
 			for (size_t i = 0; i < listing.size(); ++i) {
 				auto const& entry = listing[i];
@@ -1055,20 +1055,8 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		chmodDialog.reset();
 		pRecursiveOperation->SetChmodData(std::move(chmodData));
 
-		CServerPath currentPath;
-		const wxTreeItemId selected = GetSelection();
-		if (selected) {
-			currentPath = GetPathFromItem(selected);
-		}
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters(), currentPath);
-	}
-	else {
-		const wxTreeItemId selected = GetSelection();
-		if (selected) {
-			CServerPath currentPath = GetPathFromItem(selected);
-			m_state.ChangeRemoteDir(currentPath);
-		}
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters());
 	}
 }
 
@@ -1102,15 +1090,9 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	pRecursiveOperation->AddRecursionRoot(std::move(root));
 
-	CServerPath currentPath;
-	const wxTreeItemId selected = GetSelection();
-	if (selected) {
-		currentPath = GetPathFromItem(selected);
-	}
-
 	const bool addOnly = event.GetId() == XRCID("ID_ADDTOQUEUE");
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), currentPath, !addOnly);
+	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !addOnly);
 }
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
@@ -1173,7 +1155,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 			m_state.ChangeRemoteDir(startDir);
 		}
 
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters(), currentPath);
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters());
 	}
 }
 
@@ -1371,7 +1353,7 @@ bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 	}
 
 	CDirectoryListing listing;
-	if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
+	if (m_state.engine_->CacheLookup(path, listing) == FZ_REPLY_OK) {
 		RefreshItem(item, listing, false);
 	}
 	else {
@@ -1433,7 +1415,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 		}
 
 		CDirectoryListing listing;
-		if (m_state.m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
+		if (m_state.engine_->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
 			RefreshItem(parent.item, listing, false);
 		else if (filter.HasActiveFilters()) {
 			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child)) {
@@ -1512,18 +1494,18 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 void CRemoteTreeView::UpdateSortMode()
 {
-	CFileListCtrlSortBase::NameSortMode sortMode;
+	NameSortMode sortMode;
 	switch (COptions::Get()->get_int(OPTION_FILELIST_NAMESORT))
 	{
 	case 0:
 	default:
-		sortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		sortMode = NameSortMode::case_insensitive;
 		break;
 	case 1:
-		sortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		sortMode = NameSortMode::case_sensitive;
 		break;
 	case 2:
-		sortMode = CFileListCtrlSortBase::namesort_natural;
+		sortMode = NameSortMode::natural;
 		break;
 	}
 	sortFunction_ = CFileListCtrlSortBase::GetCmpFunction(sortMode);
