@@ -1,15 +1,16 @@
 #ifndef FILEZILLA_INTERFACE_REMOTETREEVIEW_HEADER
 #define FILEZILLA_INTERFACE_REMOTETREEVIEW_HEADER
 
-#include <option_change_event_handler.h>
 #include "systemimagelist.h"
 #include "state.h"
 #include "filter.h"
+#include "option_change_event_handler.h"
 #include "treectrlex.h"
 
+
 class CQueueView;
 class CWindowTinter;
-class CRemoteTreeView final : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
+class CRemoteTreeView final : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, public COptionChangeEventHandler
 {
 	friend class CRemoteTreeViewDropTarget;
 
@@ -53,7 +54,7 @@ protected:
 
 	void UpdateSortMode();
 
-	virtual void OnOptionsChanged(changed_options_t const& options);
+	virtual void OnOptionsChanged(watched_options const& options);
 
 	std::unique_ptr<CWindowTinter> m_windowTinter;
 
