@@ -12,8 +12,8 @@
 
 #include <libfilezilla/translate.hpp>
 
-DECLARE_EVENT_TYPE(fzEVT_PROCESSASYNCREQUESTQUEUE, -1)
-DEFINE_EVENT_TYPE(fzEVT_PROCESSASYNCREQUESTQUEUE)
+wxDECLARE_EVENT(fzEVT_PROCESSASYNCREQUESTQUEUE, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_PROCESSASYNCREQUESTQUEUE, wxCommandEvent);
 
 BEGIN_EVENT_TABLE(CAsyncRequestQueue, wxEvtHandler)
 EVT_COMMAND(wxID_ANY, fzEVT_PROCESSASYNCREQUESTQUEUE, CAsyncRequestQueue::OnProcessQueue)
@@ -378,10 +378,11 @@ bool CAsyncRequestQueue::ProcessFileExistsNotification(t_queueEntry &entry)
 						notification.overwriteAction = CFileExistsNotification::rename;
 						notification.newName = dlg.GetValue().ToStdWstring();
 
-						// If request got processed successfully, notify queue about filename change
-						if (SendReply(entry) && m_pQueueView) {
-							m_pQueueView->RenameFileInTransfer(entry.pEngine, dlg.GetValue().ToStdWstring(), notification.download);
+						if (m_pQueueView) {
+							m_pQueueView->RenameFileInTransfer(entry.pEngine, dlg.GetValue().ToStdWstring(), notification.download, notification.new_writer_factory_);
 						}
+						SendReply(entry);
+
 						return true;
 					}
 				}
@@ -521,6 +522,6 @@ bool CAsyncRequestQueue::SendReply(t_queueEntry& entry)
 void CAsyncRequestQueue::OnStateChange(CState* pState, t_statechange_notifications, std::wstring const&, const void*)
 {
 	if (pState) {
-		ClearPending(pState->m_pEngine);
+		ClearPending(pState->engine_.get());
 	}
 }
