@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "bookmarks_dialog.h"
 #include "filezillaapp.h"
 #include "sitemanager.h"
@@ -9,6 +9,7 @@
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
 
+#include <wx/dirdlg.h>
 #include <wx/statline.h>
 
 BEGIN_EVENT_TABLE(CNewBookmarkDialog, wxDialogEx)
