@@ -7,28 +7,29 @@
 class CNewBookmarkDialog final : public wxDialogEx
 {
 public:
-	CNewBookmarkDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server);
+	CNewBookmarkDialog(wxWindow* parent, std::wstring& site_path, Site const* site);
 
-	int Run(const wxString &local_path, const CServerPath &remote_path);
+	int Run(wxString const& local_path, CServerPath const& remote_path);
 
 protected:
 	wxWindow* m_parent;
 	std::wstring &m_site_path;
-	ServerWithCredentials const* m_server;
+	Site const* site_;
 
 	DECLARE_EVENT_TABLE()
 	void OnOK(wxCommandEvent&);
 	void OnBrowse(wxCommandEvent&);
 };
 
+class wxTreeCtrlEx;
 class CBookmarksDialog final : public wxDialogEx
 {
 public:
-	CBookmarksDialog(wxWindow* parent, std::wstring& site_path, ServerWithCredentials const* server);
+	CBookmarksDialog(wxWindow* parent, std::wstring& site_path, Site const* site);
 
 	int Run();
 
-	static bool GetGlobalBookmarks(std::vector<wxString> &bookmarks);
+	static bool GetGlobalBookmarks(std::vector<std::wstring> &bookmarks);
 	static bool GetBookmark(const wxString& name, wxString &local_dir, CServerPath &remote_dir, bool &sync, bool &comparison);
 	static bool AddBookmark(const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison);
 
@@ -43,11 +44,11 @@ protected:
 	void SaveSiteSpecificBookmarks();
 	void SaveGlobalBookmarks();
 
-	wxWindow* m_parent;
+	wxWindow* m_parent{};
 	std::wstring &m_site_path;
-	ServerWithCredentials const* m_server;
+	Site const* site_{};
 
-	wxTreeCtrl *m_pTree{};
+	wxTreeCtrlEx* tree_{};
 	wxTreeItemId m_bookmarks_global;
 	wxTreeItemId m_bookmarks_site;
 
