@@ -1,18 +1,42 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "buildinfo.h"
 
+#include "../include/version.h"
+
 #include <libfilezilla/format.hpp>
 
-std::wstring CBuildInfo::GetVersion()
+#include <sqlite3.h>
+
+std::wstring GetDependencyVersion(gui_lib_dependency d)
 {
-	return fz::to_wstring(PACKAGE_VERSION);
+	switch (d) {
+	case gui_lib_dependency::wxwidgets:
+		return wxVERSION_NUM_DOT_STRING_T;
+	case gui_lib_dependency::sqlite:
+		return fz::to_wstring_from_utf8(sqlite3_libversion());
+	default:
+		return std::wstring();
+	}
 }
 
+std::wstring GetDependencyName(gui_lib_dependency d)
+{
+	switch (d) {
+	case gui_lib_dependency::wxwidgets:
+		return L"wxWidgets";
+	case gui_lib_dependency::sqlite:
+		return L"SQLite";
+	default:
+		return std::wstring();
+	}
+}
+
+
 std::wstring CBuildInfo::GetBuildDateString()
 {
 	// Get build date. Unfortunately it is in the ugly Mmm dd yyyy format.
 	// Make a good yyyy-mm-dd out of it
-	std::wstring date = fz::to_wstring(__DATE__);
+	std::wstring date = fz::to_wstring(std::string(__DATE__));
 	while (date.find(L"  ") != std::wstring::npos) {
 		fz::replace_substrings(date, L"  ", L" ");
 	}
@@ -59,7 +83,7 @@ std::wstring CBuildInfo::GetBuildDateString()
 
 std::wstring CBuildInfo::GetBuildTimeString()
 {
-	return fz::to_wstring(__TIME__);
+	return fz::to_wstring(std::string(__TIME__));
 }
 
 fz::datetime CBuildInfo::GetBuildDate()
@@ -71,7 +95,7 @@ fz::datetime CBuildInfo::GetBuildDate()
 std::wstring CBuildInfo::GetCompiler()
 {
 #ifdef USED_COMPILER
-	return fz::to_wstring(USED_COMPILER);
+	return fz::to_wstring(std::string(USED_COMPILER));
 #elif defined __VISUALC__
 	int version = __VISUALC__;
 	return fz::sprintf(L"Visual C++ %d", version);
@@ -85,14 +109,14 @@ std::wstring CBuildInfo::GetCompilerFlags()
 #ifndef USED_CXXFLAGS
 	return std::wstring();
 #else
-	return fz::to_wstring(USED_CXXFLAGS);
+	return fz::to_wstring(std::string(USED_CXXFLAGS));
 #endif
 }
 
 std::wstring CBuildInfo::GetBuildType()
 {
 #ifdef BUILDTYPE
-	std::wstring buildtype = fz::to_wstring(BUILDTYPE);
+	std::wstring buildtype = fz::to_wstring(std::string(BUILDTYPE));
 	if (buildtype == L"official" || buildtype == L"nightly") {
 		return buildtype;
 	}
@@ -163,7 +187,7 @@ std::wstring CBuildInfo::GetHostname()
 #ifndef USED_HOST
 	return std::wstring();
 #else
-	return fz::to_wstring(USED_HOST);
+	return fz::to_wstring(std::string(USED_HOST));
 #endif
 }
 
@@ -172,17 +196,17 @@ std::wstring CBuildInfo::GetBuildSystem()
 #ifndef USED_BUILD
 	return std::wstring();
 #else
-	return fz::to_wstring(USED_BUILD);
+	return fz::to_wstring(std::string(USED_BUILD));
 #endif
 }
 
 bool CBuildInfo::IsUnstable()
 {
-	if (GetVersion().find(L"beta") != std::wstring::npos) {
+	if (GetFileZillaVersion().find(L"beta") != std::wstring::npos) {
 		return true;
 	}
 
-	if (GetVersion().find(L"rc") != std::wstring::npos) {
+	if (GetFileZillaVersion().find(L"rc") != std::wstring::npos) {
 		return true;
 	}
 
@@ -224,7 +248,10 @@ std::wstring CBuildInfo::GetCPUCaps(char separator)
 
 	int const max = reg[0];
 
-	// function (aka leave), subfunction (subleave), register, bit, description
+	cpuid(0x80000000, 0, reg);
+	int const extmax = reg[0];
+
+	// function (aka leaf), subfunction (subleaf), register, bit, description
 	std::tuple<int, int, int, int, std::wstring> const caps[] = {
 		std::make_tuple(1, 0, 3, 25, L"sse"),
 		std::make_tuple(1, 0, 3, 26, L"sse2"),
@@ -237,23 +264,69 @@ std::wstring CBuildInfo::GetCPUCaps(char separator)
 		std::make_tuple(1, 0, 2, 25, L"aes"),
 		std::make_tuple(1, 0, 2, 1,  L"pclmulqdq"),
 		std::make_tuple(1, 0, 2, 30, L"rdrnd"),
-		std::make_tuple(7, 0, 1, 3,  L"bmi2"),
+		std::make_tuple(7, 0, 1, 3,  L"bmi"),
 		std::make_tuple(7, 0, 1, 8,  L"bmi2"),
-		std::make_tuple(7, 0, 1, 19, L"adx")
+		std::make_tuple(7, 0, 1, 19, L"adx"),
+		std::make_tuple(0x80000001, 0, 3, 29, L"lm")
 	};
 
 	for (auto const& cap : caps) {
-		if (max >= std::get<0>(cap)) {
-			cpuid(std::get<0>(cap), std::get<1>(cap), reg);
-			if (reg[std::get<2>(cap)] & (1 << std::get<3>(cap))) {
-				if (!ret.empty()) {
-					ret += separator;
-				}
-				ret += std::get<4>(cap);
+		int const leaf = std::get<0>(cap);
+		if (leaf > 0 && max < leaf) {
+			continue;
+		}
+		if (leaf < 0 && leaf > extmax) {
+			continue;
+		}
+
+		cpuid(leaf, std::get<1>(cap), reg);
+		if (reg[std::get<2>(cap)] & (1 << std::get<3>(cap))) {
+			if (!ret.empty()) {
+				ret += separator;
 			}
+			ret += std::get<4>(cap);
 		}
 	}
 #endif
 
 	return ret;
 }
+
+#ifdef FZ_WINDOWS
+namespace {
+bool IsAtLeast(int major, int minor = 0)
+{
+	OSVERSIONINFOEX vi{};
+	vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+	vi.dwMajorVersion = major;
+	vi.dwMinorVersion = minor;
+	vi.dwPlatformId = VER_PLATFORM_WIN32_NT;
+
+	DWORDLONG mask{};
+	VER_SET_CONDITION(mask, VER_MAJORVERSION, VER_GREATER_EQUAL);
+	VER_SET_CONDITION(mask, VER_MINORVERSION, VER_GREATER_EQUAL);
+	VER_SET_CONDITION(mask, VER_PLATFORMID, VER_EQUAL);
+	return VerifyVersionInfo(&vi, VER_MAJORVERSION | VER_MINORVERSION | VER_PLATFORMID, mask) != 0;
+}
+}
+#endif
+
+bool GetRealOsVersion(int& major, int& minor)
+{
+#ifndef FZ_WINDOWS
+	return wxGetOsVersion(&major, &minor) != wxOS_UNKNOWN;
+#else
+	major = 4;
+	minor = 0;
+	while (IsAtLeast(++major, minor))
+	{
+	}
+	--major;
+	while (IsAtLeast(major, ++minor))
+	{
+	}
+	--minor;
+
+	return true;
+#endif
+}
