@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "chmoddialog.h"
 #include "xrc_helper.h"
 
