@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "clearprivatedata.h"
 #include "commandqueue.h"
 #include "ipcmutex.h"
@@ -151,9 +151,8 @@ void CClearPrivateDataDialog::Delete()
 
 bool CClearPrivateDataDialog::ClearReconnect()
 {
-	COptions::Get()->SetOptionXml(OPTION_TAB_DATA, pugi::xml_node());
-	COptions::Get()->RequireCleanup();
-	COptions::Get()->SaveIfNeeded();
+	COptions::Get()->Cleanup();
+	COptions::Get()->Save();
 
 	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
 	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter) {
@@ -168,7 +167,7 @@ bool CClearPrivateDataDialog::ClearReconnect()
 
 void CClearPrivateDataDialog::RemoveXmlFile(std::wstring const& name)
 {
-	std::wstring const path = COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR);
+	std::wstring const path = COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR);
 	if (!name.empty() && !path.empty()) {
 		fz::remove_file(fz::to_native(path + name + L".xml"));
 		fz::remove_file(fz::to_native(path + name + L".xml~"));
