@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "commandqueue.h"
 #include "Mainfrm.h"
 #include "state.h"
@@ -9,7 +9,7 @@
 
 #include <algorithm>
 
-DEFINE_EVENT_TYPE(fzEVT_GRANTEXCLUSIVEENGINEACCESS)
+wxDEFINE_EVENT(fzEVT_GRANTEXCLUSIVEENGINEACCESS, wxCommandEvent);
 
 int CCommandQueue::m_requestIdCounter = 0;
 
@@ -136,7 +136,7 @@ void CCommandQueue::Finish(std::unique_ptr<COperationNotification> && pNotificat
 		return;
 	}
 
-	ProcessReply(pNotification->nReplyCode, pNotification->commandId);
+	ProcessReply(pNotification->replyCode_, pNotification->commandId_);
 }
 
 void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
@@ -145,10 +145,6 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 		return;
 	}
 	if (nReplyCode & FZ_REPLY_DISCONNECTED) {
-		if (commandId == Command::none && !m_CommandList.empty()) {
-			// Pending event, has no relevance during command execution
-			return;
-		}
 		if (nReplyCode & FZ_REPLY_PASSWORDFAILED) {
 			CLoginManager::Get().CachedPasswordFailed(m_state.GetSite().server);
 		}
@@ -158,37 +154,28 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 		return;
 	}
 
+	auto & commandInfo = m_CommandList.front();
+
 	if (commandId != Command::connect &&
 		commandId != Command::disconnect &&
 		(nReplyCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED)
 	{
-		bool reconnect = false;
-		if (nReplyCode == FZ_REPLY_NOTCONNECTED) {
-			reconnect = true;
-		}
-		else if (nReplyCode & FZ_REPLY_DISCONNECTED) {
-			auto & info = m_CommandList.front();
-			if (!info.didReconnect) {
-				info.didReconnect = true;
-				reconnect = true;
-			}
-		}
-
-		if (reconnect) {
-			// Try automatic reconnect
-			Site const& site = m_state.GetSite();
-			if (site) {
-				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(site.server, site.Handle(), site.credentials));
-				ProcessNextCommand();
-				return;
+		if (nReplyCode & FZ_REPLY_DISCONNECTED) {
+			if (!commandInfo.didReconnect) {
+				// Try automatic reconnect
+				commandInfo.didReconnect = true;
+				Site const& site = m_state.GetSite();
+				if (site) {
+					m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(site.server, site.Handle(), site.credentials));
+					ProcessNextCommand();
+					return;
+				}
 			}
 		}
 	}
 
 	++m_inside_commandqueue;
 
-	auto const& commandInfo = m_CommandList.front();
-
 	if (commandInfo.command->GetId() == Command::list && nReplyCode != FZ_REPLY_OK) {
 		if ((nReplyCode & FZ_REPLY_LINKNOTDIR) == FZ_REPLY_LINKNOTDIR) {
 			// Symbolic link does not point to a directory. Either points to file
@@ -308,7 +295,7 @@ void CCommandQueue::ProcessDirectoryListing(CDirectoryListingNotification const&
 	if (!listingNotification.GetPath().empty()) {
 		pListing = std::make_shared<CDirectoryListing>();
 		if (listingNotification.Failed() ||
-			m_state.m_pEngine->CacheLookup(listingNotification.GetPath(), *pListing) != FZ_REPLY_OK)
+			m_state.engine_->CacheLookup(listingNotification.GetPath(), *pListing) != FZ_REPLY_OK)
 		{
 			pListing = std::make_shared<CDirectoryListing>();
 			pListing->path = listingNotification.GetPath();
