@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "cmdline.h"
 #include "commandqueue.h"
 #include "context_control.h"
@@ -25,10 +25,13 @@
 #include "osx_sandbox_userdirs.h"
 #endif
 
+#include <wx/menu.h>
 #include <wx/wupdlock.h>
 
-DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1)
-DEFINE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED)
+#include <array>
+
+wxDECLARE_EVENT(fzEVT_TAB_CLOSING_DEFERRED, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_TAB_CLOSING_DEFERRED, wxCommandEvent);
 
 BEGIN_EVENT_TABLE(CContextControl, wxSplitterWindow)
 EVT_MENU(XRCID("ID_TABCONTEXT_REFRESH"), CContextControl::OnTabRefresh)
@@ -91,7 +94,7 @@ bool CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 
 		// See if we can reuse an existing context
 		for (size_t i = 0; i < m_context_controls.size(); i++) {
-			if (m_context_controls[i].used()) {
+			if (m_context_controls[i].used() || !m_context_controls[i].pState) {
 				continue;
 			}
 
@@ -140,10 +143,6 @@ bool CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 		CContextManager::Get()->SetCurrentContext(pState);
 	}
 
-	if (!m_mainFrame.RestoreSplitterPositions()) {
-		m_mainFrame.SetDefaultSplitterPositions();
-	}
-
 	if (m_tabs) {
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
 	}
@@ -162,22 +161,44 @@ void CContextControl::CreateContextControls(CState& state)
 #else
 	const wxPoint initial_position(wxDefaultPosition);
 #endif
+	wxSize paneSizes[3] = { {-1, -1}, {-1, -1}, {-1, -1}};
+	std::tuple<double, int, int> splitterPositions;
 
 	if (!m_context_controls.empty()) {
+
+		auto & currentControls = m_context_controls[m_current_context_controls != -1 ? m_current_context_controls : 0];
+
+		if (currentControls.pViewSplitter) {
+			paneSizes[0] = currentControls.pViewSplitter->GetSize();
+		}
+		if (currentControls.pLocalSplitter) {
+			paneSizes[1] = currentControls.pLocalSplitter->GetSize();
+		}
+		if (currentControls.pRemoteSplitter) {
+			paneSizes[2] = currentControls.pRemoteSplitter->GetSize();
+		}
+
+		splitterPositions = currentControls.GetSplitterPositions();
+		if (currentControls.pLocalListView) {
+			currentControls.pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
+		}
+		if (currentControls.pRemoteListView) {
+			currentControls.pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
+		}
+		
 		if (!m_tabs) {
 			m_tabs = new wxAuiNotebookEx();
 
-			wxSize splitter_size = m_context_controls[m_current_context_controls].pViewSplitter->GetSize();
-			m_tabs->Create(this, wxID_ANY, initial_position, splitter_size, wxNO_BORDER | wxAUI_NB_SCROLL_BUTTONS | wxAUI_NB_WINDOWLIST_BUTTON | wxAUI_NB_CLOSE_ON_ALL_TABS | wxAUI_NB_TAB_MOVE);
+			m_tabs->Create(this, wxID_ANY, initial_position, paneSizes[0], wxNO_BORDER | wxAUI_NB_SCROLL_BUTTONS | wxAUI_NB_WINDOWLIST_BUTTON | wxAUI_NB_CLOSE_ON_ALL_TABS | wxAUI_NB_TAB_MOVE);
 			m_tabs->SetExArtProvider();
 			m_tabs->SetSelectedFont(*wxNORMAL_FONT);
 			m_tabs->SetMeasuringFont(*wxNORMAL_FONT);
 
-			m_context_controls[m_current_context_controls].pViewSplitter->Reparent(m_tabs);
+			currentControls.pViewSplitter->Reparent(m_tabs);
 
-			m_tabs->AddPage(m_context_controls[m_current_context_controls].pViewSplitter, m_context_controls[m_current_context_controls].pState->GetTitle());
-			m_tabs->SetTabColour(0, m_context_controls[m_current_context_controls].pState->GetSite().m_colour);
-			ReplaceWindow(m_context_controls[m_current_context_controls].pViewSplitter, m_tabs);
+			m_tabs->AddPage(currentControls.pViewSplitter, currentControls.pState->GetTitle());
+			m_tabs->SetTabColour(0, currentControls.pState->GetSite().m_colour);
+			ReplaceWindow(currentControls.pViewSplitter, m_tabs);
 
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(CContextControl::OnTabChanged), 0, this);
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE, wxAuiNotebookEventHandler(CContextControl::OnTabClosing), 0, this);
@@ -191,24 +212,23 @@ void CContextControl::CreateContextControls(CState& state)
 #endif
 		}
 
-		m_mainFrame.RememberSplitterPositions();
-		m_context_controls[m_current_context_controls].pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
-		m_context_controls[m_current_context_controls].pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
-
 		parent = m_tabs;
 	}
 
 	CContextControl::_context_controls context_controls;
 
 	context_controls.pState = &state;
-	context_controls.pViewSplitter = new CSplitterWindowEx(parent, -1, initial_position, wxDefaultSize, wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pViewSplitter = new CSplitterWindowEx(parent, -1, initial_position, paneSizes[0], wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pViewSplitter->SetMinSize(wxDefaultSize);
 	context_controls.pViewSplitter->SetMinimumPaneSize(50, 100);
 	context_controls.pViewSplitter->SetSashGravity(0.5);
 
-	context_controls.pLocalSplitter = new CSplitterWindowEx(context_controls.pViewSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pLocalSplitter = new CSplitterWindowEx(context_controls.pViewSplitter, -1, wxDefaultPosition, paneSizes[1], wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pLocalSplitter->SetMinSize(wxDefaultSize);
 	context_controls.pLocalSplitter->SetMinimumPaneSize(50, 100);
 
-	context_controls.pRemoteSplitter = new CSplitterWindowEx(context_controls.pViewSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pRemoteSplitter = new CSplitterWindowEx(context_controls.pViewSplitter, -1, wxDefaultPosition, paneSizes[2], wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
+	context_controls.pRemoteSplitter->SetMinSize(wxDefaultSize);
 	context_controls.pRemoteSplitter->SetMinimumPaneSize(50, 100);
 
 	context_controls.pLocalTreeViewPanel = new CView(context_controls.pLocalSplitter);
@@ -225,7 +245,7 @@ void CContextControl::CreateContextControls(CState& state)
 	context_controls.pRemoteTreeViewPanel->SetWindow(context_controls.pRemoteTreeView);
 	context_controls.pRemoteListViewPanel->SetWindow(context_controls.pRemoteListView);
 
-	bool show_filelist_statusbars = COptions::Get()->GetOptionVal(OPTION_FILELIST_STATUSBAR) != 0;
+	bool show_filelist_statusbars = COptions::Get()->get_int(OPTION_FILELIST_STATUSBAR) != 0;
 
 	CFilelistStatusBar* pLocalFilelistStatusBar = new CFilelistStatusBar(context_controls.pLocalListViewPanel);
 	if (!show_filelist_statusbars) {
@@ -258,8 +278,8 @@ void CContextControl::CreateContextControls(CState& state)
 	context_controls.pRemoteListSearchPanel = new CListSearchPanel(context_controls.pRemoteListViewPanel, context_controls.pRemoteListView, context_controls.pState, false);
 	context_controls.pRemoteListViewPanel->SetSearchPanel(context_controls.pRemoteListSearchPanel);
 
-	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
-	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
+	const int layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
+	const int swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
 
 	if (layout == 1) {
 		if (swap) {
@@ -278,7 +298,7 @@ void CContextControl::CreateContextControls(CState& state)
 		}
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL)) {
+	if (COptions::Get()->get_int(OPTION_SHOW_TREE_LOCAL)) {
 		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, state);
 		context_controls.pLocalTreeViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		if (layout == 3 && swap) {
@@ -298,7 +318,7 @@ void CContextControl::CreateContextControls(CState& state)
 		context_controls.pLocalSplitter->Initialize(context_controls.pLocalListViewPanel);
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE)) {
+	if (COptions::Get()->get_int(OPTION_SHOW_TREE_REMOTE)) {
 		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, state);
 		context_controls.pRemoteTreeViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		if (layout == 3 && !swap) {
@@ -327,6 +347,15 @@ void CContextControl::CreateContextControls(CState& state)
 		}
 	}
 
+	if (!m_context_controls.empty()) {
+		context_controls.SetSplitterPositions(splitterPositions);
+	}
+	else {
+		context_controls.pViewSplitter->SetRelativeSashPosition(0.5);
+		context_controls.pLocalSplitter->SetRelativeSashPosition(0.4);
+		context_controls.pLocalSplitter->SetRelativeSashPosition(0.4);
+	}
+
 	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalListView);
 	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteListView);
 	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalTreeView);
@@ -677,7 +706,7 @@ void CContextControl::SaveTabs()
 		}
 	}
 
-	COptions::Get()->SetOptionXml(OPTION_TAB_DATA, xml);
+	COptions::Get()->set(OPTION_TAB_DATA, xml);
 }
 
 void CContextControl::RestoreTabs()
@@ -688,17 +717,22 @@ void CContextControl::RestoreTabs()
 
 	int selected = 0;
 
-	auto xml = COptions::Get()->GetOptionXml(OPTION_TAB_DATA);
+	auto xml = COptions::Get()->get_xml(OPTION_TAB_DATA);
 
-	bool selectedOnly = COptions::Get()->GetOptionVal(OPTION_STARTUP_ACTION) != 2;
+	bool selectedOnly = COptions::Get()->get_int(OPTION_STARTUP_ACTION) != 2;
 
 	CCommandLine const* pCommandLine = wxGetApp().GetCommandLine();
 	if (pCommandLine && pCommandLine->BlocksReconnectAtStartup()) {
 		selectedOnly = true;
 	}
 
-	pugi::xml_node tabs = xml ? xml->child("Tabs") : pugi::xml_node();
+	pugi::xml_node tabs = xml.child("Tabs");
 	if (tabs) {
+#ifdef __WXMSW__
+		// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
+		// on OS X.
+		wxWindowUpdateLocker lock(this);
+#endif
 		for (auto tab = tabs.child("Tab"); tab; tab = tab.next_sibling("Tab")) {
 
 			if (tab.attribute("selected").as_int()) {
@@ -735,3 +769,66 @@ void CContextControl::RestoreTabs()
 
 	SelectTab(selected);
 }
+
+namespace {
+bool SwitchFocus(wxWindow *focus, wxWindow *first, wxWindow *second)
+{
+	if (focus == first) {
+		if (second && second->IsShownOnScreen() && second->IsEnabled()) {
+			second->SetFocus();
+		}
+		return true;
+	}
+	return false;
+}
+}
+
+void CContextControl::_context_controls::SwitchFocusedSide()
+{
+	std::array<std::pair<wxWindow*, wxWindow*>, 3> ctrls =
+	{{
+		{pLocalListView, pRemoteListView},
+		{pLocalTreeView, pRemoteTreeView},
+		{pLocalViewHeader, pRemoteViewHeader}
+	}};
+	auto *focus = wxWindow::FindFocus();
+	while (focus) {
+		for (auto & p : ctrls) {
+			if (SwitchFocus(focus, p.first, p.second)) {
+					return;
+			}
+			if (SwitchFocus(focus, p.second, p.first)) {
+					return;
+			}
+		}
+		focus = focus->GetParent();
+	}
+}
+
+std::tuple<double, int, int> CContextControl::_context_controls::GetSplitterPositions()
+{
+	std::tuple<double, int, int> ret;
+
+	std::get<0>(ret) = pViewSplitter ? pViewSplitter->GetRelativeSashPosition() : 0.5f;
+	std::get<1>(ret) = pLocalSplitter ? pLocalSplitter->GetSashPosition() : 135;
+	std::get<2>(ret) = pRemoteSplitter ? pRemoteSplitter->GetSashPosition() : 135;
+
+	return ret;
+}
+
+void CContextControl::_context_controls::SetSplitterPositions(std::tuple<double, int, int> const& positions)
+{
+	if (pViewSplitter) {
+		double pos = std::get<0>(positions);
+		if (pos < 0 || pos > 1) {
+			pos = 0.5;
+		}
+		pViewSplitter->SetRelativeSashPosition(pos);
+	}
+	if (pLocalSplitter) {
+		pLocalSplitter->SetSashPosition(std::get<1>(positions));
+	}
+	if (pRemoteSplitter) {
+		pRemoteSplitter->SetSashPosition(std::get<2>(positions));
+	}
+}
