@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "dialogex.h"
 #include "msgbox.h"
 #include "xrc_helper.h"
@@ -14,10 +14,14 @@ BEGIN_EVENT_TABLE(wxDialogEx, wxDialog)
 EVT_CHAR_HOOK(wxDialogEx::OnChar)
 END_EVENT_TABLE()
 
-int wxDialogEx::m_shown_dialogs = 0;
+std::vector<wxDialogEx*> wxDialogEx::shown_dialogs_;
 
 #ifdef __WXMAC__
+std::vector<void*> wxDialogEx::shown_dialogs_creation_events_;
+
 static int const pasteId = wxNewId();
+static int const copyId = wxNewId();
+static int const cutId = wxNewId();
 static int const selectAllId = wxNewId();
 
 extern wxTextEntry* GetSpecialTextEntry(wxWindow*, wxChar);
@@ -33,6 +37,14 @@ bool wxDialogEx::ProcessEvent(wxEvent& event)
 		e->Paste();
 		return true;
 	}
+	else if (e && event.GetId() == copyId) {
+		e->Copy();
+		return true;
+	}
+	else if (e && event.GetId() == cutId) {
+		e->Cut();
+		return true;
+	}
 	else if (e && event.GetId() == selectAllId) {
 		e->SelectAll();
 		return true;
@@ -128,7 +140,10 @@ int wxDialogEx::ShowModal()
 	::ReleaseCapture();
 #endif
 
-	++m_shown_dialogs;
+	shown_dialogs_.push_back(this);
+#ifdef __WXMAC__
+	shown_dialogs_creation_events_.push_back(wxGetApp().MacGetCurrentEvent());
+#endif
 
 	if (acceleratorTable_.empty()) {
 		SetAcceleratorTable(wxNullAcceleratorTable);
@@ -139,7 +154,10 @@ int wxDialogEx::ShowModal()
 
 	int ret = wxDialog::ShowModal();
 
-	--m_shown_dialogs;
+#ifdef __WXMAC__
+	shown_dialogs_creation_events_.pop_back();
+#endif
+	shown_dialogs_.pop_back();
 
 	return ret;
 }
@@ -154,10 +172,15 @@ bool wxDialogEx::ReplaceControl(wxWindow* old, wxWindow* wnd)
 	return true;
 }
 
-bool wxDialogEx::CanShowPopupDialog()
+bool wxDialogEx::CanShowPopupDialog(wxTopLevelWindow * parent)
 {
-	if (m_shown_dialogs != 0 || IsShowingMessageBox()) {
-		// There already is a dialog or message box showing
+	if (IsShowingMessageBox()) {
+		// There already a message box showing
+		return false;
+	}
+
+	if (!shown_dialogs_.empty() && shown_dialogs_.back() != parent) {
+		// There is an open dialog which isn't the expected parent
 		return false;
 	}
 
@@ -174,7 +197,8 @@ bool wxDialogEx::CanShowPopupDialog()
 #endif
 
 #ifdef __WXMAC__
-	if (wxGetApp().MacGetCurrentEvent()) {
+	void* ev = wxGetApp().MacGetCurrentEvent();
+	if (ev && (shown_dialogs_creation_events_.empty() || ev != shown_dialogs_creation_events_.back())) {
 		// We're inside an event handler for a native mac event, such as a popup menu
 		return false;
 	}
@@ -217,12 +241,14 @@ wxSizerFlags const DialogLayout::grow(wxSizerFlags().Expand());
 wxSizerFlags const DialogLayout::valign(wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 wxSizerFlags const DialogLayout::halign(wxSizerFlags().Align(wxALIGN_CENTER_HORIZONTAL));
 wxSizerFlags const DialogLayout::valigng(wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Expand());
+wxSizerFlags const DialogLayout::ralign(wxSizerFlags().Align(wxALIGN_RIGHT));
 
 DialogLayout::DialogLayout(wxTopLevelWindow * parent)
 	: parent_(parent)
 {
 	gap = dlgUnits(3);
 	border = dlgUnits(3);
+	indent = dlgUnits(10);
 }
 
 int DialogLayout::dlgUnits(int num) const
@@ -247,6 +273,12 @@ wxFlexGridSizer* DialogLayout::createFlex(int cols, int rows) const
 	return new wxFlexGridSizer(rows, cols, g, g);
 }
 
+wxGridSizer* DialogLayout::createGrid(int cols, int rows) const
+{
+	int const g = gap;
+	return new wxGridSizer(rows, cols, g, g);
+}
+
 wxGridBagSizer* DialogLayout::createGridBag(int cols, int rows) const
 {
 	int const g = gap;
@@ -312,10 +344,17 @@ std::tuple<wxStaticBox*, wxFlexGridSizer*> DialogLayout::createStatBox(wxSizer*
 {
 	auto* boxSizer = new wxStaticBoxSizer(wxHORIZONTAL, parent->GetContainingWindow(), title);
 	auto* box = boxSizer->GetStaticBox();
-	parent->Add(boxSizer, 1, wxGROW);
+	parent->Add(boxSizer, nullID, wxGROW);
 
 	auto* flex = createFlex(cols, rows);
-	boxSizer->Add(flex, 1, wxALL|wxGROW, border);
+#ifdef __WXMSW__
+	int style = wxGROW | wxLEFT | wxBOTTOM | wxRIGHT;
+	int const b = dlgUnits(2);
+#else
+	int style = wxGROW | wxALL;
+	int const b = border;
+#endif
+	boxSizer->Add(flex, 1, style, b);
 
 	return std::make_tuple(box, flex);
 }
@@ -329,6 +368,8 @@ std::wstring LabelEscape(std::wstring const& label)
 void FixPasswordPaste(std::vector<wxAcceleratorEntry> & entries)
 {
 	entries.emplace_back(wxACCEL_CMD, 'V', pasteId);
+	entries.emplace_back(wxACCEL_CMD, 'C', copyId);
+	entries.emplace_back(wxACCEL_CMD, 'X', cutId);
 	entries.emplace_back(wxACCEL_CMD, 'A', selectAllId);
 }
 #endif
