@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "export.h"
 #include "filezillaapp.h"
 #include "xmlfunctions.h"
@@ -6,6 +6,8 @@
 #include "queue.h"
 #include "xrc_helper.h"
 
+#include <wx/filedlg.h>
+
 CExportDialog::CExportDialog(wxWindow* parent, CQueueView* pQueueView)
 	: m_parent(parent), m_pQueueView(pQueueView)
 {
@@ -73,6 +75,7 @@ void CExportDialog::Run()
 		}
 	}
 	if (settings) {
+		COptions::Get()->Save();
 		CInterProcessMutex mutex(MUTEX_OPTIONS);
 		CXmlFile file(wxGetApp().GetSettingsFile(_T("filezilla")));
 		auto document = file.Load();
