@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "fileexistsdlg.h"
 #include "file_utils.h"
 #include "Options.h"
@@ -10,6 +10,10 @@
 #include <wx/display.h>
 #include <wx/statbox.h>
 
+#ifndef __WXMSW__
+#include <wx/mimetype.h>
+#endif
+
 BEGIN_EVENT_TABLE(CFileExistsDlg, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CFileExistsDlg::OnOK)
 EVT_BUTTON(XRCID("wxID_CANCEL"), CFileExistsDlg::OnCancel)
@@ -68,7 +72,7 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	targetDetails->Add(new wxStaticText(this, XRCID("ID_FILE1_SIZE"), wxString()));
 	targetDetails->Add(new wxStaticText(this, XRCID("ID_FILE1_TIME"), wxString()));
 
-	inner->AddSpacer(lay.dlgUnits(10));
+	inner->AddSpacer(lay.indent);
 
 	auto* right = lay.createFlex(1);
 	inner->Add(right);
@@ -87,8 +91,8 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 
 
 	right->Add(new wxCheckBox(this, XRCID("ID_ALWAYS"), _("&Always use this action")));
-	right->Add(new wxCheckBox(this, XRCID("ID_QUEUEONLY"), _("Apply to &current queue only")), 0, wxLEFT, lay.dlgUnits(10));
-	right->Add(new wxCheckBox(this, XRCID("ID_UPDOWNONLY"), wxString()), 0, wxLEFT, lay.dlgUnits(10));
+	right->Add(new wxCheckBox(this, XRCID("ID_QUEUEONLY"), _("Apply to &current queue only")), 0, wxLEFT, lay.indent);
+	right->Add(new wxCheckBox(this, XRCID("ID_UPDOWNONLY"), wxString()), 0, wxLEFT, lay.indent);
 
 	auto* buttons = lay.createButtonSizer(this, main, true);
 
@@ -113,13 +117,13 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	return true;
 }
 
-void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t size, fz::datetime const& time, wxString const& iconFile)
+void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t size, fz::datetime const& time, std::wstring const& iconFile)
 {
 	std::wstring labelName = LabelEscape(GetPathEllipsis(name, FindWindow(left ? XRCID("ID_FILE1_NAME") : XRCID("ID_FILE2_NAME"))));
 
 	wxString sizeStr = _("Size unknown");
 	if (size >= 0) {
-		bool const thousands_separator = COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0;
+		bool const thousands_separator = COptions::Get()->get_int(OPTION_SIZE_USETHOUSANDSEP) != 0;
 		sizeStr = CSizeFormat::Format(size, true, CSizeFormat::bytes, thousands_separator, 0);
 	}
 
@@ -148,7 +152,7 @@ bool CFileExistsDlg::SetupControls()
 	return true;
 }
 
-void CFileExistsDlg::LoadIcon(int id, const wxString &file)
+void CFileExistsDlg::LoadIcon(int id, std::wstring const& file)
 {
 	wxStaticBitmap *pStatBmp = static_cast<wxStaticBitmap *>(FindWindow(id));
 	if (!pStatBmp) {
@@ -158,12 +162,11 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 	wxSize size = CThemeProvider::GetIconSize(iconSizeNormal);
 	pStatBmp->SetInitialSize(size);
 	pStatBmp->InvalidateBestSize();
-	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
 	memset(&fileinfo, 0, sizeof(fileinfo));
-	if (SHGetFileInfo(file.wc_str(), FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
+	if (SHGetFileInfo(file.c_str(), FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
 		wxBitmap bmp;
 		bmp.Create(size.x, size.y);
 
@@ -189,53 +192,48 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 
 		return;
 	}
+#else
+	std::wstring ext = GetExtension(file);
+	if (!ext.empty() && ext != L".") {
+		std::unique_ptr<wxFileType> pType(wxTheMimeTypesManager->GetFileTypeFromExtension(ext));
+		if (pType) {
+			wxIconLocation loc;
+			if (pType->GetIcon(&loc) && loc.IsOk()) {
+				wxLogNull* tmp = new wxLogNull;
+				wxIcon icon(loc);
+				delete tmp;
+				if (icon.Ok()) {
+					int width = icon.GetWidth();
+					int height = icon.GetHeight();
+					if (width && height) {
+						wxBitmap bmp;
+						bmp.Create(icon.GetWidth(), icon.GetHeight());
 
-#endif //__WXMSW__
+						wxMemoryDC* dc = new wxMemoryDC;
 
-	std::wstring ext = GetExtension(file.ToStdWstring());
-	if (ext.empty()) {
-		return;
-	}
+						wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
+						wxBrush brush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 
-	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-	if (pType) {
-		wxIconLocation loc;
-		if (pType->GetIcon(&loc) && loc.IsOk()) {
-			wxLogNull *tmp = new wxLogNull;
-			wxIcon icon(loc);
-			delete tmp;
-			if (!icon.Ok()) {
-				delete pType;
-				return;
-			}
+						dc->SelectObject(bmp);
 
-			int width = icon.GetWidth();
-			int height = icon.GetHeight();
-			if (width && height) {
-				wxBitmap bmp;
-				bmp.Create(icon.GetWidth(), icon.GetHeight());
+						dc->SetPen(pen);
+						dc->SetBrush(brush);
+						dc->DrawRectangle(0, 0, width, height);
 
-				wxMemoryDC *dc = new wxMemoryDC;
+						dc->DrawIcon(icon, 0, 0);
+						delete dc;
 
-				wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
-				wxBrush brush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
+						pStatBmp->SetBitmap(bmp);
 
-				dc->SelectObject(bmp);
-
-				dc->SetPen(pen);
-				dc->SetBrush(brush);
-				dc->DrawRectangle(0, 0, width, height);
-
-				dc->DrawIcon(icon, 0, 0);
-				delete dc;
-
-				pStatBmp->SetBitmap(bmp);
-
-				return;
+						return;
+					}
+				}
 			}
 		}
-		delete pType;
 	}
+#endif
+
+	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 }
 
 void CFileExistsDlg::OnOK(wxCommandEvent&)
