@@ -1,14 +1,16 @@
-#ifndef __FILELIST_STATUSBAR_H__
-#define __FILELIST_STATUSBAR_H__
+#ifndef FILEZILLA_INTERFACE_FILELIST_STATUSBAR_HEADER
+#define FILEZILLA_INTERFACE_FILELIST_STATUSBAR_HEADER
 
-#include <option_change_event_handler.h>
+#include "option_change_event_handler.h"
 
+#include <wx/statusbr.h>
 #include <wx/timer.h>
 
-class CFilelistStatusBar final : public wxStatusBar, protected COptionChangeEventHandler
+class CFilelistStatusBar final : public wxStatusBar, public COptionChangeEventHandler
 {
 public:
 	CFilelistStatusBar(wxWindow* pParent);
+	~CFilelistStatusBar();
 
 	void SetDirectoryContents(int count_files, int count_dirs, int64_t total_size, int unknown_size, int hidden);
 	void Clear();
@@ -33,7 +35,7 @@ public:
 	void SetConnected(bool connected);
 protected:
 
-	virtual void OnOptionsChanged(changed_options_t const& options);
+	virtual void OnOptionsChanged(watched_options const& options);
 
 	bool m_connected{};
 	int m_count_files{};
@@ -56,4 +58,4 @@ protected:
 	void OnTimer(wxTimerEvent& event);
 };
 
-#endif //__FILELIST_STATUSBAR_H__
+#endif
