@@ -1,6 +1,6 @@
 #ifndef FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 // This works around a bug in GCC, appears to be [Bug pch/12707]
-#include <filezilla.h>
+#include "filezilla.h"
 #endif
 #include "filelistctrl.h"
 #include "filezillaapp.h"
@@ -9,16 +9,21 @@
 #include <algorithm>
 #include "filelist_statusbar.h"
 #include "themeprovider.h"
+
+#ifndef __WXMSW__
+#include <wx/mimetype.h>
+#endif
+
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
 #include <gtk/gtk.h>
 #endif
 
 #ifndef __WXMSW__
-DECLARE_EVENT_TYPE(fz_EVT_FILELIST_FOCUSCHANGE, -1)
-DECLARE_EVENT_TYPE(fz_EVT_DEFERRED_MOUSEEVENT, -1)
+wxDECLARE_EVENT(fz_EVT_FILELIST_FOCUSCHANGE, wxCommandEvent);
+wxDECLARE_EVENT(fz_EVT_DEFERRED_MOUSEEVENT, wxCommandEvent);
 #ifndef FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
-DEFINE_EVENT_TYPE(fz_EVT_FILELIST_FOCUSCHANGE)
-DEFINE_EVENT_TYPE(fz_EVT_DEFERRED_MOUSEEVENT)
+wxDEFINE_EVENT(fz_EVT_FILELIST_FOCUSCHANGE, wxCommandEvent);
+wxDEFINE_EVENT(fz_EVT_DEFERRED_MOUSEEVENT, wxCommandEvent);
 #endif
 #endif
 
@@ -276,6 +281,16 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 #ifndef __WXMSW__
 	GetMainWindow()->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 #endif
+
+#ifdef __WXMSW__
+	Bind(wxEVT_SYS_COLOUR_CHANGED, [this](wxSysColourChangedEvent& evt) {
+		// Delay refresh until parent had time to process change.
+		CallAfter([this](){
+			InitColors();
+		});
+		evt.Skip();
+	});
+#endif
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=-1*/, int direction /*=-1*/, bool updateSelections /*=true*/)
@@ -341,7 +356,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		}
 	}
 
-	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
+	const int dirSortOption = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
 
 	if (column == m_sortColumn && direction != m_sortDirection && !m_indexMapping.empty() &&
 		dirSortOption != 1)
@@ -414,7 +429,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 
 template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
 {
-	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
+	const int dirSortOption = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
 
 	CFileListCtrlSortBase::DirSortMode dirSortMode;
 	switch (dirSortOption)
@@ -439,22 +454,22 @@ template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFile
 	return dirSortMode;
 }
 
-template<class CFileData> CFileListCtrlSortBase::NameSortMode CFileListCtrl<CFileData>::GetNameSortMode()
+template<class CFileData> NameSortMode CFileListCtrl<CFileData>::GetNameSortMode()
 {
-	const int nameSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT);
+	const int nameSortOption = COptions::Get()->get_int(OPTION_FILELIST_NAMESORT);
 
-	CFileListCtrlSortBase::NameSortMode nameSortMode;
+	NameSortMode nameSortMode;
 	switch (nameSortOption)
 	{
 	case 0:
 	default:
-		nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		nameSortMode = NameSortMode::case_insensitive;
 		break;
 	case 1:
-		nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		nameSortMode = NameSortMode::case_sensitive;
 		break;
 	case 2:
-		nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		nameSortMode = NameSortMode::natural;
 		break;
 	}
 
@@ -762,9 +777,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 	ShowColumnEditor();
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
+template<class CFileData> void CFileListCtrl<CFileData>::InitSort(interfaceOptions optionID)
 {
-	wxString sortInfo = COptions::Get()->GetOption(optionID);
+	wxString sortInfo = COptions::Get()->get_string(optionID);
 	if (!sortInfo.empty()) {
 		m_sortDirection = sortInfo[0] - '0';
 	}
@@ -928,9 +943,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 		return;
 	}
 
-	if (old_focus != -1) {
+	if (old_focus >= 0) {
 		bool selected = GetItemState(old_focus, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-		if (!selected && m_selections[old_focus]) {
+		if (!selected && static_cast<size_t>(old_focus) < m_selections.size() && m_selections[old_focus]) {
 			// Need to deselect all
 			if (m_pFilelistStatusBar) {
 				m_pFilelistStatusBar->UnselectAll();
@@ -967,6 +982,15 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 
 template <class CFileData> void CFileListCtrl<CFileData>::UpdateSelections(int min, int max)
 {
+	if (min < 0 || min > max) {
+		return;
+	}
+
+	size_t smax = static_cast<size_t>(max);
+	if (m_selections.size() <= smax || m_indexMapping.size() <= smax || m_fileData.size() <= smax) {
+		return;
+	}
+
 	for (int i = min; i <= max; ++i) {
 		bool selected = GetItemState(i, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
 		if (selected == m_selections[i]) {
@@ -1074,14 +1098,19 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 
 	const int code = event.GetKeyCode();
 	const int mods = event.GetModifiers();
-	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META))) {
+	if (code == 'A' && (mods & wxMOD_CMD || (mods & (wxMOD_CONTROL | wxMOD_META)) == (wxMOD_CONTROL | wxMOD_META))) {
+		int mask = fill;
+		if (mods & wxMOD_SHIFT) {
+			mask |= normal;
+		}
+
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
 			const CFileData& data = m_fileData[m_indexMapping[i]];
-			if (data.comparison_flags != fill) {
-				SetSelection(i, true);
+			if (data.comparison_flags & mask) {
+				SetSelection(i, false);
 			}
 			else {
-				SetSelection(i, false);
+				SetSelection(i, true);
 			}
 		}
 		if (m_hasParent && GetItemCount()) {
