@@ -1,7 +1,8 @@
 #ifndef FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
 #define FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
 
-#include "local_path.h"
+#include "../include/local_path.h"
+#include "../include/engine_options.h"
 
 #include <vector>
 
@@ -45,7 +46,7 @@ public:
 	void AddStartupProfileRecord(std::string const& msg);
 
 protected:
-	void CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, std::string const& env, int setting, std::wstring const& description);
+	void CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, char const* env, engineOptions setting, std::wstring const& description);
 
 	bool InitDefaultsDir();
 	bool LoadResourceFiles();
