@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "fzputtygen_interface.h"
 #include "Options.h"
 #include "filezillaapp.h"
@@ -7,6 +7,8 @@
 #include <libfilezilla/process.hpp>
 #include <libfilezilla/translate.hpp>
 
+#include <wx/filedlg.h>
+
 CFZPuttyGenInterface::CFZPuttyGenInterface(wxWindow* parent)
 	: m_parent(parent)
 {
@@ -169,7 +171,7 @@ bool CFZPuttyGenInterface::LoadProcess(bool silent)
 	}
 	m_initialized = true;
 
-	std::wstring executable = COptions::Get()->GetOption(OPTION_FZSFTP_EXECUTABLE);
+	std::wstring executable = COptions::Get()->get_string(OPTION_FZSFTP_EXECUTABLE);
 	size_t pos = executable.rfind(wxFileName::GetPathSeparator());
 	if (pos == std::wstring::npos) {
 		if (!silent) {
