@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "filezillaapp.h"
 #include "import.h"
 #include "ipcmutex.h"
@@ -8,6 +8,8 @@
 #include "queue.h"
 #include "xrc_helper.h"
 
+#include <wx/filedlg.h>
+
 CImportDialog::CImportDialog(wxWindow* parent, CQueueView* pQueueView)
 	: m_parent(parent), m_pQueueView(pQueueView)
 {
@@ -26,7 +28,7 @@ void CImportDialog::Run()
 
 	wxFileName fn(dlg.GetPath());
 	wxString const path = fn.GetPath();
-	wxString const settingsDir(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR));
+	wxString const settingsDir(COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR));
 	if (path == settingsDir) {
 		wxMessageBoxEx(_("You cannot import settings from FileZilla's own settings directory."), _("Error importing"), wxICON_ERROR, m_parent);
 		return;
@@ -79,7 +81,8 @@ void CImportDialog::Run()
 			}
 
 			if (settings && xrc_call(*this, "ID_SETTINGS", &wxCheckBox::IsChecked)) {
-				COptions::Get()->Import(fz3Root.child("Settings"));
+				auto settings = fz3Root.child("Settings");
+				COptions::Get()->Import(settings);
 				wxMessageBoxEx(_("The settings have been imported. You have to restart FileZilla for all settings to have effect."), _("Import successful"), wxOK, this);
 			}
 
