@@ -1,10 +1,10 @@
-#ifndef __LED_H__
-#define __LED_H__
+#ifndef FILEZILLA_INTERFACE_LED_HEADER
+#define FILEZILLA_INTERFACE_LED_HEADER
 
 #include <wx/event.h>
 #include <wx/timer.h>
 
-DECLARE_EVENT_TYPE(fzEVT_UPDATE_LED_TOOLTIP, -1)
+wxDECLARE_EVENT(fzEVT_UPDATE_LED_TOOLTIP, wxCommandEvent);
 
 class CLed final : public wxWindow
 {
@@ -31,4 +31,4 @@ protected:
 	void OnEnterWindow(wxMouseEvent& event);
 };
 
-#endif //__LED_H__
+#endif
