@@ -20,8 +20,8 @@
 #include "themeprovider.h"
 #endif
 
-DECLARE_EVENT_TYPE(fzEVT_POSTSCROLL, -1)
-DEFINE_EVENT_TYPE(fzEVT_POSTSCROLL)
+wxDECLARE_EVENT(fzEVT_POSTSCROLL, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_POSTSCROLL, wxCommandEvent);
 
 BEGIN_EVENT_TABLE(wxListCtrlEx, wxListCtrlExBase)
 EVT_COMMAND(wxID_ANY, fzEVT_POSTSCROLL, wxListCtrlEx::OnPostScrollEvent)
@@ -32,7 +32,9 @@ EVT_LIST_ITEM_SELECTED(wxID_ANY, wxListCtrlEx::OnSelectionChanged)
 EVT_KEY_DOWN(wxListCtrlEx::OnKeyDown)
 EVT_LIST_BEGIN_LABEL_EDIT(wxID_ANY, wxListCtrlEx::OnBeginLabelEdit)
 EVT_LIST_END_LABEL_EDIT(wxID_ANY, wxListCtrlEx::OnEndLabelEdit)
-#ifndef __WXMSW__
+#ifdef __WXMSW__
+EVT_SYS_COLOUR_CHANGED(wxListCtrlEx::SystemColorChange)
+#else
 EVT_LIST_COL_DRAGGING(wxID_ANY, wxListCtrlEx::OnColumnDragging)
 #endif
 END_EVENT_TABLE()
@@ -1168,3 +1170,11 @@ wxRect wxListCtrlEx::GetActualClientRect() const
 
 	return windowRect;
 }
+
+#ifdef __WXMSW__
+void wxListCtrlEx::SystemColorChange(wxSysColourChangedEvent& event)
+{
+	SetTextColour(GetDefaultAttributes().colFg);
+	event.Skip();
+}
+#endif
