@@ -1,15 +1,20 @@
 #include "filezilla.h"
 #include "listingcomparison.h"
+#include "filelistctrl.h"
 #include "filter.h"
 #include "Options.h"
 #include "state.h"
 
 CComparableListing::CComparableListing(wxWindow* pParent)
 {
-	m_pComparisonManager = 0;
 	m_pParent = pParent;
 
 	// Init backgrounds for directory comparison
+	InitColors();
+}
+
+void CComparableListing::InitColors()
+{
 	wxColour background = m_pParent->GetBackgroundColour();
 	if (background.Red() + background.Green() + background.Blue() >= 384) {
 		// Light background
@@ -23,8 +28,6 @@ CComparableListing::CComparableListing(wxWindow* pParent)
 		m_comparisonBackgrounds[1].SetBackgroundColour(wxColour(192, 192, 64));
 		m_comparisonBackgrounds[2].SetBackgroundColour(wxColour(64, 192, 64));
 	}
-
-	m_pOther = 0;
 }
 
 bool CComparableListing::IsComparing() const
@@ -97,13 +100,14 @@ bool CComparisonManager::CompareListings()
 	int64_t localSize, remoteSize;
 	fz::datetime localDate, remoteDate;
 
-	const int dirSortMode = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
+	int const dirSortMode = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
+	auto const nameSortMode = static_cast<NameSortMode>(COptions::Get()->get_int(OPTION_FILELIST_NAMESORT));
 
 	bool gotLocal = m_pLeft->get_next_file(localFile, localPath, localDir, localSize, localDate);
 	bool gotRemote = m_pRight->get_next_file(remoteFile, remotePath, remoteDir, remoteSize, remoteDate);
 
 	while (gotLocal && gotRemote) {
-		int cmp = CompareFiles(dirSortMode, localPath, localFile, remotePath, remoteFile, localDir, remoteDir);
+		int cmp = CompareFiles(dirSortMode, nameSortMode, localPath, localFile, remotePath, remoteFile, localDir, remoteDir);
 		if (!cmp) {
 			if (!m_comparisonMode) {
 				const CComparableListing::t_fileEntryFlags flag = (localDir || localSize == remoteSize) ? CComparableListing::normal : CComparableListing::different;
@@ -183,7 +187,7 @@ bool CComparisonManager::CompareListings()
 	return true;
 }
 
-int CComparisonManager::CompareFiles(const int dirSortMode, std::wstring_view const& local_path, std::wstring_view const& local, std::wstring_view const& remote_path, std::wstring_view const& remote, bool localDir, bool remoteDir)
+int CComparisonManager::CompareFiles(int const dirSortMode, NameSortMode const nameSortMode, std::wstring_view const& local_path, std::wstring_view const& local, std::wstring_view const& remote_path, std::wstring_view const& remote, bool localDir, bool remoteDir)
 {
 	switch (dirSortMode)
 	{
@@ -202,19 +206,12 @@ int CComparisonManager::CompareFiles(const int dirSortMode, std::wstring_view co
 		break;
 	}
 
-#ifdef __WXMSW__
-	auto cmp = fz::stricmp(local, remote);
+	auto const f = CFileListCtrlSortBase::GetCmpFunction(nameSortMode);
+	auto cmp = f(local, remote);
 	if (!cmp) {
-		return fz::stricmp(local_path, remote_path);
+		return f(local_path, remote_path);
 	}
 	return cmp;
-#else
-	auto cmp = local.compare(remote);
-	if (!cmp) {
-		return local_path.compare(remote_path);
-	}
-	return cmp;
-#endif
 }
 
 CComparisonManager::CComparisonManager(CState& state)
