@@ -1,11 +1,10 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "loginmanager.h"
 
 #include "dialogex.h"
 #include "filezillaapp.h"
 #include "Options.h"
 #include "textctrlex.h"
-#include "xrc_helper.h"
 
 #include <algorithm>
 
@@ -29,9 +28,9 @@ bool CLoginManager::GetPassword(Site & site, bool silent)
 	}
 
 	if (site.credentials.encrypted_) {
-		auto priv = decryptors_.find(site.credentials.encrypted_);
-		if (priv != decryptors_.end() && priv->second) {
-			return site.credentials.Unprotect(priv->second);
+		auto priv = GetDecryptor(site.credentials.encrypted_);
+		if (priv) {
+			return site.credentials.Unprotect(priv);
 		}
 
 		if (!silent) {
@@ -92,11 +91,23 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 		inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(name)));
 	}
 
-	inner->Add(new wxStaticText(&pwdDlg, -1, _("Host:")));
+	if (site.server.GetProtocol() == STORJ) {
+		inner->Add(new wxStaticText(&pwdDlg, -1, _("Satellite:")));
+	}
+	else {
+		inner->Add(new wxStaticText(&pwdDlg, -1, _("Host:")));
+	}
 	inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(site.Format(ServerFormat::with_optional_port))));
 
-	inner->Add(new wxStaticText(&pwdDlg, -1, _("User:")));
-	inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(site.server.GetUser())));
+	if (!site.server.GetUser().empty()) {
+		if (site.server.GetProtocol() == STORJ) {
+			inner->Add(new wxStaticText(&pwdDlg, -1, _("API Key:")));
+		}
+		else {
+			inner->Add(new wxStaticText(&pwdDlg, -1, _("User:")));
+		}
+		inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(site.server.GetUser())));
+	}
 
 	inner = lay.createFlex(2);
 	main->Add(inner);
@@ -111,6 +122,7 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 	password->SetFocus();
 	inner->Add(password, lay.valign);
 
+	main->AddSpacer(0);
 	auto* remember = new wxCheckBox(&pwdDlg, -1, _("&Remember master password until FileZilla is closed"));
 	remember->SetValue(true);
 	main->Add(remember);
@@ -233,6 +245,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 	}
 
 	wxTextCtrl* password{};
+	wxTextCtrl* key{}; // for storj
 	if (!site.server.GetUser().empty() || site.credentials.logonType_ != LogonType::interactive) {
 		inner->Add(new wxStaticText(&pwdDlg, -1, _("&Password:")), lay.valign);
 		password = new wxTextCtrlEx(&pwdDlg, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD);
@@ -244,7 +257,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 
 		if (site.server.GetProtocol() == STORJ) {
 			inner->Add(new wxStaticText(&pwdDlg, -1, _("Encryption &key:")), lay.valign);
-			auto * key = new wxTextCtrlEx(&pwdDlg, XRCID("ID_ENCRYPTIONKEY"));
+			key = new wxTextCtrlEx(&pwdDlg, -1);
 			key->SetMinSize(wxSize(150, -1));
 			inner->Add(key, lay.valign);
 		}
@@ -252,6 +265,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 
 	wxCheckBox* remember{};
 	if (canRemember) {
+		main->AddSpacer(0);
 		remember = new wxCheckBox(&pwdDlg, -1, _("&Remember password until FileZilla is closed"));
 		remember->SetValue(true);
 		main->Add(remember);
@@ -287,7 +301,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 
 /* FIXME?
 	if (site.server.GetProtocol() == STORJ) {
-		std::wstring encryptionKey = XRCCTRL(pwdDlg, "ID_ENCRYPTIONKEY", wxTextCtrl)->GetValue().ToStdWstring();
+		std::wstring encryptionKey = key->GetValue().ToStdWstring();
 		if (encryptionKey.empty()) {
 			wxMessageBoxEx(_("No encryption key given."), _("Invalid input"), wxICON_EXCLAMATION);
 			continue;
@@ -297,7 +311,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 		if (password) {
 			std::wstring pass = password->GetValue().ToStdWstring();
 			if (site.server.GetProtocol() == STORJ) {
-				std::wstring encryptionKey = xrc_call(pwdDlg, "ID_ENCRYPTIONKEY", &wxTextCtrl::GetValue).ToStdWstring();
+				std::wstring encryptionKey = key->GetValue().ToStdWstring();
 				pass += L"|" + encryptionKey;
 			}
 			site.credentials.SetPass(pass);
@@ -351,24 +365,57 @@ bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten,
 		return false;
 	}
 
-	if (decryptors_.find(pub) != decryptors_.cend()) {
+	bool forgotten{};
+	auto priv = GetDecryptor(pub, &forgotten);
+	if (priv || (allowForgotten && forgotten)) {
 		return true;
 	}
 
 	wxDialogEx pwdDlg;
-	if (!pwdDlg.Load(wxGetApp().GetTopWindow(), _T("ID_ENTEROLDMASTERPASSWORD"))) {
+	if (!pwdDlg.Create(wxGetApp().GetTopWindow(), -1, _("Enter master password"))) {
 		return false;
 	}
-	xrc_call(pwdDlg, "ID_KEY_IDENTIFIER", &wxStaticText::SetLabel, pub.to_base64().substr(0, 8));
+	auto& lay = pwdDlg.layout();
+	auto* main = lay.createMain(&pwdDlg, 1);
 
-	if (!allowForgotten) {
-		xrc_call(pwdDlg, "ID_FORGOT", &wxControl::Hide);
+	main->Add(new wxStaticText(&pwdDlg, -1, _("Please enter your current master password to change the password settings.")));
+
+	auto* inner = lay.createFlex(2);
+	main->Add(inner);
+
+	inner->Add(new wxStaticText(&pwdDlg, -1, _("Key identifier:")));
+	inner->Add(new wxStaticText(&pwdDlg, -1, fz::to_wstring(pub.to_base64().substr(0, 8))));
+
+	inner->Add(new wxStaticText(&pwdDlg, -1, _("Master &Password:")), lay.valign);
+
+	auto* password = new wxTextCtrlEx(&pwdDlg, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD);
+	password->SetMinSize(wxSize(150, -1));
+	password->SetFocus();
+	inner->Add(password, lay.valign);
+
+	wxCheckBox* forgot{};
+	if (allowForgotten) {
+		main->AddSpacer(0);
+		forgot = new wxCheckBox(&pwdDlg, -1, _("I &forgot my master password. Delete all passwords stored with this key."));
+		main->Add(forgot);
 	}
-	if (!allowCancel) {
-		wxASSERT(allowForgotten);
-		xrc_call(pwdDlg, "wxID_CANCEL", &wxControl::Disable);
+
+	auto* buttons = lay.createButtonSizer(&pwdDlg, main, true);
+
+	auto ok = new wxButton(&pwdDlg, wxID_OK, _("&OK"));
+	ok->SetDefault();
+	buttons->AddButton(ok);
+
+	if (allowCancel) {
+		auto cancel = new wxButton(&pwdDlg, wxID_CANCEL, _("Cancel"));
+		buttons->AddButton(cancel);
 	}
 
+	buttons->Realize();
+
+	pwdDlg.GetSizer()->Fit(&pwdDlg);
+	pwdDlg.GetSizer()->SetSizeHints(&pwdDlg);
+
 	while (true) {
 		if (pwdDlg.ShowModal() != wxID_OK) {
 			if (allowCancel) {
@@ -377,9 +424,11 @@ bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten,
 			continue;
 		}
 
-		bool const forgot = xrc_call(pwdDlg, "ID_FORGOT", &wxCheckBox::GetValue);
-		if (!forgot) {
-			auto pass = fz::to_utf8(xrc_call(pwdDlg, "ID_PASSWORD", &wxTextCtrl::GetValue).ToStdWstring());
+		if (allowForgotten && forgot->GetValue()) {
+			decryptors_[pub] = fz::private_key();
+		}
+		else {
+			auto pass = fz::to_utf8(password->GetValue().ToStdWstring());
 			auto key = fz::private_key::from_password(pass, pub.salt_);
 
 			if (key.pubkey() != pub) {
@@ -387,9 +436,7 @@ bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten,
 				continue;
 			}
 			decryptors_[pub] = key;
-		}
-		else {
-			decryptors_[pub] = fz::private_key();
+			decryptorPasswords_.push_back(pass);
 		}
 		break;
 	}
@@ -397,17 +444,37 @@ bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten,
 	return true;
 }
 
-fz::private_key CLoginManager::GetDecryptor(fz::public_key const& pub)
+fz::private_key CLoginManager::GetDecryptor(fz::public_key const& pub, bool * forgotten)
 {
 	auto it = decryptors_.find(pub);
 	if (it != decryptors_.cend()) {
+		if (!it->second && forgotten) {
+			*forgotten = true;
+		}
 		return it->second;
 	}
 
+	for (auto const& pw : decryptorPasswords_) {
+		auto priv = fz::private_key::from_password(pw, pub.salt_);
+		if (priv && priv.pubkey() == pub) {
+			decryptors_[pub] = priv;
+			return priv;
+		}
+	}
+
 	return fz::private_key();
 }
 
-void CLoginManager::Remember(const fz::private_key &key)
+void CLoginManager::Remember(const fz::private_key &key, std::string_view const& pass)
 {
 	decryptors_[key.pubkey()] = key;
+
+	if (!pass.empty()) {
+		for (auto const& pw : decryptorPasswords_) {
+			if (pw == pass) {
+				return;
+			}
+		}
+		decryptorPasswords_.emplace_back(pass);
+	}
 }
