@@ -1,9 +1,12 @@
 #ifndef FILEZILLA_INTERFACE_MENU_BAR_HEADER
 #define FILEZILLA_INTERFACE_MENU_BAR_HEADER
 
-#include <option_change_event_handler.h>
 #include "state.h"
 
+#include "option_change_event_handler.h"
+
+#include <wx/menu.h>
+
 class CMainFrame;
 class CMenuBar final : public wxMenuBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
 {
@@ -29,9 +32,7 @@ protected:
 	void UpdateSpeedLimitMenuItem();
 
 	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
-	virtual void OnOptionsChanged(changed_options_t const& options);
-
-	DECLARE_DYNAMIC_CLASS(CMenuBar)
+	virtual void OnOptionsChanged(watched_options const& options);
 
 	DECLARE_EVENT_TABLE()
 	void OnMenuEvent(wxCommandEvent& event);
