@@ -15,8 +15,8 @@
 #include "Options.h"
 #include "xrc_helper.h"
 
-DECLARE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS, -1)
-DEFINE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS)
+wxDECLARE_EVENT(fzEVT_ON_EXTERNAL_IP_ADDRESS, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_ON_EXTERNAL_IP_ADDRESS, wxCommandEvent);
 
 BEGIN_EVENT_TABLE(CNetConfWizard, wxWizard)
 EVT_WIZARD_PAGE_CHANGING(wxID_ANY, CNetConfWizard::OnPageChanging)
