@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "quickconnectbar.h"
 #include "recentserverlist.h"
 #include "commandqueue.h"
@@ -12,6 +12,7 @@
 #include "themeprovider.h"
 
 #include <wx/bmpbuttn.h>
+#include <wx/menu.h>
 #include <wx/statline.h>
 
 BEGIN_EVENT_TABLE(CQuickconnectBar, wxPanel)
@@ -25,7 +26,7 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 {
 	m_pMainFrame = pParent;
 
-	if (!wxPanel::Create(pParent, XRCID("ID_QUICKCONNECT"))) {
+	if (!wxPanel::Create(pParent, -1)) {
 		return false;
 	}
 
@@ -42,14 +43,17 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 	mainSizer->Add(new wxStaticText(this, -1, _("&Host:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 	m_pHost = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, ConvertDialogToPixels(wxSize(63, -1)), wxTE_PROCESS_ENTER);
 	m_pHost->SetToolTip(_("Enter the address of the server. To specify the server protocol, prepend the host with the protocol identifier. If no protocol is specified, the default protocol (ftp://) will be used. You can also enter complete URLs in the form protocol://user:pass@host:port here, the values in the other fields will be overwritten then.\n\nSupported protocols are:\n- ftp:// for normal FTP with optional encryption\n- sftp:// for SSH file transfer protocol\n- ftps:// for FTP over TLS (implicit)\n- ftpes:// for FTP over TLS (explicit)"));
+	m_pHost->SetMaxLength(1000);
 	mainSizer->Add(m_pHost, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Username:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 	m_pUser = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
+	m_pUser->SetMaxLength(1000);
 	mainSizer->Add(m_pUser, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("Pass&word:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 	m_pPass = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER|wxTE_PASSWORD);
+	m_pPass->SetMaxLength(1000);
 	mainSizer->Add(m_pPass, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Port:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
@@ -93,14 +97,24 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 			event.Skip();
 		}
 	});
+
+#ifdef __WXMSW__
+	Bind(wxEVT_SYS_COLOUR_CHANGED, [this](wxSysColourChangedEvent& evt) {
+		// Delay refresh until children had time to process change.
+		CallAfter([this](){Refresh();});
+		evt.Skip();
+	});
+#endif
+
+
+	GetSizer()->Fit(this);
 	return true;
 }
 
 void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState || !pState->m_pEngine) {
-		wxMessageBoxEx(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
+	if (!pState || !pState->engine_) {
 		return;
 	}
 
@@ -174,7 +188,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		return;
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+	if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
 		site.SetLogonType(LogonType::ask);
 		CLoginManager::Get().RememberPassword(site);
 	}
@@ -192,7 +206,7 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 	wxMenu* pMenu = new wxMenu;
 
 	// We have to start with id 1 since menu items with id 0 don't work under OS X
-	if (COptions::Get()->GetOptionVal(OPTION_FTP_PROXY_TYPE)) {
+	if (COptions::Get()->get_int(OPTION_FTP_PROXY_TYPE)) {
 		pMenu->Append(1, _("Connect bypassing proxy settings"));
 	}
 	pMenu->Append(2, _("Clear quickconnect bar"));
