@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "remote_recursive_operation.h"
 #include "commandqueue.h"
 #include "chmoddialog.h"
@@ -68,11 +68,10 @@ void CRemoteRecursiveOperation::AddRecursionRoot(recursion_root && root)
 	}
 }
 
-void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, ActiveFilters const& filters, CServerPath const& finalDir, bool immediate)
+void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, ActiveFilters const& filters, bool immediate)
 {
 	wxCHECK_RET(m_operationMode == recursive_none, _T("StartRecursiveOperation called with m_operationMode != recursive_none"));
 	wxCHECK_RET(m_state.IsRemoteConnected(), _T("StartRecursiveOperation while disconnected"));
-	wxCHECK_RET(!finalDir.empty(), _T("Empty final dir in recursive operation"));
 
 	if (mode == recursive_chmod && !chmodData_) {
 		return;
@@ -129,16 +128,6 @@ bool CRemoteRecursiveOperation::NextOperation()
 		recursion_roots_.pop_front();
 	}
 
-	if (m_operationMode == recursive_delete && !m_finalDir.empty()) {
-		// After a deletion we cannot refresh if inside the deleted directories. Navigate user out if it
-		auto curPath = m_state.GetRemotePath();
-		if (!curPath.empty() && (curPath == m_finalDir || m_finalDir.IsParentOf(curPath, false))) {
-			StopRecursiveOperation();
-			m_state.ChangeRemoteDir(m_finalDir, std::wstring(), LIST_FLAG_REFRESH);
-			return false;
-		}
-	}
-
 	StopRecursiveOperation();
 	m_state.RefreshRemote();
 	return false;
@@ -265,16 +254,16 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 
 			bool isLink{};
 			fz::native_string name;
-			bool isDir{};
+			fz::local_filesys::type t{};
 			int64_t size{};
 			fz::datetime time;
 			int attributes{};
-			while (fs.get_next_file(name, isLink, isDir, &size, &time, &attributes)) {
+			while (fs.get_next_file(name, isLink, t, &size, &time, &attributes)) {
 				if (isLink) {
 					continue;
 				}
 				auto const wname = fz::to_wstring(name);
-				if (filter.FilenameFiltered(m_filters.first, wname, dir.localDir.GetPath(), isDir, size, attributes, time)) {
+				if (filter.FilenameFiltered(m_filters.first, wname, dir.localDir.GetPath(), t == fz::local_filesys::dir, size, attributes, time)) {
 					continue;
 				}
 
@@ -286,7 +275,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 					if (!filter.FilenameFiltered(m_filters.second, entry.name, remotePath, entry.is_dir(), entry.size, 0, entry.time)) {
 						// Both local and remote items exist
 
-						if (isDir == entry.is_dir() || entry.is_link()) {
+						if ((t == fz::local_filesys::dir) == entry.is_dir() || entry.is_link()) {
 							// Normal item, nothing we should do
 							continue;
 						}
@@ -347,7 +336,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			case recursive_synchronize_download:
 				{
 					std::wstring localFile = CQueueView::ReplaceInvalidCharacters(entry.name);
-					if (pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
+					if (pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
 						localFile = StripVMSRevision(localFile);
 					}
 					m_pQueue->QueueFile(!m_immediate, true,
