@@ -1,17 +1,24 @@
 #ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_CONNECTION_HEADER
 #define FILEZILLA_INTERFACE_OPTIONSPAGE_CONNECTION_HEADER
 
+#include "optionspage.h"
+
+#include <memory>
+
 class COptionsPageConnection final : public COptionsPage
 {
 public:
-	virtual wxString GetResourceName() const override { return _T("ID_SETTINGS_CONNECTION"); }
+	COptionsPageConnection();
+	~COptionsPageConnection();
+
+	virtual bool CreateControls(wxWindow* parent) override;
 	virtual bool LoadPage() override;
 	virtual bool SavePage() override;
 	virtual bool Validate() override;
 
-protected:
-	DECLARE_EVENT_TABLE()
-	void OnWizard(wxCommandEvent& event);
+private:
+	struct impl;
+	std::unique_ptr<impl> impl_;
 };
 
 #endif
