@@ -6,15 +6,19 @@
 class COptionsPageUpdateCheck final : public COptionsPage
 {
 public:
-	virtual wxString GetResourceName() { return _T("ID_SETTINGS_UPDATECHECK"); }
-	virtual bool LoadPage();
-	virtual bool SavePage();
-	virtual bool Validate();
+	COptionsPageUpdateCheck();
+	virtual ~COptionsPageUpdateCheck();
+
+	virtual bool CreateControls(wxWindow* parent) override;
+	virtual bool LoadPage() override;
+	virtual bool SavePage() override;
+	virtual bool Validate() override;
 
 protected:
 	void OnRunUpdateCheck(wxCommandEvent&);
 
-	DECLARE_EVENT_TABLE()
+	struct impl;
+	std::unique_ptr<impl> impl_;
 };
 
 #endif
