@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "sitemanager.h"
 
 #include "dialogex.h"
@@ -10,6 +10,8 @@
 
 #include <libfilezilla/translate.hpp>
 
+#include <wx/menu.h>
+
 namespace {
 struct background_color {
 	wxColour const color;
@@ -571,7 +573,7 @@ std::wstring CSiteManager::AddServer(Site site)
 	AddTextElement(xServer, name);
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return std::wstring();
 		}
 
@@ -696,7 +698,7 @@ bool CSiteManager::AddBookmark(std::wstring sitePath, wxString const& name, wxSt
 	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
 		}
 
@@ -753,7 +755,7 @@ bool CSiteManager::ClearBookmarks(std::wstring sitePath)
 	}
 
 	if (!file.Save(false)) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
 		}
 
@@ -801,7 +803,7 @@ wxString CSiteManager::GetColourName(int i)
 
 void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask)
 {
-	bool const forget = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
+	bool const forget = COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 0;
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
 			Rewrite(loginManager, child, on_failure_set_to_ask);
@@ -822,7 +824,7 @@ void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element,
 
 void CSiteManager::Rewrite(CLoginManager & loginManager, bool on_failure_set_to_ask)
 {
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+	if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
 	}
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
@@ -849,16 +851,26 @@ void CSiteManager::Save(pugi::xml_node element, Site const& site)
 	SetServer(element, site);
 
 	// Save comments
-	AddTextElement(element, "Comments", site.comments_);
+	if (!site.comments_.empty()) {
+		AddTextElement(element, "Comments", site.comments_);
+	}
 
 	// Save colour
-	AddTextElement(element, "Colour", CSiteManager::GetColourIndex(site.m_colour));
+	int col = CSiteManager::GetColourIndex(site.m_colour);
+	if (col) {
+		AddTextElement(element, "Colour", col);
+	}
 
 	// Save local dir
-	AddTextElement(element, "LocalDir", site.m_default_bookmark.m_localDir);
+	if (!site.m_default_bookmark.m_localDir.empty()) {
+		AddTextElement(element, "LocalDir", site.m_default_bookmark.m_localDir);
+	}
 
 	// Save remote dir
-	AddTextElement(element, "RemoteDir", site.m_default_bookmark.m_remoteDir.GetSafePath());
+	auto const sp = site.m_default_bookmark.m_remoteDir.GetSafePath();
+	if (!sp.empty()) {
+		AddTextElement(element, "RemoteDir", sp);
+	}
 
 	AddTextElementUtf8(element, "SyncBrowsing", site.m_default_bookmark.m_sync ? "1" : "0");
 	AddTextElementUtf8(element, "DirectoryComparison", site.m_default_bookmark.m_comparison ? "1" : "0");
@@ -869,10 +881,15 @@ void CSiteManager::Save(pugi::xml_node element, Site const& site)
 		AddTextElement(node, "Name", bookmark.m_name);
 
 		// Save local dir
-		AddTextElement(node, "LocalDir", bookmark.m_localDir);
+		if (!bookmark.m_localDir.empty()) {
+			AddTextElement(node, "LocalDir", bookmark.m_localDir);
+		}
 
 		// Save remote dir
-		AddTextElement(node, "RemoteDir", bookmark.m_remoteDir.GetSafePath());
+		auto const sp = bookmark.m_remoteDir.GetSafePath();
+		if (!sp.empty()) {
+			AddTextElement(node, "RemoteDir", sp);
+		}
 
 		AddTextElementUtf8(node, "SyncBrowsing", bookmark.m_sync ? "1" : "0");
 		AddTextElementUtf8(node, "DirectoryComparison", bookmark.m_comparison ? "1" : "0");
