@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "sitemanager_dialog.h"
 
 #include "asksavepassworddialog.h"
@@ -18,12 +18,16 @@
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
 
+#include <wx/dirdlg.h>
 #include <wx/dnd.h>
-#include <wx/file.h>
+#include <wx/filedlg.h>
+#include <wx/menu.h>
 #include <wx/statline.h>
+#include <wx/wupdlock.h>
 
 #include <algorithm>
 #include <array>
+#include <list>
 
 BEGIN_EVENT_TABLE(CSiteManagerDialog, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CSiteManagerDialog::OnOK)
@@ -582,7 +586,7 @@ public:
 		}
 		m_lastSelectionIt = m_lastSelection.cbegin();
 
-		m_kiosk = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE);
+		m_kiosk = COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -753,7 +757,7 @@ bool CSiteManagerDialog::Load()
 		return true;
 	}
 
-	std::wstring lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = COptions::Get()->get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '0') {
 		if (lastSelection == _T("0")) {
 			tree_->SafeSelectItem(treeId);
@@ -815,7 +819,7 @@ bool CSiteManagerDialog::Save(pugi::xml_node element, wxTreeItemId treeId)
 		bool res = Save(element, m_ownSites);
 
 		if (!xml.Save(false)) {
-			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+			if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 				return res;
 			}
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), xml.GetFileName(), xml.GetError());
@@ -878,7 +882,7 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 
 void CSiteManagerDialog::OnNewFolder(wxCommandEvent&)
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -920,8 +924,8 @@ bool CSiteManagerDialog::Verify()
 	}
 
 	if (data->m_site) {
-		bool const predefined = IsPredefinedItem(item);
-		if (!m_pNotebook_Site->Verify(predefined)) {
+		Site newSite = *data->m_site;
+		if (!m_pNotebook_Site->UpdateSite(newSite, false)) {
 			return false;
 		}
 	}
@@ -1051,7 +1055,7 @@ void CSiteManagerDialog::OnRename(wxCommandEvent&)
 
 void CSiteManagerDialog::OnDelete(wxCommandEvent&)
 {
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -1111,7 +1115,7 @@ void CSiteManagerDialog::OnDelete(wxCommandEvent&)
 
 void CSiteManagerDialog::OnSelChanging(wxTreeEvent& event)
 {
-	if (m_is_deleting) {
+	if (m_is_deleting || tree_->ShouldIgnoreChangeEvent()) {
 		return;
 	}
 
@@ -1140,7 +1144,7 @@ void CSiteManagerDialog::OnSelChanging(wxTreeEvent& event)
 
 void CSiteManagerDialog::OnSelChanged(wxTreeEvent& evt)
 {
-	if (m_is_deleting) {
+	if (m_is_deleting || tree_->ShouldIgnoreChangeEvent()) {
 		return;
 	}
 
@@ -1150,12 +1154,13 @@ void CSiteManagerDialog::OnSelChanged(wxTreeEvent& evt)
 		m_is_deleting = false;
 	}
 
+	wxWindowUpdateLocker l(this);
 	SetCtrlState();
 }
 
 void CSiteManagerDialog::OnNewSite(wxCommandEvent&)
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -1195,8 +1200,7 @@ bool CSiteManagerDialog::UpdateItem()
 	}
 
 	if (data->m_site) {
-		UpdateServer(*data->m_site, tree_->GetItemText(item));
-		return true;
+		return UpdateServer(*data->m_site, tree_->GetItemText(item));
 	}
 	else {
 		wxASSERT(data->m_bookmark);
@@ -1222,11 +1226,16 @@ bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, Site const& site)
 	return true;
 }
 
-void CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
+bool CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
 {
-	m_pNotebook_Site->UpdateSite(site);
+	Site newSite = site;
+	newSite.SetName(name.ToStdWstring());
+	if (!m_pNotebook_Site->UpdateSite(newSite, true)) {
+		return false;
+	}
 
-	site.SetName(name.ToStdWstring());
+	site = newSite;
+	return true;
 }
 
 bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
@@ -1296,7 +1305,7 @@ void CSiteManagerDialog::OnItemActivated(wxTreeEvent&)
 
 void CSiteManagerDialog::SetCtrlState()
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 
 	wxTreeItemId item;
 	if (selections.size() == 1) {
@@ -1403,22 +1412,34 @@ void CSiteManagerDialog::OnSearch(wxCommandEvent&)
 
 	wxString search = dlg.GetValue().Lower();
 
-	m_is_deleting = true;
-	tree_->UnselectAll();
-
 	bool match{};
-	wxTreeItemId item = tree_->GetRootItem();
-	while (item) {
-		auto name = tree_->GetItemText(item).Lower();
-		if (name.find(search) != wxString::npos) {
-			tree_->SafeSelectItem(item, false);
-			match = true;
-		}
 
-		item = tree_->GetNextItemSimple(item, true);
+	{
+		wxWindowUpdateLocker l(this);
+
+		m_is_deleting = true;
+		tree_->UnselectAll();
+
+		wxTreeItemId item = tree_->GetRootItem();
+		while (item) {
+			bool select{};
+			if (item != tree_->GetRootItem() && item != m_ownSites && item != m_predefinedSites) {
+				auto name = tree_->GetItemText(item).Lower();
+				if (name.find(search) != wxString::npos) {
+					select = true;
+				}
+			}
+
+			if (select) {
+				tree_->SafeSelectItem(item, false);
+				match = true;
+			}
+
+			item = tree_->GetNextItemSimple(item, true);
+		}
+		SetCtrlState();
+		m_is_deleting = false;
 	}
-	SetCtrlState();
-	m_is_deleting = false;
 
 	if (match) {
 		tree_->SetFocus();
@@ -1449,7 +1470,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent&)
 		items.push_back(item);
 	}
 	else {
-		auto selections = tree_->GetSelections();
+		auto selections = tree_->GetAllSelections();
 
 		wxTreeItemId ancestor;
 		for (auto const& item : selections) {
@@ -1516,7 +1537,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_Expanded);
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_SelectedExpanded);
 
-	std::wstring lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = COptions::Get()->get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '1') {
 		if (lastSelection == _T("1")) {
 			tree_->SafeSelectItem(m_predefinedSites);
@@ -1560,7 +1581,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	}
 #endif
 
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
 		event.Veto();
 		return;
 	}
@@ -1578,7 +1599,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	std::vector<wxTreeItemId> items;
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 
 	bool predefined{};
 
@@ -1882,7 +1903,7 @@ void CSiteManagerDialog::RememberLastSelected()
 		path = GetSitePath(sel);
 	}
 	
-	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, path);
+	COptions::Get()->set(OPTION_SITEMANAGER_LASTSELECTED, path);
 }
 
 void CSiteManagerDialog::OnContextMenu(wxTreeEvent&)
@@ -1919,7 +1940,7 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 
 	auto servers = exportRoot.append_child("Servers");
 
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 
 	wxTreeItemId ancestor;
 	for (auto const& item : selections) {
