@@ -1,7 +1,6 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "state.h"
 #include "commandqueue.h"
-#include "FileZillaEngine.h"
 #include "Options.h"
 #include "Mainfrm.h"
 #include "queue.h"
@@ -11,8 +10,11 @@
 #include "listingcomparison.h"
 #include "xrc_helper.h"
 
+#include "../include/FileZillaEngine.h"
+
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/glue/wx.hpp>
+#include <libfilezilla/glue/wxinvoker.hpp>
 
 #include <algorithm>
 
@@ -234,7 +236,7 @@ CState::~CState()
 {
 	delete m_pComparisonManager;
 	delete m_pCommandQueue;
-	delete m_pEngine;
+	engine_.reset();
 	delete m_pLocalRecursiveOperation;
 	delete m_pRemoteRecursiveOperation;
 
@@ -548,10 +550,10 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 	if (!site) {
 		return false;
 	}
-	if (!m_pEngine) {
+	if (!engine_) {
 		return false;
 	}
-	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle()) {
+	if (engine_->IsConnected() || engine_->IsBusy() || !m_pCommandQueue->Idle()) {
 		m_pCommandQueue->Cancel();
 	}
 	m_pRemoteRecursiveOperation->StopRecursiveOperation();
@@ -569,7 +571,7 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 
 bool CState::Disconnect()
 {
-	if (!m_pEngine) {
+	if (!engine_) {
 		return false;
 	}
 
@@ -589,14 +591,14 @@ bool CState::Disconnect()
 
 bool CState::CreateEngine()
 {
-	wxASSERT(!m_pEngine);
-	if (m_pEngine) {
+	wxASSERT(!engine_);
+	if (engine_) {
 		return true;
 	}
 
-	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext(), m_mainFrame);
+	engine_ = std::make_unique<CFileZillaEngine>(m_mainFrame.GetEngineContext(), fz::make_invoker(m_mainFrame, [frame = &m_mainFrame](CFileZillaEngine* engine){ frame->OnEngineEvent(engine); }));
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, *this);
+	m_pCommandQueue = new CCommandQueue(engine_.get(), &m_mainFrame, *this);
 
 	return true;
 }
@@ -605,8 +607,7 @@ void CState::DestroyEngine()
 {
 	delete m_pCommandQueue;
 	m_pCommandQueue = 0;
-	delete m_pEngine;
-	m_pEngine = 0;
+	engine_.reset();
 }
 
 void CState::RegisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification, CStateEventHandler* insertBefore)
@@ -847,7 +848,7 @@ void DoHandleDroppedFiles(CState & state, CMainFrame & mainFrame, T const& files
 	memcpy(to, static_cast<wchar_t const*>(path.GetPath().c_str()), (path.GetPath().size() + 1) * sizeof(wchar_t));
 	to[path.GetPath().size() + 1] = 0; // End of list
 
-	SHFILEOPSTRUCT op = { 0 };
+	SHFILEOPSTRUCT op{};
 	op.pFrom = from;
 	op.pTo = to;
 	op.wFunc = copy ? FO_COPY : FO_MOVE;
@@ -963,15 +964,16 @@ bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 			continue;
 		}
 
-		bool is_dir, is_link;
+		bool is_link{};
+		fz::local_filesys::type t{};
 		fz::native_string file;
-		while (fs.get_next_file(file, is_link, is_dir, 0, 0, 0)) {
+		while (fs.get_next_file(file, is_link, t, 0, 0, 0)) {
 			if (file.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (is_dir) {
+			if (t == fz::local_filesys::dir) {
 				if (is_link) {
 					continue;
 				}
@@ -1031,7 +1033,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 		m_pRemoteRecursiveOperation->AddRecursionRoot(std::move(root));
 
 		CFilterManager filter;
-		m_pRemoteRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), pRemoteDataObject->GetServerPath(), !queueOnly);
+		m_pRemoteRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queueOnly);
 	}
 
 	return true;
@@ -1039,7 +1041,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 
 bool CState::IsRemoteConnected() const
 {
-	if (!m_pEngine) {
+	if (!engine_) {
 		return false;
 	}
 
