@@ -1,9 +1,13 @@
 #ifndef FILEZILLA_INTERFACE_SYSTEMIMAGELIST_HEADER
 #define FILEZILLA_INTERFACE_SYSTEMIMAGELIST_HEADER
 
+#include "filezilla.h"
+
 #ifdef __WXMSW__
 #include <shellapi.h>
 #include <commctrl.h>
+#else
+#include <map>
 #endif
 
 enum class iconType
