@@ -1,15 +1,18 @@
 #ifndef FILEZILLA_INTERFACE_TOOLBAR_HEADER
 #define FILEZILLA_INTERFACE_TOOLBAR_HEADER
 
-#include <option_change_event_handler.h>
 #include "state.h"
 
+#include "option_change_event_handler.h"
+
+#include <wx/toolbar.h>
+
 class CMainFrame;
 
 class CToolBar final : public wxToolBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
 {
 public:
-	CToolBar() = default;
+	CToolBar();
 	virtual ~CToolBar();
 
 	void UpdateToolbarState();
@@ -28,7 +31,7 @@ protected:
 	void MakeTools();
 
 	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
-	virtual void OnOptionsChanged(changed_options_t const& options);
+	virtual void OnOptionsChanged(watched_options const& options);
 
 	CMainFrame* m_pMainFrame{};
 
