@@ -4,9 +4,9 @@
 #if FZ_MANUALUPDATECHECK
 
 #include "../include/notification.h"
-
 #include <wx/timer.h>
 
+#include <libfilezilla/buffer.hpp>
 #include <libfilezilla/uri.hpp>
 
 #include <functional>
@@ -65,8 +65,9 @@ public:
 	virtual void UpdaterStateChanged(UpdaterState s, build const& v) = 0;
 };
 
+class memory_writer_factory;
 class CFileZillaEngineContext;
-class CUpdater final : public wxEvtHandler, private EngineNotificationHandler
+class CUpdater final : public wxEvtHandler
 {
 public:
 	CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context, std::function<void(CActiveNotification const&)> const& activityNotificationHandler);
@@ -113,7 +114,7 @@ protected:
 	fz::uri GetUrl();
 	void ProcessNotification(std::unique_ptr<CNotification> && notification);
 	void ProcessOperation(COperationNotification const& operation);
-	void ProcessData(CDataNotification& dataNotification);
+	bool FilterOutput();
 	void ParseData();
 	UpdaterState ProcessFinishedDownload();
 	UpdaterState ProcessFinishedData(bool can_download);
@@ -126,14 +127,14 @@ protected:
 
 	void SetState(UpdaterState s);
 
-	virtual void OnEngineEvent(CFileZillaEngine* engine);
-	void DoOnEngineEvent(CFileZillaEngine* engine);
+	void OnEngineEvent(CFileZillaEngine* engine);
 
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& ev);
 
 	UpdaterState state_;
 	std::wstring local_file_;
+	fz::buffer output_buffer_;
 
 	CFileZillaEngineContext& engine_context_;
 	CFileZillaEngine* engine_{};
