@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "viewheader.h"
 #include "commandqueue.h"
 #include "graphics.h"
@@ -68,6 +68,7 @@ CViewHeader::CViewHeader(wxWindow* pParent, const wxString& label)
 	Create(pParent, wxID_ANY);
 
 	m_pComboBox = new CComboBoxEx(this);
+	m_pComboBox->SetMaxLength(20000);
 	m_pLabel = new wxStaticText(this, wxID_ANY, label, wxDefaultPosition, wxDefaultSize);
 	wxSize size = GetSize();
 	size.SetHeight(m_pComboBox->GetBestSize().GetHeight() + border_offset);
@@ -324,8 +325,8 @@ void CViewHeader::SetFocus()
 }
 
 #ifdef __WXGTK__
-DECLARE_EVENT_TYPE(fzEVT_LOCALVIEWHEADERSETTEXTSEL, -1)
-DEFINE_EVENT_TYPE(fzEVT_LOCALVIEWHEADERSETTEXTSEL)
+wxDECLARE_EVENT(fzEVT_LOCALVIEWHEADERSETTEXTSEL, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_LOCALVIEWHEADERSETTEXTSEL, wxCommandEvent);
 #endif
 
 BEGIN_EVENT_TABLE(CLocalViewHeader, CViewHeader)
