@@ -1,12 +1,12 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "volume_enumerator.h"
 
 #ifdef FZ_WINDOWS
 
 #include <wx/msw/registry.h>
 
-DEFINE_EVENT_TYPE(fzEVT_VOLUMEENUMERATED)
-DEFINE_EVENT_TYPE(fzEVT_VOLUMESENUMERATED)
+wxDEFINE_EVENT(fzEVT_VOLUMEENUMERATED, wxCommandEvent);
+wxDEFINE_EVENT(fzEVT_VOLUMESENUMERATED, wxCommandEvent);
 
 CVolumeDescriptionEnumeratorThread::CVolumeDescriptionEnumeratorThread(wxEvtHandler* pEvtHandler, fz::thread_pool & pool)
 	: m_pEvtHandler(pEvtHandler)
