@@ -1,9 +1,10 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "welcome_dialog.h"
 #include "buildinfo.h"
 #include "Options.h"
 #include "themeprovider.h"
 #include "xrc_helper.h"
+#include "../include/version.h"
 #include <wx/hyperlink.h>
 #include <wx/statbmp.h>
 #include <wx/statline.h>
@@ -22,14 +23,14 @@ void CWelcomeDialog::RunDelayed(wxWindow* parent)
 
 bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 {
-	const wxString ownVersion = CBuildInfo::GetVersion();
-	wxString greetingVersion = COptions::Get()->GetOption(OPTION_GREETINGVERSION);
+	const wxString ownVersion = GetFileZillaVersion();
+	wxString greetingVersion = COptions::Get()->get_string(OPTION_GREETINGVERSION);
 
-	wxString const resources = COptions::Get()->GetOption(OPTION_GREETINGRESOURCES);
-	COptions::Get()->SetOption(OPTION_GREETINGRESOURCES, _T(""));
+	wxString const resources = COptions::Get()->get_string(OPTION_GREETINGRESOURCES);
+	COptions::Get()->set(OPTION_GREETINGRESOURCES, _T(""));
 
 	if (!force) {
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
 		}
 
@@ -39,10 +40,10 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 			// Been there done that
 			return true;
 		}
-		COptions::Get()->SetOption(OPTION_GREETINGVERSION, ownVersion.ToStdWstring());
+		COptions::Get()->set(OPTION_GREETINGVERSION, ownVersion.ToStdWstring());
 
-		if (greetingVersion.empty() && !COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE)) {
-			COptions::Get()->SetOption(OPTION_PROMPTPASSWORDSAVE, 1);
+		if (greetingVersion.empty() && !COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE)) {
+			COptions::Get()->set(OPTION_PROMPTPASSWORDSAVE, 1);
 		}
 	}
 
@@ -63,7 +64,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 	auto headerLeft = lay.createFlex(1);
 	header->Add(headerLeft, lay.valign)->SetProportion(1);
 
-	auto heading = new wxStaticText(this, -1, _T("FileZilla ") + CBuildInfo::GetVersion());
+	auto heading = new wxStaticText(this, -1, _T("FileZilla ") + GetFileZillaVersion());
 	heading->SetFont(heading->GetFont().Bold());
 	headerLeft->Add(heading);
 	headerLeft->Add(new wxStaticText(this, -1, _("The free open source FTP solution")));
@@ -74,7 +75,6 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 
 	main->Add(new wxStaticLine(this), lay.grow);
 
-	int const leftIndent = lay.dlgUnits(10);
 	wxString const url = _T("https://welcome.filezilla-project.org/welcome?type=client&category=%s&version=") + ownVersion;
 
 	main->Add(new wxPanel(this, XRCID("ID_HEADERMESSAGE_PANEL")), lay.halign)->Show(false);
@@ -83,7 +83,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 		auto news = new wxStaticText(this, -1, _("What's new"));
 		news->SetFont(news->GetFont().Bold());
 		main->Add(news);
-		main->Add(new wxHyperlinkCtrl(this, -1, wxString::Format(_("New features and improvements in %s"), CBuildInfo::GetVersion()), wxString::Format(url, _T("news")) + _T("&oldversion=") + greetingVersion), 0, wxLEFT, leftIndent);
+		main->Add(new wxHyperlinkCtrl(this, -1, wxString::Format(_("New features and improvements in %s"), GetFileZillaVersion()), wxString::Format(url, _T("news")) + _T("&oldversion=") + greetingVersion), 0, wxLEFT, lay.indent);
 	}
 
 	main->AddSpacer(0);
@@ -92,8 +92,8 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 	helpHeading->SetFont(helpHeading->GetFont().Bold());
 	main->Add(helpHeading);
 
-	main->Add(new wxHyperlinkCtrl(this, -1, _("Asking questions in the FileZilla Forums"), wxString::Format(url, _T("support_forum"))), 0, wxLEFT, leftIndent);
-	main->Add(new wxHyperlinkCtrl(this, -1, _("Reporting bugs and feature requests"), wxString::Format(url, _T("support_more"))), 0, wxLEFT, leftIndent);
+	main->Add(new wxHyperlinkCtrl(this, -1, _("Asking questions in the FileZilla Forums"), wxString::Format(url, _T("support_forum"))), 0, wxLEFT, lay.indent);
+	main->Add(new wxHyperlinkCtrl(this, -1, _("Reporting bugs and feature requests"), wxString::Format(url, _T("support_more"))), 0, wxLEFT, lay.indent);
 
 	main->AddSpacer(0);
 
@@ -101,9 +101,9 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 	documentationHeading->SetFont(helpHeading->GetFont().Bold());
 	main->Add(documentationHeading);
 
-	main->Add(new wxHyperlinkCtrl(this, -1, _("Basic usage instructions"), wxString::Format(url, _T("documentation_basic"))), 0, wxLEFT, leftIndent);
-	main->Add(new wxHyperlinkCtrl(this, -1, _("Configuring FileZilla and your network"), wxString::Format(url, _T("documentation_network"))), 0, wxLEFT, leftIndent);
-	main->Add(new wxHyperlinkCtrl(this, -1, _("Further documentation"), wxString::Format(url, _T("documentation_more"))), 0, wxLEFT, leftIndent);
+	main->Add(new wxHyperlinkCtrl(this, -1, _("Basic usage instructions"), wxString::Format(url, _T("documentation_basic"))), 0, wxLEFT, lay.indent);
+	main->Add(new wxHyperlinkCtrl(this, -1, _("Configuring FileZilla and your network"), wxString::Format(url, _T("documentation_network"))), 0, wxLEFT, lay.indent);
+	main->Add(new wxHyperlinkCtrl(this, -1, _("Further documentation"), wxString::Format(url, _T("documentation_more"))), 0, wxLEFT, lay.indent);
 
 	main->Add(new wxStaticText(this, -1, _("You can always open this dialog again through the help menu.")));
 
@@ -163,7 +163,7 @@ void CreateMessagePanel(wxWindow& dlg, char const* ctrl, wxXmlResource& resource
 void CWelcomeDialog::InitFooter(wxString const& resources)
 {
 #if FZ_WINDOWS && FZ_MANUALUPDATECHECK
-	if (CBuildInfo::GetBuildType() == _T("official") && !COptions::Get()->GetOptionVal(OPTION_DISABLE_UPDATE_FOOTER)) {
+	if (CBuildInfo::GetBuildType() == _T("official") && !COptions::Get()->get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
 		if (!resources.empty()) {
 			wxLogNull null;
 
