@@ -1,8 +1,11 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "buildinfo.h"
 #include "xmlfunctions.h"
-#include "xmlutils.h"
 #include "Options.h"
+
+#include "../include/version.h"
+#include "../include/xmlutils.h"
+
 #include <wx/ffile.h>
 #include <wx/log.h>
 
@@ -99,12 +102,14 @@ bool CXmlFile::Modified()
 {
 	wxCHECK(!m_fileName.empty(), false);
 
-	if (m_modificationTime.empty())
+	if (m_modificationTime.empty()) {
 		return true;
+	}
 
 	fz::datetime const modificationTime = fz::local_filesys::get_modification_time(fz::to_native(m_fileName));
-	if (!modificationTime.empty() && modificationTime == m_modificationTime)
+	if (!modificationTime.empty() && modificationTime == m_modificationTime) {
 		return false;
+	}
 
 	return true;
 }
@@ -121,7 +126,7 @@ void CXmlFile::UpdateMetadata()
 		return;
 	}
 
-	SetTextAttribute(m_element, "version", CBuildInfo::GetVersion());
+	SetTextAttribute(m_element, "version", GetFileZillaVersion());
 
 	std::string const platform =
 #ifdef FZ_WINDOWS
@@ -134,14 +139,16 @@ void CXmlFile::UpdateMetadata()
 	SetTextAttributeUtf8(m_element, "platform", platform);
 }
 
-bool CXmlFile::Save(bool printError)
+bool CXmlFile::Save(bool printError, bool updateMetadata)
 {
 	m_error.clear();
 
 	wxCHECK(!m_fileName.empty(), false);
 	wxCHECK(m_document, false);
 
-	UpdateMetadata();
+	if (updateMetadata) {
+		UpdateMetadata();
+	}
 
 	bool res = SaveXmlFile();
 	m_modificationTime = fz::local_filesys::get_modification_time(fz::to_native(m_fileName));
@@ -344,7 +351,7 @@ bool GetServer(pugi::xml_node node, Site & site)
 				std::wstring encoding = GetTextAttribute(passElement, "encoding");
 
 				if (encoding == _T("base64")) {
-					std::string decoded = fz::base64_decode(passElement.child_value());
+					std::string decoded = fz::base64_decode_s(passElement.child_value());
 					pass = fz::to_wstring_from_utf8(decoded);
 				}
 				else if (encoding == _T("crypt")) {
@@ -383,11 +390,11 @@ bool GetServer(pugi::xml_node node, Site & site)
 		return false;
 	}
 
-	wxString pasvMode = GetTextElement(node, "PasvMode");
-	if (pasvMode == _T("MODE_PASSIVE")) {
+	std::string_view pasvMode = node.child_value("PasvMode");
+	if (pasvMode == "MODE_PASSIVE") {
 		site.server.SetPasvMode(MODE_PASSIVE);
 	}
-	else if (pasvMode == _T("MODE_ACTIVE")) {
+	else if (pasvMode == "MODE_ACTIVE") {
 		site.server.SetPasvMode(MODE_ACTIVE);
 	}
 	else {
@@ -397,14 +404,11 @@ bool GetServer(pugi::xml_node node, Site & site)
 	int maximumMultipleConnections = GetTextElementInt(node, "MaximumMultipleConnections");
 	site.server.MaximumMultipleConnections(maximumMultipleConnections);
 
-	wxString encodingType = GetTextElement(node, "EncodingType");
-	if (encodingType == _T("Auto")) {
-		site.server.SetEncodingType(ENCODING_AUTO);
-	}
-	else if (encodingType == _T("UTF-8")) {
+	std::string_view encodingType = node.child_value("EncodingType");
+	if (encodingType == "UTF-8") {
 		site.server.SetEncodingType(ENCODING_UTF8);
 	}
-	else if (encodingType == _T("Custom")) {
+	else if (encodingType == "Custom") {
 		std::wstring customEncoding = GetTextElement(node, "CustomEncoding");
 		if (customEncoding.empty()) {
 			return false;
@@ -462,7 +466,9 @@ void SetServer(pugi::xml_node node, Site const& site)
 	AddTextElement(node, "Host", site.server.GetHost());
 	AddTextElement(node, "Port", site.server.GetPort());
 	AddTextElement(node, "Protocol", protocol);
-	AddTextElement(node, "Type", site.server.GetType());
+	if (site.server.HasFeature(ProtocolFeature::ServerType)) {
+		AddTextElement(node, "Type", site.server.GetType());
+	}
 
 	ProtectedCredentials credentials = site.credentials;
 
@@ -478,7 +484,7 @@ void SetServer(pugi::xml_node node, Site const& site)
 				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", pass);
 				if (passElement) {
 					SetTextAttribute(passElement, "encoding", _T("crypt"));
-					SetTextAttributeUtf8(passElement, "pubkey", credentials.encrypted_.to_base64());
+					SetTextAttributeUtf8(passElement, "pubkey", credentials.encrypted_.to_base64(false));
 				}
 			}
 			else {
@@ -498,33 +504,42 @@ void SetServer(pugi::xml_node node, Site const& site)
 	}
 	AddTextElement(node, "Logontype", static_cast<int>(credentials.logonType_));
 
-	AddTextElement(node, "TimezoneOffset", site.server.GetTimezoneOffset());
-	switch (site.server.GetPasvMode())
-	{
-	case MODE_PASSIVE:
-		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
-		break;
-	case MODE_ACTIVE:
-		AddTextElementUtf8(node, "PasvMode", "MODE_ACTIVE");
-		break;
-	default:
-		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
-		break;
+	if (site.server.GetTimezoneOffset()) {
+		AddTextElement(node, "TimezoneOffset", site.server.GetTimezoneOffset());
 	}
-	AddTextElement(node, "MaximumMultipleConnections", site.server.MaximumMultipleConnections());
 
-	switch (site.server.GetEncodingType())
-	{
-	case ENCODING_AUTO:
-		AddTextElementUtf8(node, "EncodingType", "Auto");
-		break;
-	case ENCODING_UTF8:
-		AddTextElementUtf8(node, "EncodingType", "UTF-8");
-		break;
-	case ENCODING_CUSTOM:
-		AddTextElementUtf8(node, "EncodingType", "Custom");
-		AddTextElement(node, "CustomEncoding", site.server.GetCustomEncoding());
-		break;
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::TransferMode)) {
+		switch (site.server.GetPasvMode())
+		{
+		case MODE_PASSIVE:
+			AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
+			break;
+		case MODE_ACTIVE:
+			AddTextElementUtf8(node, "PasvMode", "MODE_ACTIVE");
+			break;
+		default:
+			AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
+			break;
+		}
+	}
+	if (site.server.MaximumMultipleConnections()) {
+		AddTextElement(node, "MaximumMultipleConnections", site.server.MaximumMultipleConnections());
+	}
+
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::Charset)) {
+		switch (site.server.GetEncodingType())
+		{
+		case ENCODING_AUTO:
+			AddTextElementUtf8(node, "EncodingType", "Auto");
+			break;
+		case ENCODING_UTF8:
+			AddTextElementUtf8(node, "EncodingType", "UTF-8");
+			break;
+		case ENCODING_CUSTOM:
+			AddTextElementUtf8(node, "EncodingType", "Custom");
+			AddTextElement(node, "CustomEncoding", site.server.GetCustomEncoding());
+			break;
+		}
 	}
 
 	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
@@ -590,10 +605,10 @@ void CXmlFile::GetRawDataHere(char* p, size_t size) // p has to big enough to ho
 	m_document.save(writer);
 }
 
-bool CXmlFile::ParseData(char* data)
+bool CXmlFile::ParseData(uint8_t const* data, size_t len)
 {
 	Close();
-	m_document.load_string(data);
+	m_document.load_buffer(data, len);
 	m_element = m_document.child(m_rootName.c_str());
 	if (!m_element) {
 		Close();
@@ -607,5 +622,5 @@ bool CXmlFile::IsFromFutureVersion() const
 		return false;
 	}
 	std::wstring const version = GetTextAttribute(m_element, "version");
-	return CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str()) < CBuildInfo::ConvertToVersionNumber(version.c_str());
+	return CBuildInfo::ConvertToVersionNumber(GetFileZillaVersion().c_str()) < CBuildInfo::ConvertToVersionNumber(version.c_str());
 }
