@@ -5,7 +5,6 @@
 #include <stdio.h>
 #include <assert.h>
 
-#define DEFINE_PLUG_METHOD_MACROS
 #include "tree234.h"
 #include "putty.h"
 #include "network.h"
@@ -16,16 +15,11 @@
 
 #include "winsecur.h"
 
-Socket make_handle_socket(HANDLE send_H, HANDLE recv_H, HANDLE stderr_H,
-                          Plug plug, int overlapped);
-
-Socket new_named_pipe_client(const char *pipename, Plug plug)
+Socket *new_named_pipe_client(const char *pipename, Plug *plug)
 {
     HANDLE pipehandle;
     PSID usersid, pipeowner;
     PSECURITY_DESCRIPTOR psd;
-    char *err;
-    Socket ret;
 
     assert(strncmp(pipename, "\\\\.\\pipe\\", 9) == 0);
     assert(strchr(pipename + 9, '\\') == NULL);
@@ -39,11 +33,9 @@ Socket new_named_pipe_client(const char *pipename, Plug plug)
             break;
 
         if (GetLastError() != ERROR_PIPE_BUSY) {
-            err = dupprintf("Unable to open named pipe '%s': %s",
-                            pipename, win_strerror(GetLastError()));
-            ret = new_error_socket(err, plug);
-            sfree(err);
-            return ret;
+            return new_error_socket_fmt(
+                plug, "Unable to open named pipe '%s': %s",
+                pipename, win_strerror(GetLastError()));
         }
 
         /*
@@ -54,46 +46,38 @@ Socket new_named_pipe_client(const char *pipename, Plug plug)
          * take excessively long.)
          */
         if (!WaitNamedPipe(pipename, NMPWAIT_USE_DEFAULT_WAIT)) {
-            err = dupprintf("Error waiting for named pipe '%s': %s",
-                            pipename, win_strerror(GetLastError()));
-            ret = new_error_socket(err, plug);
-            sfree(err);
-            return ret;
+            return new_error_socket_fmt(
+                plug, "Error waiting for named pipe '%s': %s",
+                pipename, win_strerror(GetLastError()));
         }
     }
 
     if ((usersid = get_user_sid()) == NULL) {
         CloseHandle(pipehandle);
-        err = dupprintf("Unable to get user SID");
-        ret = new_error_socket(err, plug);
-        sfree(err);
-        return ret;
+        return new_error_socket_fmt(
+            plug, "Unable to get user SID: %s", win_strerror(GetLastError()));
     }
 
     if (p_GetSecurityInfo(pipehandle, SE_KERNEL_OBJECT,
                           OWNER_SECURITY_INFORMATION,
                           &pipeowner, NULL, NULL, NULL,
                           &psd) != ERROR_SUCCESS) {
-        err = dupprintf("Unable to get named pipe security information: %s",
-                        win_strerror(GetLastError()));
-        ret = new_error_socket(err, plug);
-        sfree(err);
         CloseHandle(pipehandle);
-        return ret;
+        return new_error_socket_fmt(
+            plug, "Unable to get named pipe security information: %s",
+            win_strerror(GetLastError()));
     }
 
     if (!EqualSid(pipeowner, usersid)) {
-        err = dupprintf("Owner of named pipe '%s' is not us", pipename);
-        ret = new_error_socket(err, plug);
-        sfree(err);
         CloseHandle(pipehandle);
         LocalFree(psd);
-        return ret;
+        return new_error_socket_fmt(
+            plug, "Owner of named pipe '%s' is not us", pipename);
     }
 
     LocalFree(psd);
 
-    return make_handle_socket(pipehandle, pipehandle, NULL, plug, TRUE);
+    return make_handle_socket(pipehandle, pipehandle, NULL, plug, true);
 }
 
 #endif /* !defined NO_SECURITY */
